<?php
class zipfile {

	var $datasec = array();
	var $ctrldir = array();
	var $offset  = 0;

	function zipfile($archive, $base, $files=array()) {
		$this->addList($base, $files);

		$data    = implode('', $this->datasec);
		$ctrldir = implode('', $this->ctrldir);

		$data .= "{$ctrldir}\x50\x4b\x05\x06\x00\x00\x00\x00".
			pack('v', sizeof($this->ctrldir)). // total # of entries "on this disk"
			pack('v', sizeof($this->ctrldir)). // total # of entries overall
			pack('V', strlen($ctrldir)).       // size of central dir
			pack('V', strlen($data)).          // offset to start of central dir
			"\x00\x00";                        // .zip file comment length

	    $fd = @fopen($base.$archive, 'w');
	    if(!$fd) return;
	    @fwrite($fd, $data);
	    @fclose($fd);
	}

	// Returns a 4 byte DOS date and time format
	function DosTime() {
		$t = getdate();

		if($t['year'] < 1980) {
			$t['year']    = 1980;
			$t['mon']     = 1;
			$t['mday']    = 1;
			$t['hours']   = 0;
			$t['minutes'] = 0;
			$t['seconds'] = 0;
		}

		return (($t['year']-1980)<<25)|($t['mon']<<21)|($t['mday']<<16)|($t['hours']<<11)|($t['minutes']<<5)|($t['seconds']>>1);
    }

	// adds an array of files
	function addList($base, $files) {
		foreach($files as $file) {
			if(is_file($base.$file)) $this->addFile(file_get_contents($base.$file), $file);
			elseif(is_dir($base.$file)) {
				$this->addDir($file);
				$list = scandir($base.$file);
				unset($list[0], $list[1]); // . and ..
				if(substr($file, -1) != '/') $file .= '/';
				$count = count($list);
				for($i = 2; $i < $count; $i++) $list[$i] = $file.$list[$i];
				$this->addList($base, $list);
			}
		}
	}

	// compress an individual file
	function addFile($data, $name) {
		// fix the filename
		$name = str_replace('\\', '/', $name);

		// get the current date/time
		$t = dechex($this->DosTime());
		$hexdtime = '\x'.$t[6].$t[7].'\x'.$t[4].$t[5].'\x'.$t[2].$t[3].'\x'.$t[0].$t[1];
		eval('$hexdtime = "'.$hexdtime.'";');

		// "local file header" segment
		$ulen  = strlen($data);
		$crc   = crc32($data);
		$zdata = gzcompress($data);
		$zdata = substr($zdata, 2, strlen($zdata) - 6); // fix crc bug
		$clen  = strlen($zdata);

		$fr  = "\x50\x4b\x03\x04\x14\x00\x00\x00\x08\x00{$hexdtime}";
		$fr .= pack('V', $crc);          // crc32
		$fr .= pack('V', $clen);         // compressed filesize
		$fr .= pack('V', $ulen);         // uncompressed filesize
		$fr .= pack('v', strlen($name)); // length of filename
		$fr .= pack('v', 0);             // extra field length
		$fr .= $name;

		// "file data" segment
		$fr .= $zdata;

		// "data descriptor" segment
		$fr .= pack('V', $crc);  // crc32
		$fr .= pack('V', $clen); // compressed filesize
		$fr .= pack('V', $ulen); // uncompressed filesize

		// add the compressed file data
		$this->datasec[] = $fr;

		// now add to central directory record
		$cdrec  = "\x50\x4b\x01\x02\x00\x00\x14\x00\x00\x00\x08\x00{$hexdtime}";
		$cdrec .= pack('V', $crc);          // crc32
		$cdrec .= pack('V', $clen);         // compressed filesize
		$cdrec .= pack('V', $ulen);         // uncompressed filesize
		$cdrec .= pack('v', strlen($name)); // length of filename
		$cdrec .= pack('v', 0);             // extra field length
		$cdrec .= pack('v', 0);             // file comment length
		$cdrec .= pack('v', 0);             // disk number start
		$cdrec .= pack('v', 0);             // internal file attributes
		$cdrec .= pack('V', 32);            // external file attributes - 'archive' bit set
		$cdrec .= pack('V', $this->offset); // relative offset of local header
		$cdrec .= $name;

		$this->offset += strlen($fr);

        // add the central directory record
        $this->ctrldir[] = $cdrec;
    }

	// compress a directory name into the archive
	function addDir($name) {
		$name = str_replace('\\', '/', $name);

		// "local file header" segment
		$fr  = "\x50\x4b\x03\x04\x0a\x00\x00\x00\x00\x00\x00\x00\x00\x00";
		$fr .= pack('V', 0);              // crc32
		$fr .= pack('V', 0);              // compressed filesize
		$fr .= pack('V', 0);              // uncompressed filesize
		$fr .= pack('v', strlen($name) ); // length of pathname
		$fr .= pack('v', 0);              // extra field length
		$fr .= $name;

		// "data descriptor" segment
		$fr .= pack('V', 0); // crc32
		$fr .= pack('V', 0); // compressed filesize
		$fr .= pack('V', 0); // uncompressed filesize

		// add this entry to array
		$this->datasec[] = $fr;

		$new_offset = strlen(implode('', $this->datasec));

		// ext. file attributes mirrors MS-DOS directory attr byte, detailed
		// at http://support.microsoft.com/support/kb/articles/Q125/0/19.asp

		// now add to central record
		$cdrec  = "\x50\x4b\x01\x02\x00\x00\x0a\x00\x00\x00\x00\x00\x00\x00\x00\x00";
		$cdrec .= pack('V', 0);             // crc32
		$cdrec .= pack('V', 0);             // compressed filesize
		$cdrec .= pack('V', 0);             // uncompressed filesize
		$cdrec .= pack('v', strlen($name)); // length of filename
		$cdrec .= pack('v', 0);             // extra field length
		$cdrec .= pack('v', 0);             // file comment length
		$cdrec .= pack('v', 0);             // disk number start
		$cdrec .= pack('v', 0);             // internal file attributes
		$ext    = "\x00\x00\x10\x00\xff\xff\xff\xff";
		$cdrec .= pack('V', 16);            // external file attributes, 'directory' bit set
		$cdrec .= pack('V', $this->offset); // relative offset of local header
		$cdrec .= $name;

		$this->offset = $new_offset;

		// add the central directory record
		$this->ctrldir[] = $cdrec;
	}
} // end zipfile
?>