<?php
/**
 * The class extends the Exception class 
 *
 * @package jrCMS: Exceptions
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: cException.php 603 2011-09-16 07:03:04Z jensraabe $
 *  
 * @copyright Copyright (c) 2008..2011, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */
 
interface IException{
	
	// Own methods
	public function log($sql=False);						// logging function
	public function setLogPath($subpath=Null);	// use alternate subpath
	public function ctrlLogSize();							// control/clean log file size
	
	// Protected methods inherited from Exception class 
	public function getMessage();								// Exception message 
	public function getCode();									// User-defined Exception code
	public function getFile();									// Source filename
	public function getLine();									// Source line
	public function getTrace();									// An array of the backtrace()
	public function getTraceAsString();					// Formated string of trace
	
	// Overrideable methods inherited from Exception class 
	public function __toString();				// formated string for display
	public function __construct($message = null, $code = 0);
}

class MyException extends Exception implements IException{
	
	CONST VERSION			= '$Revision: 603 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2011-09-16 09:03:04 +0200 (Fr, 16 Sep 2011) $';

	protected $message = 'Unknown exception';		// Exception message
	private   $string;								// Unknown
	protected $code = 0;							// User-defined exception code
	protected $file;								// Source filename of exception
	protected $line;								// Source line of exception
	private   $trace;								// Unknown

	public function __construct($message = null, $code = 0){
		
		parent::__construct($message, $code);

		//set default log file
		//we may use DOCUMENT_ROOT
		//$this->logfile = 'content/log/jrcms.log';
		$this->logfile = $_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/content/log/jrcms.log';
		
		//set error handler
		set_error_handler(array (&$this, 'own_error_handler'),E_ALL);
	}
	
	public function __toString(){
		return get_class($this) . " '{$this->message}' in {$this->file}({$this->line})\n"	
														. "Code: {$this->getCode()} \n"
														. "{$this->getTraceAsString()}\n<br />";
	}

	public function log($sql=False) {
		
		$db = new DBInterface();
		$msg = $this->getMessage();
		if($sql) {
			//at first we assume an DB failure due to missing elements
			//and try to correct the general tables, but please be 
			//reminded that plugins have to handle an update itself
			require_once('jrcmsinit.php');
			if($db->check_table(CONFIG::sql_tbl_menu) === FALSE){
				$result = init_db();
			}
			$result = upgrade_db();
		}	
		$db = NULL;	
			
		$this->own_error_handler($this->getCode(), $msg, $this->getFile(), $this->getLine(), $msg);
	}	
	
	public function setLogPath($subpath=Null) {
		if(isset($subpath))
			$this->logfile = $subpath.'/log/jrcms.log';
		else
			//$this->logfile = 'content/log/jrcms.log';
			$this->logfile = $_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/content/log/jrcms.log';
	}
	
	public function ctrlLogSize() {

		$max_fz = 100*1000;  //entry * number of entries
		//size in byte
		$fz = ($this->logfile && @is_file($this->logfile)) ? filesize($this->logfile) : NULL;
		if($fz > $max_fz) {
			//clean the logfile
			$fp = fopen($this->logfile, "w");
			fclose($fp);
		}
	}
	
	function own_error_handler($errno, $errstr, $errfile, $errline, $vars) {

		$emailTxt = "<p>An error ($errno) is occurred on line  
				<strong>$errline</strong> and in the <strong>file: $errfile.</strong>  
				<p> $errstr </p>";   
		$emailTxt .= "<pre>" . print_r($vars, 1) . "</pre>";   
		$emailHeaders = 'Content-type: text/html; charset=iso-8859-1' . "\r\n";   
		$logEntry = "> ".CONFIG::project." ".CONFIG::projectVersion." #".date( 'Y-m-d H:i:s')." #Code:$errno #File: ".basename($errfile)." #Line:$errline #Msg:$errstr\n";

		// write an log entry
		error_log($logEntry, 3, $this->logfile);
		
		if ( ($errno !== E_NOTICE) && ($errno < 2048) ) {
			//critical defect try to inform the webmaster
			if(filter_var(CONFIG::webmaster, FILTER_VALIDATE_EMAIL) !== FALSE)
				@error_log($emailTxt, 1, CONFIG::webmaster, $emailHeaders);
			die("There was an error. The webmaster is informed. Please try again later.");
		} else {
			//redirect if possible
			if (!headers_sent($filename, $linenum)) 
				header("Location: ".CONFIG::protocol.'://'.$_SERVER['HTTP_HOST'].CONFIG::srcPath);
			exit;
		}
	}
}
?>
