<?php
/**
 * This Class module provides the cms core functionality
 * 
 * Needed/Prerequisites are the classes CONFIG and MENU and USER
 *
 * @package: JRCMS
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @version $Id: cJrcms.php 607 2011-09-20 05:58:51Z jensraabe $
 *
 * @copyright Copyright (c) 2011, Jens Raabe (MIT)
 * license: MIT-style license, see jrcms.js
 */

require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iJrcms.php');

/**
 * @class JRCMS
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @copyright Copyright (c) 2009, Jens Raabe (MIT)
 */ 
class JRCMS extends MENU implements iJrcms{

	CONST VERSION			= '$Revision: 607 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2011-09-20 07:58:51 +0200 (Di, 20 Sep 2011) $';

	// do nothing at start, explicit call of parent constructor
	// is done for documentation only, if __construct does not
	// exist in child the parent constructor will be called implicitly
	public function __construct(){
		parent::__construct();
	}
	
// ------------------------------
// Utilities
// ------------------------------
	
	/**
	 * provides the plaint XML container for transport of content, menu and styles
	 * but be aware that the data mostly have to be fomatted as transparent
	 * container  <![CDATA[xxxx]]>.
	 * All data have to be provided in assoziative arrays. 
	 *
	 * @access public
	 * @param array $contArray, array $menuArray, array $stylesArray
	 * @return mixed Boolean, string (array)
	 */
	public function resp_xml($contArray, $menuArray=NULL, $stylesArray=NULL){

		if(isset($menuArray)) {
			$contArray = array_merge($contArray, $menuArray);
		}			
		if(isset($stylesArray))
			$contArray = array_merge($contArray, $stylesArray);
		return $this->array_to_xml($contArray);
		
	} 
	
// ------------------------------
	
	/**
	 * convert an assoziative array in an XML structur as well as a convential array
	 * by using a special convention that normal indexed arrays using an XML node 
	 * '_array' with an attribute 'index' for the array index itself. Converted back
	 * within jrcms.js/xml2Obj.
	 * Furthermore array Ids with a leading number are converted to a well formed
	 * XML element name with a prefix '___'. The prefix will be removed inside
	 * jrcms.js/getNameXmlNode.
	 * 
	 * Optional an one dimensional assoziative array could be supplied containing 
	 * options for further processing. This array is converted to a own node.
	 * 
	 * @access public
	 * @param array $data
	 * @return string (XML)
	 */
	public function array_to_xml($dataset){

		header("Content-Type: text/xml; charset=utf-8");
		$xml = new XmlWriter();
		$xml->openMemory();
		$xml->startDocument('1.0', 'UTF-8');
		$xml->startElement('root');
		function validElementName($key) {
			return (!is_int($key)
					//first valid character in table 65 = 'A'
					? (ord(substr($key,0,1))>=65?$key:'___'.$key)
					: '_array');
		}
		function write(XMLWriter $xml, $data){
		    foreach($data as $key => $value){
		        if(is_array($value)){
		            $xml->startElement(validElementName($key));
		            if(is_int($key))
		            	$xml->writeAttribute('index' , $key );
		            write($xml, $value);
		            $xml->endElement();
		            continue;
		        }
		        if(is_string($value)) {
		            $xml->writeElement($key, $value);
//		        	$xml->startElement($key);
//		            $xml->writeCData($value);
//		            $xml->endElement();
		        } else {
		        	$xml->startElement($key);
		        	if(is_bool($value)){ 
		        		$xml->writeAttribute('itemType', 'bool');
		        		$value = ($value?1:0);
		        	} else { 
		        		$xml->writeAttribute('itemType', 'number');
		        	}
		        	$xml->writeRaw($value);
		            $xml->endElement();
		        }
		    }
		}
		write($xml, $dataset);
		$xml->endElement();
		$xml->endDocument();
		$xmlStr = $xml->outputMemory(true);
		$xml = NULL;
		return $xmlStr;		
	}

// ------------------------------

	/**
	 * provides the assoziative content array as well as controls and additional data
	 *
	 * @access public
	 * @param string $contHtml, string $ctrlHtml, string $rightContentHtml, array $dataArray
	 * @return string
	 */
	public function resp_cont_xml($contHtml, $ctrlHtml=NULL, $rightContHtml=NULL, $dataArray=NULL){

		$content = Array('conthtml' => $contHtml );
		if(isset($ctrlHtml))
			$content['ctrlhtml'] = $ctrlHtml;
		if(isset($rightContHtml))
			$content['rightconthtml'] = $rightContHtml;
		if(isset($dataArray))
			$content['contdata'] = $dataArray;
		return $content;	
		
	}

// ------------------------------

	/**
	 * provides the assoziative array for menus
	 *
	 * @access public
	 * @param string $menuHtml, string $menuDiv
	 * @return string
	 */
	public function resp_menu_xmlelem($menuHtml, $menuDiv='divVMenu'){

		return (isset($menuHtml))
						? Array('menuhtml' => preg_replace("/(\015\012)|(\015)|(\012)/",'',$menuHtml),
								'menudiv' => $menuDiv)
						: NULL;
	}

// ------------------------------

	/**
	 * provides the assoziative array for styles
	 *
	 * @access public
	 * @param string $cssData
	 * @return string
	 */
	public function resp_styles_xmlelem($cssData){

		return (isset($cssData))
						? Array('cssdata' => $cssData)
						: NULL;
		
	}
	
// ------------------------------

	/**
	 * Converts a task/button list (buttons or icons) to a HTML list for the XML container.
	 * An action list could be assigned or the client have to use the identifier for separate
	 * event definition and control.
	 * Prerequisites are:
	 *   $staticTaskList (array)
	 *   $lang_php for each used 'id'	
	 *   
	 * Parameters are:
	 *   $selTaskList:	selects on of the predefined (icon) lists
	 *   $taskActions:	list of correspondig actions (scripts), less or equal length
	 *   $taskType:		switch between 'icon' (default) or 'button'
	 *
	 * @access public
	 * @param string $selTaskList, array $taskActions, string $taskType, string $iconpath
	 * @return string
	 */
	public function conv_task_html($selTaskList, $taskActions = Null, $taskType = 'icon', $iconpath = NULL){
		
		global $lang_php;
		$taskList = CONFIG::taskList();
		//build up the list, by using the CONFIG values
		return $this->conv_tasks($taskList[$selTaskList], $taskActions, $taskType, $iconpath);
	}																	
	
// ------------------------------

	/**
	 * Converts a task/button list (buttons or icons) to a HTML list
	 * An action list could be assigned or the client have to use the identifier for separate
	 * event definition and control.
	 *   
	 * Parameters are:
	 *   $taskList:		predefined (icon) lists
	 *   $taskActions:	list of correspondig actions (scripts), less or equal length
	 *   $taskType:		switch between 'icon' (default) or 'button'
	 *
	 * @access public
	 * @param string $taskList, array $taskActions, string $taskType, string $iconpath
	 * @return string
	 */
	public function conv_tasks($taskList, $taskActions = Null, $taskType = 'icon', $iconpath = NULL){
		
		global $lang_php;
		//ensure processing with a copy not a reference
		$cpyTaskList = $taskList;
		//merge the ctrlActions with the predefined controls list
		// a) with an provided action list
		// b) with an empty list, where the events are assigned by the client
		if(is_array($taskActions) && (count($taskActions) <= count($cpyTaskList))) {
			foreach($cpyTaskList as $key => $value){
				if(isset($taskActions[$key])) $cpyTaskList[$key]["script"] = $taskActions[$key];
			}
		} else if (isset($taskActions) && !is_array($taskActions) && (count($taskActions) > count($cpyTaskList)))
			throw new MyException("[cJrcms]: convCtrlsHtml wrong parameter assigned!"); 
		
		//generate the HTML list
		$str = '<ul class="taskbuttons'.($taskType == "iconText"?" iconText":"").'">';
		foreach($cpyTaskList as &$value){
			if(isset($value['script']) && $value['script'] == '##' && $taskType == 'icon')
				$str.= '<li><span class="buttonSeparator"><img src="'.(!isset($iconpath)?CONFIG::taskIconPath:$iconpath).'task_separator.gif" /></span></li>';
			else if ((isset($value['script']) && $value['script'] != '##') ||
					  !isset($value['script'])) {
				if($taskType == 'icon') {
					$str.= '<li><span id="'.$value['id'].'" class="'.(isset($value['script'])?('taskenabled'):('taskdisabled')).'"
					       '.(isset($value['script'])?(' onclick="'.$value['script'].'">'):('>'));
					$str.= '<img src="'.(!isset($iconpath)?CONFIG::taskIconPath:$iconpath).$value['id'].
					        (isset($value['script'])?('_enabled'):('_disabled')).'.gif" alt="'.
					        (isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'" title="'.
					        (isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'" /></span></li>';
				} else if($taskType == 'iconText') {
					$str.= '<li><span id="'.$value['id'].'" class="'.(isset($value['script'])?('taskenabled'):('taskdisabled')).'"
					       '.(isset($value['script'])?(' onclick="'.$value['script'].'">'):('>'));
					$str.= '<img src="'.(!isset($iconpath)?CONFIG::taskIconPath:$iconpath).$value['id'].
					        (isset($value['script'])?('_enabled'):('_disabled')).'.gif" alt="'.
					        (isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'" title="'.
					        (isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'" /> '.
					        (isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'</span></li>';  
				} else {
					$str.= '<li><input type="button" id="'.$value['id'].'" class="'.(isset($value['script'])?('taskenabled'):('taskdisabled')).'"
						   '.(isset($value['script'])?(' onclick="'.$value['script'].'"'):(''));
					$str.= ' value="'.(isset($lang_php[$value['id']])?$lang_php[$value['id']]:$value['id']).'" /></li>'; 
				}
			}
		}
		$str.= '</ul>';
		return $str;	
	}
																		
// ------------------------------
	
	/**
 	 * pack a json string containing html and supplied parameter 
 	 * to ensure proper function remove line feeds
 	 *
 	 * @param global
 	 * @access private
 	 * @return json string
 	 */

	private function json_str($contHtml, $vmHtml=NULL, $cssData=NULL){

		header("Content-Type: text/plain; charset=utf-8");
		// build structure
		$json = '{"json_str":';
		$json.= '{';
		// content of the divMain as HTML string
		$json.= '"contHtml":\''.preg_replace("/(\015\012)|(\015)|(\012)/",'',$contHtml).'\'';
		// vertical Menu
		$json.= (isset($vmHtml))?(',"vmHtml":\''.preg_replace("/(\015\012)|(\015)|(\012)/",'',$vmHtml).'\''):('');
		// additional styles
		$json.= (isset($cssData))?(',"cssData":'.$cssData.''):('');
		$json.= '}}';
		return $json;
	}
	
// ------------------------------
// Menu
// ------------------------------

	/**
 	 * provides the menu as plaint HTML or in a XML container
 	 * 
 	 * @param string $menuDiv, string $rspType, $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function get_menu($rspType = 'html', $menuDiv = NULL, $divHMenu = False){
	
		try{
			//now retrive the menue			
			if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=-1;}}
			if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');}}
			//special parameter for IE compatibility actions
			if(isset($_GET['mt'])) {$menuType = ($_GET['mt']==1)?(True):(False);} else {if(isset($_POST['mt'])) {$menuType = ($_POST['mt']==1)?(True):(False);} else {$menuType=True;}}

			//at first we using this central routine to clean up the user sessions
			$menu = Array();
			if($ul != '000' && !$this->clean_user_sessions()) {
				//own session was outdated, force a reboot
				$_SESSION["ul"] = '000';
//				return ($rspType == 'html')
//							?('<script type="text/javascript">jrCMS.logout();</script>')
//							:($this->resp_menu_xmlelem('<script type="text/javascript">jrCMS.logout();</script>', 'divVMenu'));
//second solution - array usage
				if($rspType == 'html')
					return ('<script type="text/javascript">jrCMS.logout();</script>');
				else {
					array_push($menu,$this->resp_menu_xmlelem('<script type="text/javascript">jrCMS.logout();</script>', 'divVMenu'));
					return (Array('menu'=>$menu));
				}			
			}							
			
			//determine origin of request
			if(!isset($menuDiv) && ($id != -1) && (($response = $this->get_menu_root('',$id))!== NULL)) $menuDiv = $response[0]['htmlDiv'];
			else if(!isset($menuDiv) && ($id == -1)) $menuDiv = 'divHMenu';
			else if(!isset($menuDiv)) throw new MyException("[JRCMS failed]: get_menu input missing or DB wrong configured");

			//determine the menu requested by id
			switch($menuDiv){
				case 'divHMenu':
					//the horizontal menu is requested
					if ($id == -1) 
//						return ($rspType == 'html')
//								?($this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType))
//								:($this->resp_menu_xmlelem( $this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType), 'divHMenu'));
//second solution - array usage
						if ($rspType == 'html')
							return ($this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType));
						else {
							array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType), 'divHMenu'));
							return (Array('menus'=>$menu));
						}
					//a horizontal menu item request the vertical menu
					//per convention no right menu is possible (empty)
//					else
//						return ($rspType == 'html')
//								?($this->get_menu_html( -1, 'divVMenu', $id, $ul, $menuType))
//								:((!$divHMenu?'':$this->resp_menu_xmlelem( $this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType), 'divHMenu')).
//								  $this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divVMenu', $id, $ul, $menuType), 'divVMenu').
//								  $this->resp_menu_xmlelem( '', 'divRMenu'));
//second solution - array usage
					else if ($rspType == 'html') 
						return $this->get_menu_html( -1, 'divVMenu', $id, $ul, $menuType);
					else {
						if($divHMenu) {
							array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( $id, 'divHMenu', 1, $ul, $menuType), 'divHMenu'));
						}	
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divVMenu', $id, $ul, $menuType), 'divVMenu'));	
						array_push($menu,$this->resp_menu_xmlelem( '', 'divRMenu'));
						return (Array('menus'=>$menu));	
					}	
					break;
				case 'divVMenu':
					//we have to distinguish between to cases
					//calling a content by external reference (http:..) will require an update of the vertical menu
					//and a vertical menu item could request a right menu  
					$rootId = $this->get_menu_root('',$id);
					
//					$result = ($rspType == 'html')
//							?($this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType))
//							:(
//								(isset($_SESSION["lastMenuRootId"]) && $_SESSION["lastMenuRootId"] != $rootId[0]['rootId'])
//								//update vertical and rigth menu
//								? ( (!$divHMenu?'':$this->resp_menu_xmlelem( $this->get_menu_html( $rootId[0]['rootId'], 'divHMenu', 1, $ul, $menuType), 'divHMenu')).
//									$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divVMenu', $rootId[0]['rootId'], $ul, $menuType), 'divVMenu').
//							  		$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType), 'divRMenu'))
//							  	//update the right menu only	
//							  	: ($this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType), 'divRMenu'))
//							  );
//second solution - array usage
					if ($rspType == 'html') 
						$menu = $this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType);
					else if	(isset($_SESSION["lastMenuRootId"]) && $_SESSION["lastMenuRootId"] != $rootId[0]['rootId'])	{
						//update vertical and rigth menu
						if($divHMenu) {
							array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( $rootId[0]['rootId'], 'divHMenu', 1, $ul, $menuType), 'divHMenu'));
						}
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divVMenu', $rootId[0]['rootId'], $ul, $menuType), 'divVMenu'));
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType), 'divRMenu'));
					} else {
						//update the right menu only
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $id, $ul, $menuType), 'divRMenu'));						
					}
						
					$_SESSION["lastMenuRootId"] = $rootId[0]['rootId'];
					return (Array('menus'=>$menu));
					break;
				case 'divPupMenu':
					//popup menu is requested by special function using a vertical menu item
//					return ($rspType == 'html')
//							?($this->get_menu_html( -1, 'divPupMenu', $id, $ul, $menuType))
//							:($this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divPupMenu', $id, $ul, $menuType), 'divRMenu'));
//second solution - array usage
					if ($rspType == 'html')
						return $this->get_menu_html( -1, 'divPupMenu', $id, $ul, $menuType);
					else {
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divPupMenu', $id, $ul, $menuType), 'divRMenu'));
						return (Array('menus'=>$menu));
					}
					break;
				case 'divRMenu':
					//special case
					//the right menu shall be visible as long as used and therefore will be requested again with root	
					if(isset($_GET['root'])) {$root = $_GET['root'];} else {if(isset($_POST['root'])) {$root = $_POST['root'];} else {$root=Null;}}
//					if(isset($root) && $rspType != 'html')
//						return $this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $root, $ul, $menuType), 'divRMenu');
//second solution - array usage
					if(isset($root) && $rspType != 'html') {
						array_push($menu,$this->resp_menu_xmlelem( $this->get_menu_html( -1, 'divRMenu', $root, $ul, $menuType), 'divRMenu'));
						return (Array('menus'=>$menu));
					}
				default:

					return 'get_menu failure, wrong parameter combination';
			}
		}catch (MyException $e) {
				$e->log();
				return "Try again, Menue is missing!";		
		}
	}
	
	/**
 	 * provides the ids of given label (first in array only)
 	 * initially used for functionality of
 	 * 
 	 * @param string $label
 	 * @access public
 	 * @return string
 	 */
	public function get_menu_ids(){
	
		if(isset($_GET['l'])) {$label = $_GET['l'];} else {if(isset($_POST['l'])) {$label = $_POST['l'];} else {$label=NULL;}}
		
		try{
			
			if(($result = $this->get_menu_item_data(NULL,$label,CONFIG::divVMenu,'000'))!==False) {
				//build up the array
				$rsp['id'] = $result[0]['id'];
				$rsp['root'] = $result[0]['rootId'];
				$rsp['div'] = $result[0]['htmlDiv'];
				$rsp['cssid'] = $result[0]['cssid'];
				
				return json_encode($rsp);				
			}
			return NULL;
	
		}catch (MyException $e) {
			$e->log();
			return "Try again, Ids are missing!";		
		}
	}

// ------------------------------
// Content
// ------------------------------

	/**
 	 * provides the content as plaint HTML or in a XML container together with buttons 
 	 * in case of standard content
 	 * 
 	 * @param string $rspType, $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function get_content($rspType = 'html', $loadEditor=False){

		try{
			if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=NULL;}}
			if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');}}
			if(isset($_GET['taskType'])) {$taskType = $_GET['taskType'];} else {if(isset($_POST['taskType'])) {$taskType = $_POST['taskType'];} else {$taskType = (isset($_SESSION["taskType"]))?($_SESSION["taskType"]):('icon');}}
			
			if ($rspType == 'html') 
				return $this->get_content_html($id,$ul);
			else { 
				$showEditorTask = (	!$loadEditor &&
								//$menuDiv != CONFIG::divRContent && 
								$this->write_access($id, $ul, 'write') && 
								$this->write_access($id, $ul, 'write', CONFIG::editorLevel)) ? True : False;

				return $this->resp_cont_xml($this->get_content_html($id,$ul,$loadEditor),
											(($this->get_std_tasks_html($id,$ul,$showEditorTask) != NULL)
												?($this->conv_task_html('std',$this->get_std_tasks_html($id,$ul,$showEditorTask),$taskType))
												:(NULL)),
											$this->get_right_content_html());
			}								
		}catch (MyException $e) {
			$e->log();
			return "Try again, Content is missing!";		
		}
	}
	
// ------------------------------

	/**
 	 * The get content for node provides to kinds of node retrieval by:
 	 * label: only the correct type of content will be determined
 	 * id: a selection of node item via menu tree will be simulated (only HMenu/VMenu) 
 	 * The content itself will be terieved by using the provided script
 	 * 
 	 * @param string $rspType, $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function get_content_for_node($act=NULL){
		
		if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=NULL;}}
		if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');}}
		if(isset($_GET['root'])) {$root = $_GET['root'];} else {if(isset($_POST['root'])) {$root = $_POST['root'];} else {$root=1;}}
		if(isset($_GET['cssid'])) {$cssid = $_GET['cssid'];} else {if(isset($_POST['cssid'])) {$cssid = $_POST['cssid'];} else {$cssid=-1;}}
		if(isset($_GET['l'])) {$labelStr = $_GET['l'];} else {if(isset($_POST['l'])) {$labelStr = $_POST['l'];} else {$labelStr=NULL;}}

		try{

			if(($cnt = (isset($labelStr)
						?$this->get_menu_item_data(NULL,$labelStr,CONFIG::divVMenu,$ul)
						:$this->get_menu_item_data($id, NULL, '', $ul))) !== False) {
							
				switch($cnt[0]['htmlDiv']){
					case CONFIG::divHMenu:
					case CONFIG::divVMenu:

						$_GET['id'] = $cnt[0]['rootId'];
						//retrieve menues of selected node
						$menuXml = (!isset($labelStr) ? $this->get_menu('xml', NULL, ($act == "getcntNode")) : NULL);
						//retrieve right content
						$rightCnt = (!isset($labelStr) ? $this->get_right_content_html() : NULL);
						//retrieve the styles of selected node
						if(($result=$this->get_menu_style_id($cnt[0]['rootId']))!==False) $_GET['cssid'] = $result;
						$styleXml = (!isset($labelStr) ? $this->get_styles() : NULL);
						//to retrieve the fitting content type we request the action via client	
						$_GET['id'] = $cnt[0]['id'];
						$cntScript = '<script type="text/javascript">'.
										(isset($cnt[0]['procName'])?$cnt[0]['procName']:'jrCMS.getContent').
										'({root:'.$cnt[0]['rootId'].',id:'.$cnt[0]['id'].',cssid:'.$cnt[0]['cssid'].'});</script>';				
						break;
				//	case CONFIG::divRMenu:	
				//	break;
					default:
						$styleXml = NULL; $rightCnt = NULL; $menuXml = NULL;
						$cntScript = '<script type="text/javascript">alert("No Node data available!");</script>';
						break;
				}
				return $this->resp_xml($this->resp_cont_xml($cntScript,NULL,$rightCnt), $menuXml, $styleXml);
				
			} else {
				//error
				$cntScript = '<script type="text/javascript">alert("Wrong Node data!");</script>';
				return $this->resp_xml($this->resp_cont_xml($cntScript,NULL,NULL));
			}
			
		}catch (MyException $e) {
			$e->log();
			return "Try again, Content is missing!";		
		}
	}

// ------------------------------

	/**
 	 * The get content data is a special function which retrieve
 	 * raw content data on base of a menu Id. By using the selection
 	 * the output could be restricted to the data needed.
 	 * 
 	 * Supported is the access to the tables:
 	 * 
 	 * CONFIG::sql_tbl_cnt AS c
 	 * CONFIG::sql_tbl_xTP AS xtp
	 * CONFIG::sql_tbl_proc AS p
	 * CONFIG::sql_tbl_pTypes AS pt
	 * CONFIG::sql_tbl_ul AS ul 
 	 * 
 	 * By example 'c.cid' retreive the content ID only
 	 * 
 	 * @param interger $id
 	 * @param string $sel
 	 * @access public
 	 * @return array
 	 */
	public function get_raw_content_data($id, $sel=NULL){
		
		return $this->get_menu_node_content($id, $sel);
		
	}
	
	
// ------------------------------
	
	/**
 	 * provides the right content as plaint HTML, but like as for standard
 	 * content the type is evaluated by using the get_content_html.  
 	 * 
 	 * @param $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function get_right_content_html(){

		try{
			if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=NULL;}}
			if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');}}

			//determine origin of request
			if(!isset($menuDiv) && ($id != -1) && (($response = $this->get_menu_root('',$id))!== NULL)) $menuDiv = $response[0]['htmlDiv'];
			else if(!isset($menuDiv) && ($id == -1)) $menuDiv = 'divHMenu';
			else if(!isset($menuDiv))
				//input missing or DB wrong configured 
				throw new MyException("[JRCMS failed]: get_right_content_html input missing or DB wrong configured");
			//determine the menu requested by id
			switch($menuDiv){
				case 'divHMenu':
				case 'divVMenu':	
					if(($data = $this->get_menu_brunch( -1, CONFIG::divRContent, $id, $ul)) !== False)
						if(($data = $this->get_content_html((empty($data[0]['id']) ? NULL : $data[0]['id']) ,$ul)) !== False) 
							return $data;
					break;
				default:
					break;
			}
			return NULL;

		}catch (MyException $e) {
			$e->log();
			return "Try again, Right Content is missing!";		
		}
	}

// ------------------------------

//	/**
//	 * writes the given content in the DB
//	 * 
// 	 * @param $_GET/$_POST
//	 * @access public
//	 * @return string
//	 */
//	 
//	public function save_content(){
//		
//		global $lang_php;
//		try{
//			if(isset($_GET['root'])) {$root = $_GET['root'];} else {if(isset($_POST['root'])) {$root = $_POST['root'];} else {$root=1;}}
//			if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=NULL;}}
//			if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');}}
//			if(isset($_POST['cont'])) {$content = $_POST['cont'];} else {$content=NULL;}
//
//			// Transaction started by Edit Task
//			// CTC: CHECK OPTIMISTIC TRANSACTION - LOCK
//			if($this->ctc_check_table(CONFIG::sql_tbl_cnt, $id)) {
//			
//				// string to be saved
//				if($this->updateContentHtml($id, str_replace('%#', '&#', preg_replace("/(\015\012)|(\015)|(\012)/",'',$content)))===True)
//					// to ensure proper processing retrieve the data
//					$strHtml = $this->get_content_html($id,$ul);
//					
//			} else { 
//			
//				$strHtml = $lang_php['data_update_failure'].(isset($_SESSION["ctc"])?' \n(CTC: '.$_SESSION["ctc"].')':'');
//			}				
//			
//			// CTC: CLOSE OPTIMISTIC TRANSACTION - UNLOCK
//			$this->ctc_unlock_table(CONFIG::sql_tbl_cnt, $id);
//			
//			return $strHtml;
//						
////			throw new MyException('Content was not changed/saved');
//			
//		}catch (MyException $e) {
//			$e->log();
//		}
//	}

// ------------------------------
// CSS and additional data
// ------------------------------

	/**
 	 * provides style and/or additional data as plaint json string or in a XML container
 	 * 
	 * @param string $rspType, $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function get_styles(){

		try{
			if(isset($_GET['cssid'])) {$cssid = $_GET['cssid'];} else {if(isset($_POST['cssid'])) {$cssid = $_POST['cssid'];} else {$cssid=-1;}}

			function reduce_array($a) {
				$r = Array('cssid'=>$a['cssid']);
				if(isset($a['mnu_css'])) $r['mnu_css'] = $a['mnu_css'];
				if(isset($a['mkey'])) $r['mkey'] = $a['mkey'];
				if(isset($a['mdesc'])) $r['mdesc'] = $a['mdesc'];
				return $r;				
			}
			if(($data = $this->get_style_data($cssid)) !== False) {
				$data = array_map("reduce_array", $data);
//				return $this->resp_styles_xmlelem($data[0]);
			}
			return (isset($data)&&!empty($data)?$this->resp_styles_xmlelem($data[0]):Null);
			
		}catch (MyException $e) {
			$e->log();
			return "Try again, styles are missing!";		
		}
	}

// ------------------------------
// Login/Logout 
// ------------------------------

	/**
 	 * routine for Login/Logout Html
 	 * 
 	 * @param $_GET/$_POST
 	 * @access public
 	 * @return html string
 	 */
	public function get_login_logout_html($act=NULL){

		global $lang_php;
		try{
			if(isset($_GET['ul'])) $ul = $_GET['ul']; else if(isset($_POST['ul'])) $ul = $_POST['ul']; else $ul=(isset($_SESSION["ul"])) ? ($_SESSION["ul"]) : ('000');
			if(isset($_GET['a'])) $act = $_GET['a']; else if(isset($_POST['a'])) $act = $_POST['a']; else $act=(isset($_SESSION["a"])) ? ($_SESSION["a"]) : (NULL);
			if(isset($_GET['dsp_ur'])) $dsp_ur = $_GET['dsp_ur']; else if(isset($_POST['dsp_ur'])) $dsp_ur = $_POST['dsp_ur']; else $dsp_ur=NULL;
			
			$strHtml = '<script type="text/javascript">jrCMS.setUserLevel(\''.$ul.'\');</script>';
			if (CONFIG::loginHidden && (!isset($act) || $act == 'gindiv')) {
				if ($ul == '000') {
					$strHtml.= '<span class="nav" id="loginbutton" onclick="jrCMS.getLogin({a:\'gin\'}); return false;">'.$lang_php['u_login'].'</span>';
				} else {
					$strHtml.= '<span class="nav" id="logoutbutton" onclick="jrCMS.logout({a:\'out\'}); return false;">'.$lang_php['u_logout'].'</span>';
				}
			} else {
				if ($ul == '000' ) {
					//not logged in
					if($lang_php['u_login_contact']!='')
						$loginDialogData[] = $this->dialog_element('text', 
																	'<br />'.$lang_php['u_login_contact'].'<a href="mailto:'.CONFIG::webmaster.'">'.$lang_php['webmaster'].'</a>.<br />&nbsp;<br />', 'loginDialog', 'text');
					$loginDialogData[] = $this->dialog_element('UserName', $lang_php['u_user'], 'loginDialog', 'input:jrCMS');
					$loginDialogData[] = $this->dialog_element('UserPass', $lang_php['u_pw'], 'loginDialog', 'input:jrCMS');
					$strTask = '<input id="loginbutton" type="button" tabindex="10" value="'.$lang_php['u_login_b'].'" onclick="jrCMS.login({a:\'in\'}); return false;" />';
					$loginDialogData[] = $this->dialog_element('text', $strTask, 'loginDialog', 'text');
					$loginHeaderData = $this->dialog_header('ul', $ul, 'in');
					$dataset = array('header' => $loginHeaderData,
							 'data' => $loginDialogData);
//					$strHtml = json_encode($dataset);
					//XML
					$strHtml = $this->array_to_xml($dataset);
				} else {
					
					$loginDialogData[] = $this->dialog_element('UserName', ((($user = $this->get_user())!==False)?($user[0]['UserName']):('')), 'loginDialog', 'input:jrCMS', 'disabled');
					if(isset($dsp_ur)) 
						$ur = $this->get_userlevel_role($ul);
					$loginDialogData[] = $this->dialog_element('ul', 
						(isset($dsp_ur) && isset($ur) ? $ur : $ul), 'loginDialog', 'input:jrCMS', 'disabled');
					$strTask = '<div class="logoutctrls"><input id="logoutbutton" type="button" tabindex="10" value="'.$lang_php['u_logout'].'" onclick="jrCMS.logout({a:\'out\'}); return false;" /></div>';
					if ((CONFIG::loginWithPWMaint > 0) && (intval($ul ) >= (CONFIG::loginWithPWMaint * 100))) {
						$strTask.= '<div class="usermaint" onclick="jrCmsUser.editUser({a:\'editUserNoGrid\', uid:'.$this->get_user_id($user[0]['UserName']).'}); return false;">';
						$strTask.= '<img src="'.CONFIG::taskIconPath.'icon_maintenance.gif" title="'.$lang_php['u_usm_l2'].'" class="usermaint" /><div>';
					}
					$strTask.= '<div class="nf"></div>';
					$loginDialogData[] = $this->dialog_element('text', $strTask, 'loginDialog', 'text');
					$loginHeaderData = $this->dialog_header('ul', $ul, 'out');
					$dataset = array('header' => $loginHeaderData,
							 'data' => $loginDialogData);
//					$strHtml = json_encode($dataset);
					//XML
					$strHtml = $this->array_to_xml($dataset);
				}
			}
			return $strHtml;
			
		}catch (MyException $e) {
			$e->log();
			return "login/logout not possible";		
		}
	}

// ------------------------------

	/**
 	 * perform the Login 
 	 * 
 	 * @param $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function perf_login(){
	
		global $lang_php;  //needed for get_login_logout_html
		try{
			if(isset($_GET['un'])) {$un = $_GET['UserName'];} else {if(isset($_POST['UserName'])) {$un = $_POST['UserName'];} else {$un=NULL;}}
			if(isset($_GET['up'])) {$up = $_GET['UserPass'];} else {if(isset($_POST['UserPass'])) {$up = $_POST['UserPass'];} else {$up=NULL;}}
			if(isset($un) && isset($up)){
				if($this->login($un, $up)===True) {
					$_SESSION["ul"] = $this->logged_in();
					return '<script type="text/javascript">jrCMS.setUserLevel(\''.$_SESSION["ul"].'\');</script>'.
						   ($_SESSION["ul"] == '000'?'<script type="text/javascript">alert("'.$lang_php['input_missing'].'");jrCMS.setOptions({jr_loginFailure:true});</script>':'');
				} else {
					return '<script type="text/javascript">jrCMS.setUserLevel(\''.$_SESSION["ul"].'\');</script>'. 
						   '<script type="text/javascript">alert("'.$lang_php['logged_out'].'");</script>';
				}
			} 
			throw new MyException('Login Failure, UserName or Password missing');

		}catch (MyException $e) {
			$e->log();
			return "Login Failure";
		}
	}

// ------------------------------

	/**
 	 * perform Logout 
 	 * 
 	 * @param none
 	 * @access public
 	 * @return string
 	 */
	public function perf_logout(){
	
		global $lang_php;  //needed for get_login_logout_html
		try{
			if($this->logout()===TRUE) {
				$_SESSION["ul"] = $this->logged_in();
				return '<script type="text/javascript">jrCMS.setUserLevel(\''.$_SESSION["ul"].'\');</script>';
			}
			throw new MyException('Logout Failure, DB not changed');
			
		}catch (MyException $e) {
			$e->log();
			return "Logout failure";
		}
	}
	
// ------------------------------

	/**
 	 * Set a new user password and perform Login 
 	 *
 	 * @deprecated
 	 */
	public function set_password_perf_login(){
	
		global $lang_php;  //needed for get_login_logout_html
	}
	
// ------------------------------
// User Timestamp 
// ------------------------------

	/**
 	 * Set a user time stamp 
 	 * 
 	 * @access public
 	 * @return string
 	 */
	
	
	
	
// ------------------------------
// User Level
// ------------------------------

	/**
 	 * displays the actual user level
 	 * 
 	 * @param $_GET/$_POST
 	 * @access public
 	 * @return html string
 	 */
	public function get_userlevel(){

		try{
			if(isset($_GET['ul'])) {$ul = $_GET['ul'];} else {if(isset($_POST['ul'])) {$ul = $_POST['ul'];} else {$ul=$this->logged_in();}}
			return $ul;

		}catch (MyException $e) {
			$e->log();
			$_SESSION["ul"] = '000';
			return $_SESSION["ul"];
		}
	}

// ------------------------------
// backup DB
// ------------------------------

	/**
 	 * backup the whole SQL DB
 	 * 
 	 * @param none
 	 * @access public
 	 * @return boolean
 	 */
	public function backup(){
	
		try{
			return $this->db_backup();
		}catch (MyException $e) {
			$e->log();
			return False;
		}
	}
	
// ------------------------------
// Language
// ------------------------------

	/**
 	 * returns the used language
 	 * 
 	 * @param none
 	 * @access global
 	 * @return language string
 	 */
	public function get_lang(){
		return $_SESSION["language"];
	}
	
// ------------------------------
	
	/**
 	 * set language and return the language
 	 * 
 	 * @param $_GET/$_POST
 	 * @access public
 	 * @return string
 	 */
	public function set_lang(){
		
		if(isset($_GET['lang'])) {$l = $_GET['lang'];} else {if(isset($_POST['lang'])) {$l = $_POST['lang'];} else {$l='de';}}
		$_SESSION["language"] = $l;
		return $this->get_lang();
	}
	
// ------------------------------
// retrieves the language array for js
// ------------------------------

	/**
	 * Returns the language specifics as json for js
	 * 
 	 * @param none
	 * @access public
	 * @return string
	 */
	public function get_language_file(){
		global $lang_php, $lang_js;
		$old_lang = $lang_js;
		
		if(isset($_GET['fn'])) {$f = $_GET['fn'];} else {if(isset($_POST['fn'])) {$f = $_POST['fn'];} else {$f='lang.de';}}
		
		include_once(str_replace(CONFIG::protocol.'://'.$_SERVER['HTTP_HOST'], $_SERVER['DOCUMENT_ROOT'], $f));
		
//		return json_encode(array_merge($old_lang, $lang_js));
		$new_lang = array_map("utf8_encode", array_merge($old_lang, $lang_js));
//        $jsonstr = json_encode($new_lang);
////        $jsonstr = str_replace('\\r','',$jsonstr);
////        $jsonstr = str_replace('\\n','',$jsonstr);
////        $jsonstr = str_replace('\\t','',$jsonstr);
////        $jsonstr = str_replace('\\','',$jsonstr);
//        return $jsonstr;

        return $this->array_to_xml($new_lang);
	}
	
// ------------------------------
// preload Images
// ------------------------------

	/**
	 * Returns the images to be pre-loaded in a xml structure.
	 * 
 	 * @param none
	 * @access public
	 * @return string
	 */
	public function pre_load_images(){
	
		if(isset($_GET['ipp'])) {$imgPreloadPrefix = $_GET['ipp'];} else {if(isset($_POST['ipp'])) {$imgPreloadPrefix = $_POST['ipp'];} else {$imgPreloadPrefix='pre';}}
 		if(isset($_GET['taskType'])) {$_SESSION["taskType"] = $_GET['taskType'];} else {if(isset($_POST['taskType'])) {$_SESSION["taskType"] = $_POST['taskType'];}}
		
		$filepath = $_SERVER['DOCUMENT_ROOT'].CONFIG::docPath.'/images/';
		$imgarr = Array( 'img' => Array());
		$imgCtr = 0;
		$jpg_files = list_files_by_ext("jpg", $filepath);
		$png_files = list_files_by_ext("png", $filepath);
		$gif_files = list_files_by_ext("gif", $filepath);

		if((is_array($jpg_files)) || (is_array($gif_files)) || (is_array($png_files))) {
			$img_files = (is_array($jpg_files) && is_array($gif_files))?(array_merge($jpg_files,$gif_files)):($jpg_files);
			$img_files = (is_array($img_files) && is_array($png_files))?(array_merge($img_files,$png_files)):($img_files);
			if (!is_array($img_files)) {
				$img_files = (is_array($png_files) && is_array($gif_files))?(array_merge($gif_files,$png_files)):
											((is_array($gif_files))?($gif_files):($png_files));
			}
			foreach ($img_files as &$value) {
				if (strpos($value, $imgPreloadPrefix) !== false) {
					$imgCtr++;
					array_push($imgarr['img'],Array('src' => './images/'.$value));
				}
			}
		}
		return $this->array_to_xml($imgarr);
	}
	
// ------------------------------
// update xtp table
// ------------------------------

	/**
	 * updates the table xTp.
	 * 
 	 * @param none
	 * @access public
	 * @return string
	 */
	public function update_xtp_table(){
		
		if(isset($_GET['procs'])) {$procs = $_GET['procs'];} else {if(isset($_POST['procs'])) {$procs = $_POST['procs'];} else {$procs=NULL;}}
		
		$result = true;
		if(CONFIG::dbUpgrade && isset($procs)) {
			try{
				$procs = json_decode(stripslashes($procs), True);
			}catch (MyException $e) {
				$e->log();
				return False;
			}
				
			if(!is_array($procs))
				return false;

			foreach($procs as &$value){
				if(isset($value))
					$result &= $this->insert_procedure($value['instance'].'.'.$value['functionName'], $value['uniqueId'], $value['functionType']);
			}
		}
		return $result;
	}
	
// ------------------------------
// about and .htaccess
// ------------------------------
//TODO: htaccess

	/**
	 * retrieve the about notification
	 * 
 	 * @param none
	 * @access public
	 * @return string
	 */
	public function get_about(){

		if(($result=$this->get_single_value('g.about', CONFIG::sql_tbl_gData.' AS g')) != False)
			return $result[0]['about'];
		else
			return 'Missing project data in database.';
	}

// ------------------------------
// dialog support
// ------------------------------
	
	/**
	 * build a row of a dialog 
	 * 
 	 * @param string valueName
 	 * @param string value
 	 * @param string $valueGroup
 	 * @param string $valueDialogMethod
 	 * @param string $valueValidator
	 * @access public
	 * @return array
	 */
	public function dialog_element($valueName, $value, $valueGroup='set', $valueDialogMethod = 'input', $valueValidator = 'required'){
		return array("valueName"=>$valueName,"value"=>$value,"valueGroup"=>$valueGroup,"valueDialogMethod"=>$valueDialogMethod,"valueValidator"=>$valueValidator);	
	}

	/**
	 * build the header of a dialog
	 * @param string $refName
	 * @param mixed $value
	 * @param string $action
	 * @access public
	 * @return array
	 */
	public function dialog_header($refName, $value, $action=Null){
		return (isset($action)
				? array('refName' => $refName, 'value' => $value, 'a' => $action)
				: array('refName' => $refName, 'value' => $value, 'a' => 'todo'));
	}
}
?>