<?php
/**#@+
 * This Class module provides the imacs Server API to be used by plugins
 * 
 * Furthermore the user maintenance could be enhanced by provision of a 
 * class extending the definition of the USER class as used by the core for 
 * some predefined methods. An example of such a class could be ound at the end
 * of this file.
 * Prerequisite for the usage of this feature is a configuration in the generic
 * data table of the core which have to provide the following values:
 * 
 * 	getUserProf				Class name of the application specific 
 *							user profile interface,
 *							e.g. 'USERPROF'
 * 	getUserProfSrc			filename containing the user profile class
 *							including the relative path from the root
 *							directory e.g. '/src/plugins/jrcmsuserprof/cUserProf.php'
 * 
 * There are no verifications inside of the core! If methods as defined below
 * are not available the core functionality is available only.
 * Reserved Methods are:
 * 
 *  upCheckAccessRight			To be used for specific access rights providing 
 *								the third column of the access right string.
 *  
 *  upGetUser					Replaces the get_core_user where additional data
 *								could be provided for a user or user list
 *  
 *  upLoginWSessionParameter 	Called instead of the USER:login, which internally
 *  							processes the USER:login but give the possibilty
 *  							to set user depended session variables or perform
 *  							necessary actions during the login sequence.
 * 
 * Session variables which could be used in the upLoginWSessioParameter or by 
 * any application specific function:
 * 
 * 	// --Xinha-File Manager -----------------------
 *	//$_SESSION['allow_new_dir'] 		// boolean, control file manager feature
 *	//$_SESSION['allow_delete']			// boolean, control file manager feature
 *	//$_SESSION['images_subdir']		// subdirectory of CONFIG::uploadDIR
 *	// --------------------------------------------
 * 
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @version $Id: cJrcmsApi.php 607 2011-09-20 05:58:51Z jensraabe $
 * 
 * @copyright Copyright (c) 2011, Jens Raabe (MIT)
 * license: MIT-style license, see jrcms.js
 */

/**
 * @ignore
 */
require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iJrcmsApi.php');
require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'cJrcms.php');
require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'cUser.php');
require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'cJrcmsHelp.php');

/**
 * @class JRCMSAPI
 * @package Server
 * @subpackage API
 */ 
class JRCMSAPI implements iJrcmsApi{
	
	CONST VERSION			= '$Revision: 607 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'Core: JRCMSAPI';
	CONST LAST_CH			= '$Date: 2011-09-20 07:58:51 +0200 (Di, 20 Sep 2011) $';

	/**
 	 * @ignore in php documentation
 	 */ 
	private $cms; 	// instance: cms/menu
	/**
 	 * @ignore in php documentation
 	 */ 
	private $user; 	// instance: user/user rights

	/**
	 * instantiate the singel classes
 	 * @ignore in php documentation
 	 */ 
	public function __construct(){
		$this->cms = new JRCMS;
		$this->help = new JRCMSHELP;
	}
	
	/**
	 * destroys the singel classes
 	 * @ignore in php documentation
 	 */ 
	public function __destruct(){
		$this->cms = NULL;
		$this->help = NULL;
	}
	
// ------------------------------
	/**#@+
	 * Build the XML message container
	 * convert an assoziative array in an XML structur as well as a convential array
	 * by using a special convention that normal indexed arrays using an XML node 
	 * '_array' with an attribute 'index' for the array index itself
	 * 
	 * Optional a set of values could be provided for further control of processing
	 * 
	 * @access public
	 * @param array data array (either assoziative, classic or combined)
	 * @param array optionsset (one dimensional assoziative)
	 * @return string
	 */
	public function array2Xml($dataset){
		return $this->cms->array_to_xml($dataset);
	}
	 
// ------------------------------
	/**#@+
	 * Build the XML message container 
	 * 
	 * <p>The XML message container is the default structure for communication
	 * between Client and Server and consist of content (HTML), menus (HTML) 
	 * and styles parts.<br>
	 * To read the container at the Client the javascript function 
	 * jrCmsApi.processXolContainer have to be used</p>
	 *
	 * @access public
	 * @param string XML node of getContentAndButtons 
	 * @param string XML node of getMenu
	 * @param string XML node of getStyles
	 * @return string
	 */
	public function getMessageContainer($contArray, $menuArray=NULL, $stylesArray=NULL){
		return $this->cms->resp_xml($contArray, $menuArray, $stylesArray);
	}
// ------------------------------
	/**#@+
	 * Build the <conthtml>, <ctrlhtml>, <rightconthtml> array for XML container
	 * 
	 * <p>The content $contHtml, right content $rightContHtml and the button
	 * list $ctrlHtml are packed each in a an assoziative array.<br> 
	 * But please be aware that the right content could be used alternatively to the right menu</p>
	 * <p>Optional a data structure could be provided. In case of an element 
	 * 'dlgoptions' the processXolContainer try to process the data as a
	 * dialog.</p>
	 * <p> array2Xml( Array( 'header' => dialogHeader<br/>
	 *                      ,'data' => $content<br/>
	 *                      ,'dlgoptions' =><br/>
	 *                         Array( 'dlgContent' => <div element id><br/>
	 *                               , <further options of dialog> )));</p>
	 *
	 * @access public
	 * @param string HTML string of main frame
	 * @param string optional HTML list of task buttons/icons
	 * @param string optional right HTML content
	 * @param array optional data structure
	 * @return string
	 */
	public function getContentAndButtons($contHtml, $ctrlHtml=NULL, $rightContHtml=NULL, $dataArray=NULL){
		return $this->cms->resp_cont_xml($contHtml, $ctrlHtml, $rightContHtml, $dataArray);
	}
// ------------------------------
	/**#@+
	 * The routine converts a given task list defined in the config 
	 * by setting the provided buttons/icons values. 
	 * The result will be provided as an HTML list.
	 * The values of the task list have to correspond to a predefined button/icon list in 
	 * CONFIG::taskList.
	 * 
	 * @access public
	 * @param string	selector of lists defined in CONFIG::taskList
	 * @param array		array of actions e.g. array(8 => "jrCMS.reloadPage();return false;"))
	 * @param string	selector 'icon'/'button'
	 * @param string	own icons path (default: images/btn)
	 * @return string
	 */
	public function convertTaskList($selTaskList, $taskActions = Null, $taskType = 'icon', $iconpath = NULL){
		return $this->cms->conv_task_html($selTaskList, $taskActions, $taskType, $iconpath);
	}
// ------------------------------
	/**#@+
	 * The routine converts a given own task list 
	 * by setting the provided buttons/icons values. 
	 * The result will be provided as an HTML list.
	 * 
	 * @access public
	 * @param array		array of task buttons
	 * @param array		array of actions e.g. array(8 => "jrCMS.reloadPage();return false;"))
	 * @param string	selector 'icon'/'button'
	 * @param string	own icons path (default: src/images/btn)
	 * @return string
	 */
	public function convertOwnTaskList($taskList, $taskActions = Null, $taskType = 'icon', $iconpath = NULL, $nodeId = NULL){
                $cpyTaskActions = $taskActions;
                if(isset($taskActions) && count($taskActions)>0){
                    foreach($cpyTaskActions as $key=>$value){
                        if(is_array($value) && count($value)==2){
                            $ul = $this->getUserlevel();
                            if(!empty($nodeId) && $this->checkAccessRight($nodeId, $ul, 'create', NULL, $cpyTaskActions[$key][0]))
                                $cpyTaskActions[$key] = $cpyTaskActions[$key][1];
                            else
                                unset($cpyTaskActions[$key]);
                        }
                    }
                }
		return $this->cms->conv_tasks($taskList, $cpyTaskActions, $taskType, $iconpath);
//		return $this->cms->conv_tasks($taskList, $taskActions, $taskType, $iconpath);
	}
// ------------------------------
	/**#@+
	 * Provides an own menu
	 * 
	 * <p>This function provide the own menu structure in a array for further processing
	 * in an XML container.</p>  
	 *
	 * @access public
	 * @param string	menu (HTML)
	 * @param string	type of menu (divHMenu, divVMenu, divRMenu)
	 * @return string
	 */
	public function getOwnMenu($menuHtml, $menuDiv='divVMenu'){
		return ($this->cms->resp_menu_xmlelem($menuHtml, $menuDiv));
	}
// ------------------------------
	/**#@+
	 * Provides the menu as HTML
	 * 
	 * <p>The method retrieves the menu as HTML list or encapsulated in Array for
	 * further processing as XML.<br>
	 * During start up the first menu will be direct provided as HTML whereas the 
	 * following updates shall be processed by using the XML container. By using the
	 * response type 'xml' the structure is provided in an array.</p>  
	 *
	 * @access public
	 * @param string	type of response 'html'/'xml'
	 * @param string	type of menu (divHMenu, divVMenu, divRMenu)
	 * @return string
	 */
	public function getMenu($rspType='html', $menuDiv = NULL, $divHMenu = False){
		return ($this->cms->get_menu($rspType, $menuDiv, $divHMenu));
	}
// ------------------------------
	/**#@+
	 * Provides the menu root element of a given label/menu type
	 *
	 * @access public
	 * @param string	type of menu (divHMenu, divVMenu, divRMenu)
	 * @param string	menu label
	 * @return array
	 */
	public function getMenuRoot($menuDiv, $label){
		return $this->cms->get_menu_root($menuDiv,NULL,$label);
	}

// ------------------------------
	/**#@+
	 * Add a javascript method to the xTable
	 * 
	 * <p>A java script method will be added if not existing at the procedure
	 * and xTP table of data base.</p>
	 *
	 * @access public
	 * @param string	procedure name (instance.function)
	 * @param int		class/function ID (four digits)
	 * @param string	a predefined type (ajax, aeditor, alink, js)
	 * @return boolean
	 */
	public function insertProcedure($procName, $classId, $type, $comment=NULL){
		return $this->cms->insert_procedure($procName, $classId, $type, $comment);
	}	
	
// ------------------------------
	/**#@+
	 * Insert a menu item
	 * 
	 * <p>A single menu item with standard settings will be inserted 
	 * just after the parent element.</p>
	 *
	 * @access public
	 * @param string	lable of the item
	 * @param string	label of the parent item
	 * @param string	type of menu (divHMenu, divVMenu, divRMenu)
	 * @param string	**do not use**
	 * @return boolean
	 */
	public function insertMenuItem($label,$parentLabel,$menuDiv,$xtp){
		return $this->cms->insert_menu_item($label,$parentLabel,$menuDiv,$xtp);
	}	
	
// ------------------------------
	/**#@+
	 * Provides the CSS definitions of an menu item
	 * 
	 * <p>On base of a received menu item id the special styles 
	 * are retrieved as JSON string encapsulated in a XML node.</p>
	 * <p>The parameter $rspType is for future usage and has no effect.</p>
	 *
	 * @access	public
	 * @param	string	**do not use**
	 * @return	string
	 */
	public function getStyles(){
			return ($this->cms->get_styles());
	}

// ------------------------------
	/**#@+
	 * Retrieve content and buttons
	 * 
	 * <p>Similar to getContentAndButtons this method retrieve the content and
	 * buttons but not the right content.<br>
	 * By using the rspType = 'html' the content will be provided only otherwise
	 * the XML structure for content and buttons.
	 *
	 * @access	public
	 * @param	string	response as html/xml
	 * @return	string
	 */
	public function getContent($rspType = 'html', $loadEditor=False){
		return $this->cms->get_content($rspType, $loadEditor);
	}

// ------------------------------
	/**#@+
	 * Retrieve raw content data as selected
	 * 
 	 * <p>This special function retrieve the raw content data 
 	 * for a given menu Id. <br> By using the selection the output could be 
 	 * restricted to the data needed for further processing.<br>
 	 * Supported is the access to the tables:<br>
	 *  CONFIG::sql_tbl_cnt AS c <br>
 	 *  CONFIG::sql_tbl_xTP AS xtp <br>
	 *  CONFIG::sql_tbl_proc AS p <br>
	 *  CONFIG::sql_tbl_pTypes AS pt <br>
	 *  CONFIG::sql_tbl_ul AS ul  <br>
 	 * By example 'c.cid' retrieve the content ID only
 	 *  
	 * @access	public
	 * @param	int		menu Id (menu tree)
	 * @param	string	selection
	 * @return	array
	 */
	public function getRawContentData($id, $sel=NULL){
		return $this->cms->get_menu_node_content($id, $sel);
	}
	
// ------------------------------
	/**#@+
	 * Writes the content in the database.
	 *
	 * @access	public
	 * @param	int		menu Id (menu tree)
	 * @param	string	content as HTML string
	 * @param	int		content Id
	 * @return	boolean
	 */
	public function updateContent($id, $content, $cid){
		return $this->cms->updateContentHtml($id, $content, $cid);
	}

// ------------------------------
	/**#@+
	 * Retrieves the right content
	 *
	 * @access	public
	 * @return	string
	 */
	public function getRightContent(){
		return $this->cms->get_right_content_html();
	}
// ------------------------------
	/**#@+
	 * provide the actual user level
	 *
	 * @access 	public
	 * @return 	string	
	 */
	public function getUserlevel(){
		return $this->cms->get_userlevel();
	}
// ------------------------------
	/**#@+
	 * provide the possible user level list
	 *
	 * @access 	public
	 * @return  array	
	 */
	public function getUserlevelList(){
		return $this->cms->get_userlevels();
	}
// ------------------------------
	/**#@+
	 * provide the possible access level list
	 *
	 * @access 	public
	 * @return  array	
	 */
	public function getAccesslevelList(){
		return $this->cms->get_userlevels(False);
	}
// ------------------------------
	/**#@+
	 * retrieve a user
	 *
	 * @access 	public
	 * @param	integer	user Id
	 * @param	integer	start value of user list
	 * @param	integer	number of users
	 * @param	string	filter conditions
	 * @param	string	user level only flag
	 * @return  array
	 */
	public function getUser($uid, $start = NULL, $limit = 1, $filter=NULL, $ul_only = NULL){
		return $this->cms->get_user($uid, $ul_only, $start, $limit, $filter);
	}
// ------------------------------
	/**#@+
	 * retrieve the user list
	 *
	 * @access 	public
	 * @param	integer	start value of user list
	 * @param	integer	number of users
	 * @param	string	filter conditions
	 * @return  array
	 */
	public function getUserList($start = NULL, $limit = 1, $filter=NULL){
		return $this->cms->get_user(-1, NULL, $start, $limit, $filter);
	}
// ------------------------------
	/**#@+
	 * determine number of users
	 *
	 * @access 	public
	 * @param	string	filter conditions
	 * @return  integer
	 */
	public function getUserNumber($filter=NULL){
		return $this->cms->get_user_no($filter);
	}
// ------------------------------
	/**#@+
	 * insert a new user
	 *
	 * @access 	public
	 * @param	string	encoded string of user name
	 * @param	string  encoded password
	 * @param	string	user level
	 * @return  boolean	
	 */
	public function insertUser($name, $pass, $ul = NULL){
		$ul = (!isset($ul) && isset($_SESSION["ul"]))?($_SESSION["ul"]):($ul);
		return $this->cms->insert_user($name, $pass, $ul);
	}
// ------------------------------
	/**#@+
	 * update the user
	 *
	 * @access 	public
	 * @param	integer	user Id
	 * @param	string	encoded string of user name
	 * @param	string  encoded password
	 * @param	string	user level
	 * @return  boolean	
	 */
	public function updateUser($uid, $name = NULL, $pass = NULL, $ul = NULL){
		return $this->cms->update_user($uid, $name, $pass, $ul);
	}
// ------------------------------
	/**#@+
	 * delete a user
	 *
	 * @access 	public
	 * @param	integer	user Id
	 * @return  boolean	
	 */
	public function deleteUser($uid){
		return $this->cms->del_user($uid);
	}
// ------------------------------
	/**#@+
	 * State of actual user
	 * 
	 * <p>The response is given as user level.<br>Where '000' is synonym to not logged in.</p>
	 *
	 * @access 	public
	 * @return  string	
	 */
	public function isUserLoggedIn(){
		return $this->cms->logged_in();
	}
// ------------------------------
	/**#@+
	 * Provide the actual used language
	 *
	 * @access	public
	 * @return 	string
	 */
	public function getLanguage(){
		return $this->cms->get_lang();
	}
// ------------------------------
	/**#@+
	 * Provide the login/logout mask
	 *
	 * @access	public
	 * @param	string	action
	 * @return	string
	 */
	public function getLoginLogoutHtml($act=NULL){
		return $this->cms->get_login_logout_html($act);
	}	
// ------------------------------
	/**#@+
	 * Utiliy to quote SQL strings
	 *
	 * @access 	public
	 * @param 	string value
	 * @param	string type{VARCHAR, CHAR, NUMBER}
	 * @return	string
	 */
	public function sqlQS($value, $type){
		return $this->cms->sql_qs($value, $type);
	}
// ------------------------------
	/**#@+
	 * This function retrieves one single value from DB layer.
	 *
	 * @access 	public
	 * @param 	string db query: column(s), aggregate functions
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param	string db query: where clause evtl. incl. AND|OR
	 * @param	string db query: extensions like ORDER|GROUP BY
	 * @return	array|boolean
	 */
	public function getSingleValueFromDb($columns = NULL, $tables = NULL, $where = "1", $ext = NULL, $debug=NULL){
		return $this->cms->get_single_value($columns, $tables, $where, $ext, $debug);
	}
	
// ------------------------------
	/**#@+
	 * This function creates one single value of DB layer.
	 *
	 * @access 	public
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param 	string db query: column(s), aggregate functions
	 * @param 	string db query: value(s)
	 * @param	string db query: where clause evtl. incl. AND|OR
	 * @param	boolean response db instance id (False/True)
	 * @return	boolean
	 */
	public function insertSingleValueInDb($table = NULL, $columns = NULL, $values = NULL, $insertId = False, $debug=NULL){
		return $this->cms->insert($table, $columns, $values, $insertId, $debug);
	}	

// ------------------------------
	/**#@+
	 * This function creates a DB entry by using posted dialog values.
	 * For differentiation between numeric and string values, all
	 * numeric keys could be idendified by an array.
	 *
	 * @access 	public
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param 	array: defining numeric keys
	 * @param	boolean response db instance id (False/True)
	 * @return	boolean
	 */

	public function insertPostedValuesInDb($table = NULL, $numValueArr = NULL, $insertId = False, $debug=NULL){
		$cstr=''; $vstr='';
		foreach($_POST as $key => $value){
			$cstr.= ",".$key;
			$vstr.= (isset($numValueArr) && in_array($key, $numValueArr))
					? (",".(int)$value)
					: (",'".htmlentities((string)$value, ENT_QUOTES)."'");
		}
		return $this->cms->insert($table, substr($cstr,1), substr($vstr,1), $insertId, $debug);
}	
	
// ------------------------------
	/**#@+
	 * This function creates or replaces single values of DB layer.
	 *
	 * @access 	public
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param 	string db query: column(s), aggregate functions
	 * @param 	string db query: value(s)
	 * @param	string db query: where clause evtl. incl. AND|OR
	 * @param	boolean response db instance id (False/True)
	 * @return	boolean
	 */
	public function replaceSingleValueInDb($table = NULL, $columns = NULL, $values = NULL, $insertId = False){
		return $this->cms->replace($table, $columns, $values, $insertId);
	}	
	
// ------------------------------
	/**#@+
	 * This function updates one single value of DB layer.
	 *
	 * @access 	public
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param 	string db query: value(s)
	 * @param	string db query: where clause evtl. incl. AND|OR
	 * @param	string db query: extensions like ORDER|GROUP BY
	 * @return	boolean
	 */
	public function updateSingleValueOfDb($tables = NULL, $values=NULL, $where = "1", $ext = NULL){
		return $this->cms->update_single_value($tables, $values, $where, $ext);
	}	
	
// ------------------------------
	/**#@+
	 * This function deletes one single value of DB layer.
	 *
	 * @access 	public
	 * @param 	string db query: column(s), aggregate functions
	 * @param	string db query: table(s) evtl. incl. JOINS
	 * @param	string db query: where clause evtl. incl. AND|OR
	 * @param	string db query: extensions like ORDER|GROUP BY
	 * @return	boolean
	 */
	public function deleteSingleValueOfDb($columns="*", $tables = NULL, $where = "1", $ext = NULL){
		return $this->cms->delete_single_value($columns, $tables, $where, $ext);
	}	
	
// ------------------------------
	/**#@+
	 * This function retrieves an array from DB layer.
	 *
	 * @access 	public
	 * @param 	string	db query: column(s), aggregate functions
	 * @param	string	db query: table(s) evtl. incl. JOINS
	 * @param	string	db query: where clause evtl. incl. AND|OR
	 * @param	string	db query: start parameter for subset of query (mySql: LIMIT; msSql: TOP)
	 * @param	string	db query: row count for subset of query (mySql: LIMIT; msSql: TOP)
	 * @param	string	db query: id parameter for subset of query (only msSql: TOP)
	 * @param	string	db query: extensions like ORDER|GROUP BY
	 * @return	array|boolean
	 */
	public function getArrayFromDb($columns = NULL, $tables = NULL, $where = "1", $start = NULL, $rows = NULL, $id = NULL, $ext = NULL, $debug=NULL){
		return $this->cms->get_array($columns, $tables, $where, $start, $rows, $id, $ext, $debug);
	}
// ------------------------------
	/**#@+
	 * This function create a new table in DB.
	 *
	 * @access 	public
	 * @param 	string	table
	 * @param	array	values of table elements, [name of column, type, null, auto_inc, key, unique, default, comment]
	 * @return	boolean
	 */
	public function createTable($table, $columns, $type='MyISAM'){
		return $this->cms->create_table($table, $columns, $type);
	}
// ------------------------------
	/**#@+
	 * This function alter a table in DB.
	 * 
	 * <p>Where the 'alter' option in columns array define the action ADD, DROP, ALTER.</p>
	 *
	 * @access 	public
	 * @param 	string	table
	 * @param	array	values of table elements, [alter, name of column, type, null, auto_inc, unique, default, comment]
	 * @return	boolean
	 */
	public function alterTable($table, $columns){
		return $this->cms->alter_table($table, $columns);
	}
// ------------------------------
	/**#@+
	 * Checks the presense of a table in DB.
	 *
	 * @access 	public
	 * @param 	string	table
	 * @return	boolean
	 */
	public function checkTable($table){
		return $this->cms->check_table($table);
	}
// ------------------------------
	/**#@+
	 * Runs stored procedure on DB.
	 *
	 * <p>parameters: array(array(param1,value1),array(param2,value2))</p>
	 * @access 	public
	 * @param 	string spName	stored procedure
	 * @param	mixed params	parameter as 2 dimensional array
	 * @return	string			or false in case of error on db
	 */
	public function runStoredProcedure($spName, $params){
		return $this->cms->run_stored_proc($spName, $params);
	}
// ------------------------------
	/**#@+
	 * Check for access rights of the given menu id. In case of a provided
	 * specific userlevel of an Item this level will be used for verification.
	 * 
	 * Furthermore the method is prepared for support of application specific
	 * user profiles where the last digit 'z' is specified by the application.
	 * For control of application specific processing a kind of user level 
	 * verification could be provided.
	 * 
	 * <p>For access rights a string is used<br> 
	 *	xyz<br>
	 *	||+-- Level for special user profile rights (0..9)<br>
	 *	|+--- Level for update/edit (0..9)<br>
	 *	+---- Level for read (0..9)</p>
	 *
	 * <p>Each node shall have an user level which is defined on base of this
	 * string. The function proves for the wished right and in case no node 
	 * is used an own string (item level) could be provided.</p> 
	 *
	 * @access 	public
	 * @param 	int		id of menu item
	 * @param 	string	userlevel of current session
	 * @param 	string	access right checked for (read, write, create)
	 * @param 	string	optional itemlevel string (instead of ID)
	 * @param 	string	optional to be defined by application
	 * @return	boolean
	 */
	public function checkAccessRight($id, $ul, $acc='read', $ulOfItem=NULL, $aclSel=NULL){
		return $this->cms->write_access($id, $ul, $acc, $ulOfItem, $aclSel);
	}
// ------------------------------
	/**#@+
	 * Reserve a table entry for access protection (optimistic method)
	 * 
	 * <p>Special response method, if the reserve task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a session variable $_SESSION["ctc"] holds some further informations</p> 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctcReserveTable($table, $eId){		
		return $this->cms->ctc_reserve_table($table, $eId);		
	}			
// ------------------------------
	/**#@+
	 * Lock an access protected table (pessimistic method)
	 * 
	 * <p>Special response method, if the lock task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a the session variable $_SESSION["ctc"] holds some further informations</p> 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctcLockTable($table, $eId){		
		return $this->cms->ctc_lock_table($table, $eId);		
	}			
// ------------------------------
	/**#@+
	 * Check and Lock an a previously reserved table ID (optimistic method)
	 * 
	 * <p>Special response method, if the lock task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a session variable $_SESSION["ctc"] holds some further informations</p> 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctcCheckAndLockTable($table, $eId){		
		return $this->cms->ctc_check_table($table, $eId);		
	}			
// ------------------------------
	/**#@+
	 * UnLock an access protected table
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctcUnlockTable($table, $eId){		
		return $this->cms->ctc_unlock_table($table, $eId);
	}	
// ------------------------------
	/**#@+
	 * build a row of a dialog 
	 * 
 	 * @param string valueName
 	 * @param string value
 	 * @param string $valueGroup
 	 * @param string $valueDialogMethod
 	 * @param string $valueValidator
	 * @access public
	 * @return array
	 */
	public function dialogElement($valueName, $value, $valueGroup='set', $valueDialogMethod = 'input', $valueValidator = 'none'){
		return $this->cms->dialog_element($valueName, $value, $valueGroup, $valueDialogMethod, $valueValidator);
	}
// ------------------------------
	/**#@+
	 * build the header of a dialog
	 * 
	 * @param string $refName
	 * @param mixed $value
	 * @param string $action
	 * @access public
	 * @return array
	 */
	public function dialogHeader($refName, $value, $action=Null){
		return $this->cms->dialog_header($refName, $value, $action);
	}
	
}
/**#@+
 * Coding example for USER class enhancement
 */
//	require_once('iUserProf.php');
//	class USERPROF extends USER implements iUserProf{
//		
//		CONST VERSION			= '$Revision: 607 $';
//		CONST AUTHOR			= 'Jens Raabe';
//		CONST PACKAGE			= 'jrCmsUserProf';
//		CONST LAST_CH			= '$Date: 2011-09-20 07:58:51 +0200 (Di, 20 Sep 2011) $';
//		
//		// no own __construct/__destruct is used
//		
//	// ------------------------------
//		/**#@+
//		 * method which could be used to provide the access level (right)
//		 * for the third part of the access right string. The method is
//		 * called inside of the JRCMAPI:checkAccessRight.
//		 * 
//		 * According to the given ACL selector the access level digit
//		 * (0..9) must be returned as character.
//		 * 
//		 * @access	public
//		 * @param 	string	access control level selector
//		 * @return 	character
//		 */
//		public function upCheckAccessRight($aclSel){
//			//return max right level as dummy value
//			return '9';
//		}
//		
//	// ------------------------------
//		/**#@+
//		 * intended to replace the core function by an own list of data to be
//		 * retrieved for each user, e.g. with additional columns/properties
//		 * which are defined in a separate table.
//		 * The coding below is an example but applications should follow the
//		 * inline documentation/rules carefully. There are no verifications in the
//		 * core to speed up the performance.
//		 * 
//		 * @access	public
//		 * @param	integer	user Id
//		 * @param	string	user level only flag
//		 * @param	integer	start value of user list
//		 * @param	integer	number of users
//		 * @param	string	filter conditions
//		 * @return 	mixed Boolean, string (array), value (ul)
//		 */
//		public function upGetUser($uid = -1, $ul_only = NULL, $start = NULL, $limit = 1, $filter=NULL){
//			
//			//== do not change this part ==
//			if(isset($ul_only))
//				return $this->get_core_user($uid, $ul_only, $start, $limit, $filter);
//			else { 
//				$userdata = $this->get_core_user($uid, $ul_only, $start, $limit, $filter);
//				
//			//=====================================================
//			//== own functionality to enhance the retrieved data
//			//=====================================================
//				if(!isset($start)) {
//					// a single user is requested where the data could be
//					// enhanced by an own functionality
//					// --------------------------------
//					// conventions
//					// --------------------------------
//					// - first array of the userdata is used for the core
//					//   data itself
//					// - following arrays could be used for data provided
//					//   as dialog structure 
//					// - identifier used in dialogs must be defined in the 
//					//   project language file /lang/lang.<nn>
//					// --------------------------------
//					// maintenance of user profile data
//					// --------------------------------
//					// By using the functionality of the dialog as provided
//					// by the core (jrcms.js) different server files could 
//					// be addressed by the same dialog. This function has to
//					// be used for the maintenance. 
//					// Please have a look at jrcms.js class:JRCmsForm 
//	
//					$api = new JRCMSAPI();	
//					$userdata[] = $api->dialogElement('User_Name', 'Sandman ;-)', 'userProfDialog:jrCmsUserProf', 'input:jrCmsUserProf'); 
//					;
//				} else {
//					// a user list is requested where the data could be
//					// enhanced by own functionality
//					// this could be done either by an own DB request
//					// or by a postprocessing of the retrieved userdata
//					// ---
//					// assuming that the user maintenance isn't runtime
//					// critical the postprocessing is prefered
//					function alterUser(&$item, $key){
//						$item['Name'] = 'Sandman';
//					}
//					array_walk($userdata, 'alterUser');
//				}
//			//=====================================================
//			//== end of own functionality
//			//=====================================================
//				return $userdata;
//			}	
//		}
//	
//	// ------------------------------
//		/**#@+
//		 * method which could be used to set predefined session variables
//		 * depending on the user level or user profile.
//		 * This function is called instead of the USER:login method if
//		 * available.
//		 * 
//		 * @access	public
//		 * @param 	none
//		 * @return 	character
//		 */
//		public function upLoginWSessionParameter($name, $pass){
//			
//			//perform the login
//			if(($result = parent::login($name, $pass)) === True){
//				// based on the user data
//				// or profile data 
//				// the following session variables could be set
//				// --Xinha------------------------
//				//$_SESSION['allow_new_dir'] = true;
//				//$_SESSION['allow_delete'] = true;
//				//$_SESSION['images_subdir'] = '/exampleimages';
//				// -------------------------------
//			}
//			return $result;
//		}
//	}	
?>