<?php
/**
 * This Class module provides the data base access for all
 * menu, styles and content tables as well as generic data
 *
 * @package jrCMS: Class MENU
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: cMenu.php 607 2011-09-20 05:58:51Z jensraabe $
 * 
 * @copyright Copyright (c) 2011 Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iMenu.php');

class MENU extends DBInterface implements iMenu{
	
	CONST VERSION			= '$Revision: 607 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2011-09-20 07:58:51 +0200 (Di, 20 Sep 2011) $';
	
	// init the user interface
	public function __construct(){
		// ensure that possible construction routine will be executed
		parent::__construct();
		// look for user profile extension and merge the user class 		
		if( ($result=$this->get_single_value('g.getUserProf,g.getUserProfSrc', CONFIG::sql_tbl_gData.' AS g')) !== False) {
			if(isset($result[0]['getUserProf']) && isset($result[0]['getUserProfSrc'])){
				// we assume a user profile class is available following 
				// the conventiones as described in the example class jrcmsprof.php
				// the attribute 'getUserProfSrc' define the file including relative path
				// and 'getUserProf' define the class name itself
				// the user profile class is per definition an extended class
				// of the USER class
				require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.$result[0]['getUserProfSrc']);
				$this->user = new $result[0]['getUserProf'];
				return True;
			}	
		}
		$this->user = new USER();
	}
	public function __destruct(){
		$this->user = NULL;
	}
		
// ------------------------------

	/**
	 * Get all brunch data (lft, rgt) of the given node id.
	 *
	 * @access private
	 * @param int $link_id, int $rootId
	 * @return mixed Boolean, string (array)
	 */
	 
	private function get_brunch_info($id = NULL, $rootId = NULL, $menuDiv = 'div'){
		
		if(isset($id) && !empty($id)){

			$qdata = $this->get_single_value('rgt, lft', CONFIG::sql_tbl_menu,
								(($id != $rootId)
								?("id = ".addslashes((int)$id)." AND rootId = ".(int)$rootId." AND htmlDiv = '".$menuDiv."'")
								:("label = 'ROOT' AND rootId = ".(int)$rootId." AND htmlDiv = '".$menuDiv."'")));

			if(!empty($qdata) && is_array($qdata) && array_key_exists('0', $qdata )){
				return $qdata;
			}

		}
		return FALSE;	
	} // get_brunch_info

// ------------------------------

	/**
	 * Provides a list entry string (HTML) for a menu item.
	 * 
	 * @access private
	 * @param string $type, string $label, string $proc, string $span_class, int $rootId, int $id, int $sact
	 * @return string (HTML)
	 */
	 
	private function label_action($type, $label, $proc, $span_class, $id, $rootId=0, $sact=NULL, $cid=NULL){
		
		$sact_set = NULL;
		
		if (isset($sact) && $sact != 'none') {
			$sact_set = entities2char($sact);
		}
		
		if (isset($cid) && $cid != 'none') {
			$cid = entities2char($cid);
		} else 
			return false;
		
		$act = '';
		switch ($type) {
		case "js":
			$result = $this->get_single_value("content", CONFIG::sql_tbl_cnt, "cid = ".$cid);
			if (isset($result) && !empty($result[0]["content"]))
				// call a classic java script routine with parameters supplied in content
				$act = '<span id="'.$span_class.'_'.$id.'" class="'.$span_class.'" name="'.$label.'" onclick="'.$proc.'('.$result[0]["content"].'); return false;" '.$sact_set.'>'.$label.'</span>';
			else
				// call a classic java script routine
				$act = '<span id="'.$span_class.'_'.$id.'" class="'.$span_class.'" name="'.$label.'" onclick="'.$proc.'(); return false;" '.$sact_set.'>'.$label.'</span>';
			break;
		case "ajax":
		case "apage":
		case "aeditor":
			// Remember, that for special functions like AJAX routines the style sheet switch
			// require a separate function to be called. Per convention the styles are separated
			// by ';' in a defined sequence
			$result = $this->get_menu_style_id($id);
			if (isset($result)) {
				$css = $result;
			} else {
				$css = -1;
			}
			$act = '<span id="'.$span_class.'_'.$id.'" class="'.$span_class.'" name="'.$label.'" onclick="'.$proc.'({root:'.$rootId.',id:'.$id.',cssid:'.$css.'}, this); return false;" '.$sact_set.'>'.$label.'</span>';
			break;
		case "alink":
			/*$result = $this->get_menu_style_id($id);
			if (isset($result)) {
				$css = $result;
			} else {
				$css = -1;
			}*/
			//var_dump($cid);
			$result = $this->get_single_value("content", CONFIG::sql_tbl_cnt, "cid = ".$cid);
			if (isset($result))
				$url = $result[0]["content"]; // var_dump($result);
			else
				return false;

			$act = '<span id="'.$span_class.'_'.$id.'" class="'.$span_class.'" name="'.$label.'" onclick="window.open(\''.$url.'\'); return false;" '.$sact_set.'>'.$label.'</span>';
			break;
 		default: 
			$act = '<span id="'.$span_class.'_'.$id.'" class="'.$span_class.'" name="'.$label.'" onclick="alert(\'Not implemented '.$label.'\'); return false;">'.$label.'</span>';	
			
		}

		return $act;
	} // label_action

// ------------------------------

	/**
	 * Provides the content of a menu item.
	 * 
	 * @access private
	 * @param string $type, string $content, int $id, int $rootId, string $menuDiv, string $ul
	 * @return string (HTML)
	 */
	 
	private function label_content($type, $content, $id, $rootId, $menuDiv, $ul, $loadEditor=False){
		
		global $lang_php;
		
		$preview = (isset($lang_php['preview']))?($lang_php['preview']):("Preview");
		$submit = (isset($lang_php['save']))?($lang_php['save']):("Submit");
		
		//in case of horizontal menu the rootId is allways '1'
		//if horizontal menu item is processed
/*
 * This piece of code was problem for bug 2937001!
 * "A menu item that is configured as ajax.getImgShow from horizontal menu
	doesn't work. It seems in DOM that multiple testareas for xhina editor are
	pasted!"
	
		if (($rootId==1)&&($type=="ajax")){
			if ((strpos($content,"/") !== FALSE) && (strpos($content,"/")<2)) {
				// assume an internal page
				$type = "apage";
			} else {
				if ((strpos($content,"http") !== FALSE) && (strpos($content,"http")<2)) {
					// assume an external link
					$type = "alink";
				} else {
					// assume an editor page
					$type = "aeditor";
				}
			}
		}
*/
		$cnt = '';
		switch ($type) {
		//TODO:content still provided hopefully without side effects
		case "ajax":
			$cnt.= utf8_decode($content);
			break;
//		case "ifrm_ext":
//		case "link":
		case "alink":
			// call an external link in an iframe
			$cnt.= '<iframe  name="content" frameborder="0"';
			$cnt.= ' src="'.$content.'" class="iFContent">';
			$cnt.= 'Sorry, your Browser does not support IFRAMEs. The content is partly available as SiteMap links. </iframe>';
			break;
//		case "ifrm_int":
//		case "page":
		case "apage":
			// call a local link in an iframe
			$cnt.= '<iframe  name="content" frameborder="0"';
			$cnt.= ' src="'.$content.'" class="iFContent">';
			$cnt.= 'Sorry, your Browser does not support IFRAMEs. The content is partly available as SiteMap links. </iframe>';
			break;
//		case "ifrm_db":
			// process a HTML page from DB in an iframe
			break;
//		case "html_db":
//		case "editor":
		case "aeditor":
			// load a page from DB
//			if ($loadEditor && $menuDiv != CONFIG::divRContent && $this->write_access($id, $ul, 'write') && $this->write_access($id, $ul, 'write', CONFIG::editorLevel)) {
			if ($loadEditor && $this->write_access($id, $ul, 'write') && $this->write_access($id, $ul, 'write', CONFIG::editorLevel)) {
				// edit mode is assumed and parameter are set for usage of Xinha
				// A submit handler is used to trigger the DB upload
				$cnt.= '<form id="to_submit" name="myXinha" method="post" action="">';
				$cnt.= '<textarea id="myXinhaTextarea" name="myXinhaTextarea" rows="0" cols="0" style="width: 100%">';
				$cnt.= utf8_decode($content);
				$cnt.= '</textarea>';
				$cnt.= '<input type="hidden" name="root" id="root" value="'.$rootId.'">';
				$cnt.= '<input type="hidden" name="id" id="id" value="'.$id.'">';
				$cnt.= '<input type="hidden" name="div" id="div" value="'.$menuDiv.'">';
				$cnt.= '<input type="hidden" name="a" id="a" value="sacnt">';
				$cnt.= '</form>';
				
			} else {
				// load index with menu parameter
				$cnt.= utf8_decode($content);
			}
			break;
//		case "gallery":
//			// optional feature
//			if (CONFIG::spgm != '') {
//				require('.'.CONFIG::spgm.'/spgm.php');
//			}
//			break;
//		case "load_t":
//			// provide content as reference
//			$cnt.= $content;
//			break;
		default:
			$cnt.= '<h1>Wrong page Type: '.$type.' with rootId: '.$rootId.', id: '.$id.', and div: '.$menuDiv.' processed !</h1>';
		}
		return $cnt;
	} //label_content
	
// ------------------------------

	/**
	 * Provides the array of buttons to be used for standard content (editor)
	 * 
	 * @access private
	 * @param string $type, string $content, int $id, int $rootId, string $ul
	 * @return string
	 */
	 
	private function standard_tasks($type, $content, $id, $rootId, $menuDiv, $ul, $showEditorTask = False){
		
		global $lang_php;
		
		$preview = (isset($lang_php['preview']))?($lang_php['preview']):("Preview");
		$submit = (isset($lang_php['save']))?($lang_php['save']):("Submit");
		
		//in case of horizontal menu the rootId is allways '1'
		//if horizontal menu item is processed
		if (($rootId==1)&&($type=="ajax")){
			if ((strpos($content,"/") !== FALSE) && (strpos($content,"/")<2)) {
				// assume an internal page
				$type = "apage";
			} else {
				if ((strpos($content,"http") !== FALSE) && (strpos($content,"http")<2)) {
					// assume an external link
					$type = "alink";
				} else {
					// assume an editor page
					$type = "aeditor";
				}
			}
		}

		switch ($type) {
		case "html_db":
		case "editor":
		case "aeditor":
			// since we have an own save and preview button in the editor we do not have standard
			// buttons, but for future use we still have this function
//			$buttonlst = NULL;
			$buttonlst = $showEditorTask 
							? array(1 => "jrCMS.getContent({root:".$rootId.",id:".$id.",div:'".$menuDiv."', a:'getcntEditor'});return false;"
									,10 => "jrCmsEditor.getHelpDoc();return false;")
							: array(10 => "jrCmsEditor.getHelpDoc();return false;");
//							: NULL;
			break;
		default:
			$buttonlst = NULL;
		}
	
		return $buttonlst;
	}
	
// ------------------------------
// menu: types and procedures	
// ------------------------------	

	/**
	 * Get all typs and procedures which could be used by menu items
	 * 
	 * @access public
	 * @param string $userlevel, string $type
	 * @return mixed Boolean, string (array)
	 */
	public function get_proc_types_and_procs($userlevel='000', $type=NULL){
		
		$result = $this->get_array(
								((isset($type))
								?("ptypes.type, pr.procName, pr.classID, xtp.xtpID ")
								:("ptypes.*, ul.ul ")),
								CONFIG::sql_tbl_pTypes." AS ptypes 
								INNER JOIN ".CONFIG::sql_tbl_ul." AS ul ON ul.ulid=ptypes.ulid ".
								((isset($type))
								?("INNER JOIN ".CONFIG::sql_tbl_xTP." AS xtp ON xtp.pTypId=ptypes.pTypId
									 INNER JOIN ".CONFIG::sql_tbl_proc." AS pr ON xtp.procId=pr.procId "):("")),
								"LEFT(ul.ul,1) <= ".substr($userlevel,0,1)." AND LEFT(ul.ul,1) >= 0 ".
								((isset($type))
								?("AND ptypes.type = '".$type."' "):("")),
								NULL, NULL, NULL,
								"ORDER BY type ASC");

		if(is_array($result)) return $result;	

	} // getProcTypsAndProcs
	
// ------------------------------
	
	/**
	 * insert a new procedure and establish relation to xTP
	 * 
	 * @access public
	 * @param string $userlevel, string $type
	 * @return mixed Boolean, string (array)
	 */
	public function insert_procedure($procName, $classId, $type, $comment=NULL){
		
		//check for presence to prevent double entries
		if(($result = $this->get_single_value('*', CONFIG::sql_tbl_proc, 
								"procName = '".addslashes((string)$procName)."' AND ".
								"classId = ".addslashes((int)$classId))) !== False && empty($result)){

			if(($procId = $this->insert(CONFIG::sql_tbl_proc,
								"procName, classId, comment",
								"'".addslashes((string)$procName)."',".addslashes((int)$classId).",'".
								(isset($comment)?(addslashes((string)$comment)):('none'))."'", True)) !== False) {
				//retrieve the type ID
				$result = $this->get_single_value('pTypId', CONFIG::sql_tbl_pTypes, "type = '".addslashes((string)$type)."'");
				if(isset($result) && is_array($result) && !empty($result))
					if(($xtpID = $this->insert(CONFIG::sql_tbl_xTP,
										"pTypId, procId, comment",
										addslashes((int)$result[0]['pTypId']).",".$procId.",'".
										(isset($comment)?(addslashes((string)$comment)):('none'))."'", True)) !== False)
						//return $xtpID;
						return True;
						
				//remove previous inserted Procedure
				$result = $this->delete_single_value('*',CONFIG::sql_tbl_proc,'procId = '.addslashes((int)$procId));										
			}
			return False;
		}
		return True;
	} // insertProcedure
		
// ------------------------------
// User Access
// ------------------------------	

	/**
	 * Get userlevel of node
	 * 
	 * @access public
	 * @param int $id
	 * @return int (userlevel)
	 */

	public function get_ulevel_of_node($id){
		
		if (isset($id)) {
		
//TODO:
// Performance optimiziation
// because getMenuItemDate retrieve the whole item incl. content
//			$result = $this->get_menu_item_data($id, NULL, '', CONFIG::webmasterLevel);
//			return (is_array($result))?($result[0]['ul']):(False);
// better solution:

			$result = $this->get_menu_tree_array(NULL, CONFIG::webmasterLevel, '', $id, True);
			return (is_array($result))?($result[0]['ul']):(False);
			
// could be further improved by retrieving a single value only

		}
		return FALSE;

	} // get_ulevel_of_node

// ------------------------------	

	/**
	 * Get userlevel of node
	 *	xyz
	 *	||+-- Level for create/delete/profile (0..9)
	 *	|+--- Level for update/edit (0..9)
	 *	+---- Level for read (0..9)
	 * 
	 * @access public
	 * @param int $id, string $ul, string $acc, string $ulOfNode
	 * @return int (userlevel)
	 */

	public function write_access($id, $ul, $acc='read', $ulOfNode=NULL, $aclSel=NULL){
		
		//retrieve node rights
		if (!isset($ulOfNode)) $ulOfNode=$this->get_ulevel_of_node($id);
		//now determine the rights
		if ($ulOfNode !== False) {
			//prevent wrong processing in old DBs
			if (strLen($ul)==1 || strLen($ul)==2) $ul.="000";
			if (strLen($ulOfNode)==1 || strLen($ulOfNode)==2) $ulOfNode.="0";
			
			//if there is a specific/dynamic right required and 
			//a user profile functionality is available modify the 
			//string for the 'z' column
			if(isset($aclSel)) {
				if(isset($this->user) && method_exists($this->user,'upCheckAccessRight'))
					$ulOfNode[2] = $this->user->upCheckAccessRight($aclSel);
			}

			//now determine the access right	
			switch ($acc) {
			case "read":
				//print_r("read: ".$ul."  ".$ulOfNode);
				if (intval(substr($ul,0,1)) >= intval(substr($ulOfNode,0,1))) return True;
				break;
			case "write":
				//print_r("write: ".$ul."  ".$ulOfNode);
				if ((intval(substr($ul,1,1)) >= intval(substr($ulOfNode,1,1)))&& (intval(substr($ul,1,1))>0)) return True;
				break;
			default:
				//print_r("create/delete/profile: ".$ul."  ".$ulOfNode);
				if ((intval(substr($ul,2,1)) >= intval(substr($ulOfNode,2,1)))&& (intval(substr($ul,2,1))>0)) return True;
				break;
			}
		}
		return False;
	} // write_access

// ------------------------------
// Menu	
// ------------------------------	
	
	/**
	 * Get all brunches of the provided brunchid, that can be the brunch root only or the
	 * whole brunch. The param "userlevel" selects menu items by userlevel provided as 
	 * 3 character string where as simple solution the first character is proved only 
	 * (see write_access).
	 * 
	 * @access public
	 * @param int $brunchId, string $menuDiv, int $rootId, string $userlevel
	 * @return mixed Boolean, string (array)
	 */
	public function get_menu_brunch($brunchId=-1, $menuDiv=NULL, $rootId=NULL, $userlevel=NULL, $getRoot=FALSE){
		if(isset($brunchId) && !empty($brunchId)) {
			$result = $this->get_menu_tree_array($rootId, $userlevel, $menuDiv, $brunchId, $getRoot);
			if(is_array($result)) 
				return $result;
		}
		return FALSE;	
	} // get_menu_brunch

// ------------------------------	
	
	/**
	 * Provides the menu as list
	 * 
	 * @access public
	 * @param int $brunchId, string $menuDiv, int $rootId, string $userlevel
	 * @return string (html)
	 */
	public function get_menu_html($brunchId = -1, $menuDiv = 'div', $rootId = NULL, $userlevel = NULL, $cmpMode = false){

		//IE compatibility mode
		$_LI_CM = '';	//dynamic menu by java script
		$_LI_CMIDs = '';	//list of IDs
		
		//retrieve menu
		$menulst = $this->get_menu_brunch($brunchId, $menuDiv, $rootId, $userlevel);

		if(!isset($rootId)) $rootId = -1;
		//variable holds the end tags
		$endTag = array("ctr" => 0, "tag" => "</ul></li>");
		//array pointer used to reference next element in array
		$ptr = -1;
		if(is_array($menulst) && !empty($menulst)){
			//build up the list
			$strHtml = '<ul>';
			foreach($menulst AS $menu){
				//per convention the first entry with ID=1 (HOME) will be not processed.
				if($menu['id'] > 2) {
					$ptr++;
					//determine action
					//Attention: do not change the class "menu", which is used for additional events
	 				$act = $this->label_action($menu['type'], $menu['label'], $menu['procName'], "menu", $menu['id'], $rootId, $menu['sact'], $menu['cid']);
					//write list item, if needed for compatibility mode a script will be assigned later 
					$strHtml.= (!$cmpMode)?('<li>'.$act):('<li ##li##>'.$act);
					//open a new sublist 
					if($menu['CHILDS'] > 0){
						//list formatted by styles only
						if(!$cmpMode) {
							$strHtml.= '<ul>';
						//list formatted by using jscript (e.g. IE compatibility)
						} else {
							$strHtml.= '<ul id="sub_ul'.$menu['id'].'">';
							//I use a placeholder for the list of Ids
							if ($menu['LEVEL'] <= 1) $_LI_CMIDs.= ($_LI_CMIDs!='')?(',#sub_ul'.$menu['id']):('#sub_ul'.$menu['id']);
							$_LI_CM = ($menu['LEVEL'] <= 1)?(' onmouseover="jrCMS.showMenuId(\'sub_ul'.$menu['id'].'\',##id##);"'):(' onmouseover="jrCMS.showMenuId(\'sub_ul'.$menu['id'].'\');"');
							//TODO:AA $strHtml = str_replace('##li##',$_LI_CM,$strHtml);
						}
						$endTag["ctr"] = $endTag["ctr"] + 1;
					//close list
					} else {
						//Level is changing and end tag pending, close the sublist
						if ($endTag != '' && (($ptr+1 <= COUNT($menulst)-1) && ($menulst[$ptr+1]['LEVEL']!= $menu['LEVEL']))){
							for ($i = 1; $i <= ($menu['LEVEL'] - $menulst[$ptr+1]['LEVEL']); $i++) {
    							$strHtml.=$endTag["tag"];
    							$endTag["ctr"] = $endTag["ctr"] -1; 
							}
						//Level remain unchanged
						} else {
							//consider as last element a pending end tag
							if(($endTag["ctr"] > 0) && ($ptr == COUNT($menulst)-1)){
								for ($i = 1; $i <= $endTag["ctr"]; $i++) {
    								$strHtml.=$endTag["tag"];
								}
								$endTag["ctr"] = 0;
							} else
								$strHtml.='</li>';
						}
						//TODO:BB
						$_LI_CM = ($cmpMode && $menu['LEVEL'] <= 1)?(' onmouseover="jrCMS.showMenuId(\'sub_ul'.$menu['id'].'\',##id##);"'):('');
					}
					//TODO:BB
					$strHtml = (!$cmpMode)?($strHtml):(str_replace('##li##',$_LI_CM,$strHtml));
				}
			}
			$strHtml.= '</ul>';
			if($cmpMode) {
				if(isset($_LI_CMIDs) && $_LI_CMIDs != '') $strHtml = $strHtml.'<style type="text/css">'.$_LI_CMIDs.'{display:none;}</style>';
				$_LI_CMIDs = str_replace(',#','\',#',$_LI_CMIDs);
				$strHtml = ($_LI_CMIDs!='')?(str_replace('##id##','['.str_replace('#','\'',$_LI_CMIDs).'\']',$strHtml)):(str_replace(',##id##','',$strHtml));
			}
		}
		
		return isset($strHtml)?($strHtml):('');
		
	} // get_menu_html

// ------------------------------	

	/**
	 * Get a single menu item inclusiv content and ROOT.
	 * 
	 * @access public
	 * @param int $id
	 * @return mixed Boolean, string (array)
	 */
	 
	public function get_menu_item_data($id, $label=NULL, $menuDiv = 'div', $userLevel = '000'){

		// get all item data
		if(isset($id) && $id > 0 && !isset($label)) {
			$result = $this->get_menu_tree_array(NULL, $userLevel, $menuDiv, $id, True);
			if(is_array($result)) {
				if(($content = $this->get_menu_node_content($id))!==FALSE) $result[0]['content'] = $content[0]['content'];
				return $result;
			}
		} else if (isset($label)){
			//retrieve data for a label
			$result = $this->get_menu_item_ids($label,$userLevel,$menuDiv);
			if(is_array($result)) {
				if(($content = $this->get_menu_node_content($result[0]['id']))!==FALSE) $result[0]['content'] = $content[0]['content'];
				return $result;
			}
		}
		return FALSE;
	} // get_menu_item_data

// ------------------------------	

	/**
	 * retrieve all root elements of a menu tree
	 * 
	 * @access public
	 * @param int $rootId, string $menuDiv
	 * @return Boolean
	 */
	public function get_menu_root($menuDiv='div', $id = NULL, $label = NULL){
		switch($menuDiv) {
			case CONFIG::divHMenu:

				return $this->get_single_value("i.id AS rootId, 'Menu' AS label",
												CONFIG::sql_tbl_menu." AS i",
												"i.label LIKE 'ROOT' AND i.htmlDiv LIKE '".$menuDiv."'");
				break;
			case CONFIG::divVMenu:
			case CONFIG::divRMenu:
			case CONFIG::divPupMenu:

				return (isset($id)&&!isset($label)) 
						? ($this->get_single_value("i.id AS rootId, i.label",
												CONFIG::sql_tbl_menu." AS i",
												"i.rootId = ".$id." AND i.label LIKE 'ROOT' AND i.htmlDiv LIKE '".$menuDiv."'"))
				 		: ($this->get_single_value("i.id AS rootId, i.label".((isset($label))?(', r.id as parentId'):('')),
												CONFIG::sql_tbl_menu." AS i INNER JOIN ".CONFIG::sql_tbl_menu." AS r ON r.label LIKE 'ROOT' AND r.htmlDiv LIKE '".$menuDiv."'",
												"i.id = r.rootId".((isset($label))?(" AND i.label LIKE '".$label."'"):(''))));
				break;
			case 'allRoots':

				return (isset($id))
						? ($this->get_single_value("i.id AS rootId, i.htmlDiv",
												CONFIG::sql_tbl_menu." AS i",
												"i.label LIKE 'ROOT' AND i.rootId =".$id))
						: NULL;
				break;
			default:
				if(isset($id)){

					return $this->get_single_value("i.rootId, i.htmlDiv, i.label",
													CONFIG::sql_tbl_menu." AS i",
													"i.id = ".$id);
					
				}
				return NULL;
		}
	} // get_menu_root

// ------------------------------	

	/**
	 * retrieve the parent of an element
	 * 
	 * @access public
	 * @param int $id, string $menuDiv
	 * @return Boolean
	 */
	public function get_menu_parent_id($id, $menuDiv){

		switch($menuDiv) {
			case CONFIG::divHMenu:
				//(1) set the root parent menu - level above
				$menuparent = array( 0 => array('id' => 1, 'label' => 'Menu', 'htmlDiv' => CONFIG::divHMenu));
				//(2) retrieve the own values
				$ownparents = $this->get_single_value(
								"m.id, m.label, m.htmlDiv",
								CONFIG::sql_tbl_menu." AS m",
								"m.id = ".$id);
				break;
			default:
				//(1) retrieve the root parent menu - level above
				//SELECT iw.id, iw.label FROM `web_menu` AS w
				//INNER JOIN `web_menu` AS iw ON iw.id = w.rootId
				//WHERE w.id = $id
				$menuparent = $this->get_single_value(
								"iw.id, iw.label, iw.htmlDiv",
								CONFIG::sql_tbl_menu." AS w INNER JOIN ".CONFIG::sql_tbl_menu." AS iw ON iw.id = w.rootId",
								"w.id = ".$id);
				//(2) retrieve the own parents (of sub tree)
				//SELECT iwr.id, iwr.label FROM `web_menu` AS w
				//INNER JOIN `web_menu` AS iwr ON iwr.rootId = w.rootId
				//AND iwr.label NOT LIKE 'ROOT' AND iwr.htmlDiv = '$menuDiv'
				//WHERE w.id = $id ORDER BY iwr.id ASC
				$ownparents = $this->get_array(
							"iwr.id, iwr.label, iwr.htmlDiv",
							CONFIG::sql_tbl_menu." w INNER JOIN ".CONFIG::sql_tbl_menu." AS iwr ON iwr.rootId = w.rootId
							AND iwr.label NOT LIKE 'ROOT' AND iwr.htmlDiv = '".$menuDiv."'",
							"w.id = ".$id,
							NULL, NULL, NULL,
							" ORDER BY iwr.id ASC");
		}
		return (isset($ownparents) && isset($menuparent))
				?(array_merge($menuparent,$ownparents))
				:((isset($menuparent))?($menuparent):(NULL));
	} // get_menu_parent_id
	
// ------------------------------	

	/**
	 * retrieve the complete parent path of a menu item
	 * 
	 * @access public
	 * @param int $id, string $menuDiv
	 * @return string
	 */
	public function get_menu_path_html($id, $menuDiv){
		
		$menuPathArray = $this->get_menu_parent_id($id, $menuDiv);
		$menuPath = '';
		switch($menuDiv) {
			case CONFIG::divHMenu:
				foreach($menuPathArray AS &$value){
					if($value['id']<$id) $menuPath .= '/'.$value['label'];
				}
				break;
			case CONFIG::divVMenu:
				foreach($menuPathArray AS $key => $value){
					if($value['id']<$id) $menuPath .= '/'.$value['label'].(($key==0)?('/'):(''));
				}
				break;
			case CONFIG::divRMenu:
			case CONFIG::divPupMenu:
			case CONFIG::divRContent:
				//retrieve the upper menu first
				$menuUpperPathArray = $this->get_menu_parent_id($menuPathArray[0]['id'], $menuDiv);
				foreach($menuUpperPathArray AS $key => $value){
					if($value['id']<$id) $menuPath .= '/'.$value['label'].(($key==0)?('/'):(''));
				}
				$menuPath .= '/';
				foreach($menuPathArray AS $key => $value){
					if($value['id']<$id && $key > 0) $menuPath .= '/'.$value['label'];
				}
			default:
				break;
		}
		return ($menuPath!='')?('&gt;'.$menuPath.'&lt;'):('');
		
	} // get_menu_path_html

// ------------------------------	

	/**
	 * Insert a new link-record for a given root id
	 * 
	 * @access public
	 * @param int $rootId, string $menuDiv
	 * @return Boolean
	 */
	public function insert_menu_root($rootId = 0, $menuDiv='div'){

		// create menu item
		$lang = (isset($_SESSION["language"])) ? ($_SESSION["language"]):("de");
		return $this->insert(CONFIG::sql_tbl_menu, 
							"parentId, rootId, label, lft, rgt, htmlDiv, ulid, cssid, lang, cid, xtpID, said",
							"0,".addslashes((int)$rootId).",'ROOT',1, 2,'".addslashes((string)$menuDiv)."',1,0,
							'".addslashes((string)$lang)."',1, 1, 1");
	} // insert_menu_root

// ------------------------------	

	/**
	 * Insert a new link-record for a given root id
	 * 
	 * @access public
	 * @param string $nodeName, int $parentId, int $rootId, string $menuDiv
	 * @return Boolean
	 */
	public function insert_menu_node($nodeName, $parentId = NULL, $rootId = 0, $menuDiv='div'){

		//create a new menu node
		if(isset($nodeName) && !empty($nodeName) && !empty($parentId)){
			
			$link_data = $this->get_brunch_info($parentId, $rootId, $menuDiv);
			if(isset($link_data) && is_array($link_data)){
				// (1) create empty content
				if(($cid = $this->insert(CONFIG::sql_tbl_cnt, 'content', "'".addslashes('initial content for '.$nodeName)."'", True)) !== False) {
					// (2) create an empty css entry
					if(($cssid = $this->insert_style_data()) !== False) {
						// (3) create an special action entry
						if(($said = $this->insert_special_action_data()) !== False) {
							// --------------------
							// (4) create menu item
							// --------------------
							$lang = (isset($_SESSION["language"])) ? ($_SESSION["language"]):("de");
							// shift left
							$result = $this->update_single_value(CONFIG::sql_tbl_menu,
													'lft = lft + 2',
													"lft > '".$link_data[0]["rgt"]."'
													 AND rgt >= '".$link_data[0]["rgt"]."'
													 AND rootId = ".(int)$rootId." AND htmlDiv= '".$menuDiv."'");
							// shift right
							$result&= $this->update_single_value(CONFIG::sql_tbl_menu,
													'rgt = rgt + 2',
													"rgt >=".$link_data[0]["rgt"]."
													 AND rootId = ".(int)$rootId." AND htmlDiv= '".$menuDiv."'");

							if($result) return $this->insert(CONFIG::sql_tbl_menu, 
													"parentId, rootId, label, lft, rgt, htmlDiv, ulid, cssid, lang, cid, xtpID, said",
													addslashes((int)$parentId).",".addslashes((int)$rootId).",'".addslashes((string)$nodeName)."',
													".$link_data[0]['rgt'].",".($link_data[0]["rgt"]+1).",'".addslashes((string)$menuDiv)."',
													1,".addslashes((int)$cssid).",'".addslashes((string)$lang)."',".addslashes((int)$cid).",
													3,".addslashes((int)$said), True);
						}
						//TODO:
						//else delete $cssid, $cid
					}
					//TODO:
					//else delete $cid
				}
			}
		}
		return False;
	} // insert_menu_node

// ------------------------------
	
	/**
	 * initialize a complete menu entry to be used in init functions
	 * the functions is intended as separate routine to prevent recursive
	 * calls of insert_menu_node
	 * 
	 * @access public
	 * @param string $label, string $parentLabel, string $menuDiv, string $xtp
	 * @return Boolean
	 */
	
	public function insert_menu_item($label, $parentLabel, $menuDiv = NULL, $subdivs = NULL, $xtp = NULL){
		
		//TODO: ATTENTION - see functionality of jrcmsmenu:insert_html_menu
		if(!empty($label) && !empty($parentLabel) && isset($menuDiv)){
			//retrieve parent menu data
			if (($result = $this->get_menu_root($menuDiv, NULL, $parentLabel)) != NULL) 
				if(($res = $this->insert_menu_node($label, $result[0]['parentId'], $result[0]['rootId'], $menuDiv)) !== False) 
					$res = insert_sub_menu_root($menuDiv, $res, $subdivs);
			
			if($result !== False && $res !== False && isset($xtp))
					;//update procedure
		}
		return (isset($result) && $result !== False)?(True):(False);
	} //initMenuNode

// ------------------------------
	
	/**
	 * initialize the root elements of a menu entry 
	 * 
	 * @access public
	 * @param string $menuDiv, int, $rootId, array $subdivs
	 * @return Boolean
	 */
	
	public function insert_sub_menu_root($menuDiv, $rootId, $subdivs = NULL){
		
		switch($menuDiv){
			case CONFIG::divHMenu:
				$result = $this->insert_menu_root($rootId, CONFIG::divVMenu);
				if(isset($subdivs) && in_array(CONFIG::divRContent,$subdivs)) $result &= $this->insert_menu_root($rootId, CONFIG::divRContent);
				//TODO: if(isset($subDiv) && in_array(CONFIG::divPupMenu,$subDiv)) $result &= $m->insert_menu_root($result, CONFIG::divPupMenu);
				break;
			case CONFIG::divVMenu:
				$result = True;
				if(isset($subdivs) && in_array(CONFIG::divRMenu,$subdivs)) $result &= $this->insert_menu_root($rootId, CONFIG::divRMenu);
				if(isset($subdivs) && in_array(CONFIG::divRContent,$subdivs)) $result &= $this->insert_menu_root($rootId, CONFIG::divRContent);
				if(isset($subdivs) && in_array(CONFIG::divPupMenu,$subdivs)) $result &= $this->insert_menu_root($rootId, CONFIG::divPupMenu);
				break;
			case CONFIG::divPupMenu:
				//TODO: Wenn divPupMenu, dann special action fr ersten Eintrag anlegen!
				break;
			default:
				$result = False;
		}
		return $result;
	}
	
// ------------------------------

	/**
	 * Delete a node id and there childs
	 * 
	 * @access public
	 * @param int $id, int $rootId
	 * @return Boolean
	 */
	public function delete_menu_node($id, $rootId, $menuDiv = NULL){
		
		$result = False;
		if (($link_data = $this->get_brunch_info($id, $rootId, $menuDiv)) !== False) {
			// (1) delete content		
			if ($this->delete_single_value(
								CONFIG::sql_tbl_cnt.".*", 
								CONFIG::sql_tbl_cnt." 
								INNER JOIN ".CONFIG::sql_tbl_menu." ON 
								lft BETWEEN '".(int)$link_data[0]['lft']."' AND '".(int)$link_data[0]['rgt']."'
					 			AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'",
								CONFIG::sql_tbl_cnt.".cid = ".CONFIG::sql_tbl_menu.".cid
								AND ".CONFIG::sql_tbl_cnt.".cid > 1") !== False) {
				// (2) delete css
				if($this->delete_single_value(
								CONFIG::sql_tbl_styles.".*", 
								CONFIG::sql_tbl_styles."
								INNER JOIN ".CONFIG::sql_tbl_menu." ON 
								lft BETWEEN '".(int)$link_data[0]['lft']."' AND '".(int)$link_data[0]['rgt']."'
					 			AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'",
								CONFIG::sql_tbl_styles.".cssid = ".CONFIG::sql_tbl_menu.".cssid
								AND ".CONFIG::sql_tbl_styles.".cssid > 1") !== False) {
					// (3) delete saction					
					if($this->delete_single_value(
								CONFIG::sql_tbl_sact.".*", 
								CONFIG::sql_tbl_sact."
								INNER JOIN ".CONFIG::sql_tbl_menu." ON 
								lft BETWEEN '".(int)$link_data[0]['lft']."' AND '".(int)$link_data[0]['rgt']."'
					 			AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'",
								CONFIG::sql_tbl_sact.".said = ".CONFIG::sql_tbl_menu.".said
								AND ".CONFIG::sql_tbl_sact.".said > 1") !== False) {
						// (4) delete item
						if($this->delete_single_value(
								CONFIG::sql_tbl_menu.".*", 
								CONFIG::sql_tbl_menu, 
								"lft BETWEEN '".(int)$link_data[0]['lft']."' AND '".(int)$link_data[0]['rgt']."'
								 AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'") !== False) {
							$result = $this->update_single_value(
								CONFIG::sql_tbl_menu,
								"lft =  lft - ".($link_data[0]['rgt']-$link_data[0]['lft']+1),
								"lft > '".$link_data[0]['rgt']."' AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'");
							$result&= $this->update_single_value(
								CONFIG::sql_tbl_menu,
								"rgt = rgt - ".($link_data[0]['rgt']-$link_data[0]['lft']+1),
								"rgt > '".$link_data[0]['rgt']."' AND rootId = '".$rootId."' AND htmlDiv= '".$menuDiv."'");
								
							if((boolean)$result === TRUE){
								switch($menuDiv){
									case CONFIG::divHMenu:
										$data = $this->get_menu_brunch(-1, CONFIG::divVMenu, $id, '999', True);
										//assumed the first element is always a ROOT
										foreach($data as &$menu) {
											if($menu['label'] != 'ROOT'){
												$dr = $this->get_menu_brunch(-1, CONFIG::divRMenu, $menu['id'], '999', True);
												$str.= $this->delete_menu_node($dr[0]['id'], $menu['id'], CONFIG::divRMenu);
												$dp = $this->get_menu_brunch(-1, CONFIG::divPupMenu, $menu['id'], '999', True);
												$str.= $this->delete_menu_node($dp[0]['id'], $menu['id'], CONFIG::divPupMenu);
												$dp = $this->get_menu_brunch(-1, CONFIG::divRContent, $menu['id'], '999', True);
												$str.= $this->delete_menu_node($dp[0]['id'], $menu['id'], CONFIG::divRContent);
											}
										}
										$result&= $this->delete_menu_node($data[0]['id'], $id, CONFIG::divVMenu);
										break;
									case CONFIG::divVMenu:
										$data = $this->get_menu_brunch(-1, CONFIG::divRMenu, $id, '999', True);
										$result&= $this->delete_menu_node($data[0]['id'], $id, CONFIG::divRMenu);
										$data = $this->get_menu_brunch(-1, CONFIG::divPupMenu, $id, '999', True);
										$result&= $this->delete_menu_node($data[0]['id'], $id, CONFIG::divPupMenu);
										$data = $this->get_menu_brunch(-1, CONFIG::divRContent, $id, '999', True);
										$result&= $this->delete_menu_node($data[0]['id'], $id, CONFIG::divRContent);
										break;
									default:
										break;
								}
							}
						}
					}
				}	
			}
			return $result;
		}
	} // delete_menu_node
	
// ------------------------------

	/**
	 * TODO
	 * Move ids or brunches, but not to the next level.
	 * 
	 * @access public
	 * @param int $id, int $rootId, string $direction
	 * @return Boolean
	 */
	public function move_menu_node($id, $rootId=NULL, $menuDiv=NULL, $direction){
		
		if(isset($id) && isset($rootId) && isset($menuDiv)){
			
			//TODO not supported now, do nothing
			if($direction == "right" || $direction == "left") return True;

			if($direction == "down"){

				$result = $this->get_single_value("(n1.rgt +1) AS L_lft, n2.id as SITE_ID",
												CONFIG::sql_tbl_menu." AS n1, ".CONFIG::sql_tbl_menu." AS n2",
												"n1.id = '".(int)$id."' AND n2.lft = (n1.rgt+1)	
												AND n1.rootId = ".(int)$rootId." AND n2.rootId = ".(int)$rootId."
												AND n1.htmlDiv = '".$menuDiv."' AND n2.htmlDiv = '".$menuDiv."'");

				// determine new id
				if(is_array($result) && array_key_exists('SITE_ID', $result[0])) $id = $result[0]['SITE_ID'];
				// no move down possible
				else return False;
			}
				
			$result = $this->get_single_value( "n1.label n1label, n1.lft n1lft, n1.rgt n1rgt,
												n2.label n2label, n2.lft n2lft, n2.rgt n2rgt",
												CONFIG::sql_tbl_menu." AS n1
												LEFT OUTER JOIN
													".CONFIG::sql_tbl_menu." AS n2
													ON (n1.lft = (n2.rgt+1) AND n1.rgt > n2.rgt)",
												"n1.id = '".(int)$id."'
												AND n1.rootId = ".(int)$rootId." AND n2.rootId = ".(int)$rootId."
												AND n1.htmlDiv = '".$menuDiv."' AND n2.htmlDiv = '".$menuDiv."'");

			//if no space left
			if (!$result[0]["n2lft"]) return FALSE;

			$decr = (($result[0]["n1lft"]-$result[0]["n2lft"]));
			$inc  = (($result[0]["n1rgt"]-$result[0]["n2rgt"]));

// Clemens
		/*replace UPDATE query with integrated IF statement, that is not supported on MS SQL*/
			$success = True;
			$menus = $this->get_array("id, label, lft, rgt", "web_menu", "lft >= ".$result[0]["n2lft"]." AND rgt<=".$result[0]["n1rgt"]."
				AND rootId = ".(int)$rootId." AND htmlDiv= '".$menuDiv."'", NULL, NULL, NULL);

			foreach($menus AS $menu){
				//var_dump ($menu["id"]);
				//var_dump ($menu["lft"]);
				//var_dump ($menu["rgt"]);
				if ( $menu["lft"] < $result[0]["n1lft"] )
					$lft = $menu["lft"] + $inc;
				else
					$lft = $menu["lft"] - $decr;
				if ( $menu["rgt"] < $result[0]["n1lft"] )
					$rgt = $menu["rgt"] + $inc;
				else
					$rgt = $menu["rgt"] - $decr;
					
				$success &= $this->update_single_value(
					CONFIG::sql_tbl_menu,
					"lft=".$lft.", rgt=".$rgt,
					"id = ".$menu["id"]
				);
				//var_dump ($lft);
				//var_dump ($rgt);

			}

			return $success;
// Clemens Ende

//			return $this->update_single_value(
//												CONFIG::sql_tbl_menu,
//												"lft=lft + IF(lft<".$result[0]["n1lft"].",".$inc.",-".$decr."),
//												rgt=rgt + IF(rgt<".$result[0]["n1lft"].",".$inc.",-".$decr.")",
//												"lft >= ".$result[0]["n2lft"]." AND rgt<=".$result[0]["n1rgt"]."
//												AND rootId = ".(int)$rootId." AND htmlDiv= '".$menuDiv."'"
//												// Clemens
//												, NULL, True);
		}
		return False;
	} // move_menu_node

// ------------------------------

	/**
	 * update a node id and there childs
	 * 
	 * @access public
	 * @param int (site id)
	 * @param string (db-table)
	 * @return Boolean
	 */
	public function update_menu_node($id, $rootId = NULL, $menuDiv = NULL, $ul = NULL, $label = NULL, $xtpId = NULL ){
		
		if(isset($id) && isset($rootId) && isset($menuDiv)) {
			$result = (isset($label))
						?($this->update_single_value(CONFIG::sql_tbl_menu,
													 "label = '".$label."'"
													.((isset($xtpId))?(", xtpID = ".$xtpId):(""))
													,"id = ".$id))
						:(True);
			// check for update of user level
			if($result === True && isset($ul)){
				// retrieve the new ulid
				if (($result = $this->get_single_value("ulid", CONFIG::sql_tbl_ul, "ul=".$ul)) !== False) {
					$link_data	= $this->get_brunch_info($id, $rootId, $menuDiv);
					return $this->update_single_value(
									CONFIG::sql_tbl_menu,
									"ulid = ".$result[0]['ulid'],
									"lft BETWEEN '".(int)$link_data[0]['lft']."' AND '".(int)$link_data[0]['rgt']."'
									AND rootId=".(int)$rootId." AND htmlDiv='".$menuDiv."'");
				}
			}
		}
		return $result;
	} // update_menu_node

// ------------------------------

	/**
	 * Get content of a node
	 * 
	 * @access public
	 * @param int $id
	 * @return Boolean
	 */
	public function get_menu_node_content($id = -1, $sel = NULL){
		
		if (!isset($sel))
			$selValues = "c.*, p.*, pt.*, m.rootId, m.htmlDiv, ul.ul";
		else 
			$selValues = $sel;
		
		if ($id != -1) {
			
			return $this->get_single_value(	$selValues,
											CONFIG::sql_tbl_cnt." AS c
											INNER JOIN ".CONFIG::sql_tbl_menu." AS m ON m.id = ".$id."
											INNER JOIN ".CONFIG::sql_tbl_xTP." AS xtp ON xtp.xtpID=m.xtpID
											INNER JOIN ".CONFIG::sql_tbl_proc." AS p ON p.procId=xtp.procId
											INNER JOIN ".CONFIG::sql_tbl_pTypes." AS pt ON pt.pTypId=xtp.pTypId
											INNER JOIN ".CONFIG::sql_tbl_ul." AS ul ON ul.ulid=m.ulid",
											"c.cid=m.cid");
		}
		return False;
	} // get_menu_node_content
	
// ------------------------------

	/**
	 * Update content of a node
	 * 
	 * @access public
	 * @param int $id, string $content
	 * @return Boolean
	 */
	public function update_menu_node_content($id, $content=NULL, $cid=NULL){
		
		if (isset($id) && isset($content) && !isset($cid)) {
			if(($result = $this->get_single_value( "c.content",
												CONFIG::sql_tbl_menu." AS m 
												INNER JOIN ".CONFIG::sql_tbl_cnt."  AS c ON c.cid=m.cid",
												"m.id = ".$id)) !== FALSE) {
				return $this->update_single_value(
									CONFIG::sql_tbl_cnt." INNER JOIN ".CONFIG::sql_tbl_menu.
									" ON ".CONFIG::sql_tbl_menu.".cid=".CONFIG::sql_tbl_cnt.".cid",
									CONFIG::sql_tbl_cnt.".content ='".(string)$content."'",
									CONFIG::sql_tbl_menu.".id = ".$id);
			}
		} else if (isset($cid) && isset($content)){
			
				return $this->update_single_value(
									CONFIG::sql_tbl_cnt,
									"content ='".(string)$content."'",
									"cid = ".$cid);
			
		}
		
		return (isset($id))?(True):(False);
	} // update_menu_node_content	


// ------------------------------	

	/**
	 * Provide special styles data of the menu
	 * 
	 * @access public
	 * @param int $id
	 */
	 
	public function get_menu_style_id($id){
		
		if(($result = $this->get_single_value( "st.*",
												CONFIG::sql_tbl_styles." AS st 
												INNER JOIN ".CONFIG::sql_tbl_menu." AS m ON m.cssid=st.cssid",
												"m.id = ".$id)) !== FALSE)
												
			return (is_array($result)) ? (isset($result[0]['cssid']) ? $result[0]['cssid'] : False) : False;
		
		return False;
		
	} // get_menu_style_id

// ------------------------------	

	/**
	 * @Deprecated
	 * @access public
	 * @param int $id
	 */
	 
	private function get_menu_styles_css($id){

		$strCSS = '';

		$result = $this->get_menu_style_id($id);

		if(isset($result)) {
			$result = $this->get_style_data($result);
		
			if(is_array($result)){
		
				$strCSS.= ($result[0]['bgh_img'] != 'none')?('<style type="text/css"> #divHeader{ background-image: url(./images/'.$result[0]['bgh_img'].'); } </style>'):('');

			}
		}
		return $strCSS;
	} // get_menu_styles_css

// ------------------------------
// content
// ------------------------------	

	/**
	 * Get content of a node
	 * 
	 * @access public
	 * @param int $id
	 * @return Boolean
	 */
	public function get_content_html($id, $ul, $loadEditor=False){
		if (isset($id)) {
		
			if(($result = $this->get_menu_node_content($id)) !== False) {
				
				if ($loadEditor)
					// CTC: START OPTIMISTIC TRANSACTION - RESERVE
					$this->ctc_reserve_table(CONFIG::sql_tbl_cnt, $result[0]['cid']);								
			
				return $this->label_content($result[0]['type'],
											$result[0]['content'], 
											$id, 
											$result[0]['rootId'], 
											$result[0]['htmlDiv'], 
											$ul,
											$loadEditor);
			}
		
		} 
		return False;
		
	} // get_content_html	
	
// ------------------------------	

	/**
	 * Update content of a node
	 * 
	 * @access public
	 * @param int $id, string $content
	 * @return Boolean
	 */
	public function updateContentHtml($id, $content, $cid){
		
		// Transaction was started by Edit Task (see MENU.get_content_html)
		// CTC: CHECK OPTIMISTIC TRANSACTION - LOCK
		if($this->ctc_check_table(CONFIG::sql_tbl_cnt, $cid)) {
			// write the content			
			$result = $this->update_menu_node_content($id, $content, $cid);
			// CTC: CLOSE OPTIMISTIC TRANSACTION - UNLOCK 	
			$result &= $this->ctc_unlock_table(CONFIG::sql_tbl_cnt, $cid);
			// CTC: RESERVE THE ENTRY AGAIN
			// because the editor should be still active and
			// due to the optimistic method this as no negative
			// impact on further actions
			$result &= $this->ctc_reserve_table(CONFIG::sql_tbl_cnt, $cid);

			return (boolean)$result;
		}
		return False;

	} // updateContentHtml	

// ------------------------------	

	/**
	 * Get standard Tasks of a node type according to the content
	 * TODO: ##### CURRENTLY NOT NECESSARY #####
	 * returns allways NULL or False
	 * 
	 * @access public
	 * @param int $id, string $ul 
	 * @return string
	 */
	public function get_std_tasks_html($id, $ul, $showEditorTask = False){
	
		if (isset($id) && ($this->write_access($id, $ul, 'write'))) {
		
			//TODO ineffektiv, wozu den ganzen content holen
			if(($result = $this->get_menu_node_content($id)) !== False)

				return $this->standard_tasks($result[0]['type'],$result[0]['content'], $id, $result[0]['rootId'], $result[0]['htmlDiv'], $ul, $showEditorTask);
		
		}
		// TODO: #### return False; 
		return NULL;
		
	} // get_std_tasks_html	
	

// ------------------------------
// special menu	styles
// ------------------------------	

	/**
	 * Get special styles
	 * Remember, that for special functions like AJAX routines the style sheet switch
	 * must be coded explicitly. The procedure below works for all pages without scripts
	 * 
	 * @Deprecated
	 * @access public
	 * @param int $id
	 * @return string
	 */
	 
	public function get_menu_styles_html($id){
	
		return $this->get_menu_styles_css($id);

	} // get_menu_styles_html	

	
// ------------------------------	

	/**
	 * Provides special styles data
	 * 
	 * @access public
	 * @param int $cssid
	 */
	 
	public function get_style_data($cssid){

		return $this->get_single_value( "st.*",
										CONFIG::sql_tbl_styles." AS st",
										"st.cssid=".$cssid);
		
	} // get_style_data

// ------------------------------	

	/**
	 * insert special styles data
	 * 
	 * @access public
	 * @param string $cssd, string $mkey, string $mdesc
	 */
	 
	public function insert_style_data($cssd=NULL, $mkey=NULL, $mdesc=NULL){

		if(($cssid = $this->insert(CONFIG::sql_tbl_styles, 'mnu_css', "'none'", True)) !== False) {
			if ($this->update_style_data($cssid, $cssd, $mkey, $mdesc) === TRUE) {
				return $cssid;
			}
		}
		return FALSE;
	}
		
// ------------------------------	

	/**
	 * update special styles data
	 * 
	 * @access public
	 * @param int $cssid, string $cssd, string $mkey, string $mdesc
	 */
	 
	public function update_style_data($cssid, $cssd=NULL, $mkey=NULL, $mdesc=NULL){
		
		if(isset($cssid) && (isset($cssd)||isset($mkey)||isset($mdesc))) {
			
				return $this->update_single_value(
									CONFIG::sql_tbl_styles,
									((isset($cssd))?("mnu_css = '".addslashes((string)$cssd)."'"):("")).
									((isset($mkey))?(((isset($cssd))?(", mkey = '".(string)$mkey."'"):("mkey = '".(string)$mkey."'"))):("")).
									((isset($mdesc))?(((isset($cssd)||isset($mkey))?(", mdesc = '".(string)$mdesc."'"):("mdesc = '".(string)$mdesc."'"))):("")),
									"cssid = ".$cssid);
		}
		return (isset($cssd)||isset($mkey)||isset($mdesc))?(False):(True);	

	}

// ------------------------------	
// special action data
// ------------------------------	

	/**
	 * TODO
	 * Get special action data
	 * Such data will be direct used for the menu item, e.g. as additional event
	 * for example "onmouseover=showsometing()"
	 * 
	 * @access public
	 * @param int $said
	 * @return string
	 */
	 
	public function get_special_action_data($said){
		
		return $this->get_single_value( "sact.*",
										CONFIG::sql_tbl_sact." AS sact", 
										"sact.said=".$said);
		
	} // get_special_action_data

// ------------------------------	

	/**
	 * update special action data
	 * 
	 * @access public
	 * @param string $sact
	 */
	public function insert_special_action_data($sact=NULL){
	
		if(($said = $this->insert(CONFIG::sql_tbl_sact, 'sact', "'none'", True)) !== False) {
			if ($this->update_style_data($said, $sact) === TRUE) {
				return $said;
			}
		}
		return FALSE;
	} // insert_special_action_data
		
// ------------------------------	

	/**
	 * update special action data
	 * 
	 * @access public
	 * @param int $said, string $sact
	 */
	 
	public function update_special_action_data($said, $sact=NULL){
		
		if(isset($said) && isset($sact)) {
			
				return $this->update_single_value(
									CONFIG::sql_tbl_sact,
									"sact = '".addslashes((string)$sact)."'",
									"said = ".$said);
		}
		return (isset($said))?(True):(False);	
	}
// ------------------------------	
// User Interface
// ------------------------------

	public function get_user($uid = -1, $ul_only = NULL, $start = NULL, $limit = 1, $filter = NULL){
		if(method_exists($this->user,'upGetUser')) {
			return $this->user->upGetUser($uid, $ul_only, $start, $limit, $filter);
		} else {
			return $this->user->get_core_user($uid, $ul_only, $start, $limit, $filter);
		}	
	}
	public function get_user_no(){
		return $this->user->get_user_no();
	}
	public function insert_user($name, $pass, $ul = '000'){
		return $this->user->insert_user($name, $pass, $ul);
	}
	public function update_user($uid, $name = NULL, $pass = NULL, $ul = NULL){
		return $this->user->update_user($uid, $name, $pass, $ul);
	}
	public function del_user($uid){
		return $this->user->del_user($uid);
	}
	public function get_user_id($name){
		return $this->user->get_user_id($name);
	}
	public function login($name, $pass){
		if(method_exists($this->user,'upLoginWSessionParameter')) {
			return $this->user->upLoginWSessionParameter($name, $pass);
		} else {
			return $this->user->login($name, $pass);
		}	
	}
	public function logout(){
		return $this->user->logout();
	}
	public function logged_in(){
		return $this->user->logged_in();
	}
	public function get_userlevels($is_user=True){
		return $this->user->get_userlevels($is_user);
	}
	public function get_userlevel_role($ul){
		return $this->user->get_userlevel_role($ul);
	}
//	public function get_user_timestamp();
//	public function set_user_timestamp();
	public function clean_user_sessions($username=NULL){
		return $this->user->clean_user_sessions($username=NULL);
	}
}
?>