<?php
/**
 * This Class module provides the MsSQL drivers of the
 * jrCMS core (usable in plugins as well). 
 *
 * @package jrCMS
 * @author Clemens Schultz, clemens.schultz@cinterion.com
 * @version $Revision: 374 $
 * 
 * @copyright Copyright (c) 2009, Clemens Schultz
 * license: MIT-style license, see jrcms.js
 */

require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iSqlDriver.php');

class MsSql_Driver implements iSql_Driver{
	
	CONST VERSION			= '$Revision: 374 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2010-11-14 20:16:15 +0100 (So, 14 Nov 2010) $';
	
	/**
	 * DB-Connection
	 */
	public $bind			= FALSE;
	/**
	 * Gets the access data
	 */
	public $dbconf			= array();
	/**
	 * Gets the sql error
	 */
	public $err				= '';
	/**
	 * Get the database data from class CONFIG and save them in the
	 * $dbconf.
	 */
	public function __construct(){
		$this->dbconf 	= CONFIG::dbData();
		$this->pconf 	= CONFIG::pData();
	}
	
// -----------------------

	/**
	 * @access public
	 * @param string $qstring
	 * @param int $count
	 * @param int $update
	 * @return string
	 *
	 * quote text and add slashes to sql input
	 *	 
	 */

	public function sql_qs($value, $type) {
		switch(strtoupper($type)){
		  case 'BLOB'     :
		  case 'VARCHAR'  :
	  	  case 'CHAR'     :
		  	if($value === ''){return 'NULL';}
	  		else { return "'" . addslashes($value) . "'";}
	  		break;
	  	  case 'NUMBER'   :
	  	  default         :
		  	if ( $value === 0 ) {return addslashes($value);}
		  	elseif ( $value === '' || $value === null) {
	  			return 'NULL';
	  		}
	  		else {
		  		return addslashes($value);
		  	}
		}
	}

// -----------------------

	/**
	 * Bind SQL DB
	 * 
	 * @access public
	 * @return Boolean
	 */
	public function db_bind(){
		try{
			if(!isset($this->dbconf['dbuser']))
				$this->dbconf = CONFIG::dbData();

			$this->bind = mssql_connect($this->dbconf['dbhost'], $this->dbconf['dbuser'], $this->dbconf['dbpwd']);
			if($this->bind == TRUE){
				$select_ = @mssql_select_db($this->dbconf['database']);
				if($select_ === TRUE){
					$this->bind = TRUE;
					return TRUE;
				}else{
					$this->err = mssql_get_last_message();
					throw new MyException('MSSQL DB select failed'.$this->err);
				}	
			}else{
				throw new MyException('MSSQL connect failed');
			}
		}catch (MyException $e) {
				$e->log(True);
				$this->bind = FALSE;
				return FALSE;		
		}
	}
	
// -----------------------
	
	/**
	 * Process all querys and updates.
	 * If update != '', the function only returns true.
	 * 
	 * count 1 = COUNT(*) Abfrage
	 * $update 1 = Update ohne result 
	 *
	 * @param string $qstring, int $count, int $update
	 * @access public
	 */
	public function db_query($qstring, $count = '', $update = ''){
		try{
			if(!empty($qstring)){
				
				if($this->bind === FALSE){
					$this->db_bind();
				}
				$query = mssql_query($qstring);
				
				if($query !== FALSE) {
					if(empty($update)){
						return $this->db_result($query, $count);
					}elseif($update == 1){
						if (is_resource($query)) {mssql_free_result($query);}
						return TRUE;	
					}
				} else {
					$this->err = mssql_get_last_message();
					throw new MyException("MSSQL Query (".$qstring.") failed".$this->err);
				}
			}else{
					return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}
	
		
// -----------------------
	
	/**
	 * Process a Stored Procedure.
	 *
	 * @param string $sp, 
	 * @param array $param_list 	of parameters
	 * @access public
	 */
	public function db_run_stored_procedure($sp = NULL, $param_list = null){
		try{
			if(!empty($sp)){
				if($this->bind === FALSE){
					$this->db_bind();
				}
				//echo "<br />Query: ";
//				print_r($this->bind);
				$qry = "EXEC ".$sp;
				foreach ($param_list as $indx=>$param) {
					$qry .= " @".$param['name']."=".$param['value'].", ";
				}
				$qry = substr($qry, 0, -2);
				// TODO: welchen Typen muss man angeben, wenn ein record set erwartet wird.
//				$rs = array();
//				mssql($proc, "RETVAL", $rs, TYPE);
				//$rs = mssql_execute($proc);
				$rs = $this->db_query($qry);
				//print_r($rs);
				
				if(!empty($rs)) {
					return $rs;
				} else {
					$this->err = mssql_get_last_message();
					throw new MyException("MSSQL SP (".$proc.") failed".$this->err);
				}
			}else{
					return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}
	
	
// -----------------------
	
	/**
	 * Return the number of rows for a query string
	 * 
	 * @param string $qstring
	 * @access public
	 */
	public function db_num_rows($qstring){
		try{
			if(!empty($qstring)){
				if($this->bind === FALSE){
					$this->db_bind();
				}
				//print_r($qstring);
				$query = mssql_query($qstring);

				if($query !== FALSE){
					$rows = mssql_num_rows($query);	
					if (is_resource($query)) {mssql_free_result($query);}
					return $rows;
				}else {
					$this->err = mssql_get_last_message();
					throw new MyException("MSSQL num rows (".$qstring.") failed".$this->err);
				}
			}else{
					return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}

// -----------------------
	
	/**
	 * Return the number of affected rows for an update string
	 * 
	 * @param string $qstring
	 * @access public
	 */
	public function db_affected_rows($qstring){
		try{
			if(!empty($qstring)){
				if($this->bind === FALSE){
					$this->db_bind();
				}

				if(mssql_query($qstring) === FALSE) {
					$this->err = mssql_get_last_message();
					throw new MyException("MSSQL ar (".$qstring.") failed".$this->err);
				}
				
				return mssql_rows_affected();
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}


// -----------------------
	
	/**
	 * Proces all results and returned an array with the wished data.
	 * 
	 * count = 1 display only the scores
	 * 	 
	 * @param string $qstring, int $count
	 * @return mixed array
	 * @access public
	 */
	public function db_result($qquery, $count = FALSE){

		$data = array();
		if($this->bind === FALSE){
			$this->db_bind();
		}
		if(!empty($qquery) && empty($count)){
			while($daten=@mssql_fetch_assoc($qquery)){
				$data[] = $daten;	
			}

			if (is_resource($qquery)) {mssql_free_result($qquery);}
			return $data;
		}elseif(!empty($count) && !empty($qquery)){
			$data[] = mssql_result($qquery, 0);	

			if (is_resource($qquery)) {mssql_free_result($qquery);}
			return $data;
		}

		if (is_resource($qquery)) {mssql_free_result($qquery);}

		return FALSE;
	}
	
// -----------------------
	
	/**
	 * Process all inserts and retuned Boolean
	 * If lastID is given, the function returns the last insert id 
	 * 	 
	 * @access public
	 * @param string $insert_string, int $last_id
	 * @return mixed Boolean or int 
	 */
	public function db_insert($insert_string, $lastID = NULL){
		try{
			if(!empty($insert_string)){
				if($this->bind === FALSE){
					$this->db_bind();
				}
				//if(mssql_query($this->dbconf['database'], $insert_string) === FALSE) throw new MyException($insert_string);
				if(!mssql_query($insert_string)) {
					$this->err = mssql_get_last_message();
					throw new MyException("MSSQL insert (".$insert_string.") failed".$this->err);
				}
				if($lastID == TRUE){
					//return mssql_insert_id();
					$result = $this->db_query("SELECT scope_identity() AS lastid");
					return $result[0]['lastid'];  
					
				}else{
					return TRUE;
				}	
			}else{
				return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}

// -----------------------
	
	public function db_backup(){
		
		//mssql --user=user --password=password --database=db --host=ip < mssql_infile.sql 

		$backupFile = $_SERVER['DOCUMENT_ROOT'].CONFIG::docPath."/backup/".$this->dbconf['database'].date("Y-m-d-H-i-s").".sql";
		$pw = ($this->dbconf['dbpwd'] != '')?("-p ".$this->dbconf['dbpwd']):(NULL);
		$command = CONFIG::mssqlpath."mssqldump -h ".$this->dbconf['dbhost']." ".$this->dbconf['database']." -u ".$this->dbconf['dbuser']." ".$pw." > ".$backupFile;
		
		$last_line = system($command, $fp);
		if ($fp==0) {
			if (!strpos($_SERVER['DOCUMENT_ROOT'], "C:")) {
				//zip
				system("gzip -9 -S .gz ".$backupFile);
				//clean up directory
				system("rm -f ".$backupFile);
			} 
			return TRUE;
		} else {
			//print_r("Return Code: ".$fp."<br />Last Line: ".$last_line."<br />Command: ".$command."<br />");
			return FALSE;
		}
		//import dump
		//system("/usr/bin/mssql -uxxx -pxxx -hxxx vxxx > /www/htdocs/admin/dump/dump.sql", $fp);
		//if ($fp==0) echo "imported"; else echo "failure";
		
	}

// ------------------------------

	public function db_get_error() {
		return $this->err;
	}
}
?>