<?php
/**
 * This Class module provides the SQL queries of the
 * jrCMS core (usable in plugins as well). 
 *
 * @package jrCMS
 * @author Clemens Schultz, clemens.schultz@cinterion.com
 * @version $Revision: 592 $
 * 
 * @copyright Copyright (c) 2009, Clemens Schultz
 * license: MIT-style license, see jrcms.js
 */

require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iSqlQueries.php');

class MsSql_Queries extends MsSql_Driver implements iSql_Queries{
	
	CONST VERSION			= '$Revision: 592 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2011-09-06 19:27:06 +0200 (Di, 06 Sep 2011) $';
	
	/**
	 * Get the database data from class CONFIG and save them in the
	 * $dbconf.
	 */
	
	// no own __construct/__destruct is used
		
// -----------------------
	
	/**
	 * Display a list all database tables.
	 * 
	 * @param string suffix
	 * @return mixed Boolean string (array)
	 */
	public function show_tables_sql($suffix=NULL){
		$annahme = array();
		if($this->bind === FALSE){
			$this->db_bind();
		}
		$qry = "select name from ".$this->dbconf['database']."..sysobjects where xtype = 'U'";
		$qstr	= (isset($suffix))?($qry." AND name LIKE '".addslashes($suffix)."%'"):($qry);
		$hits	= $this->db_query($qstr);
			
		if(is_array($hits) && array_key_exists('0', $hits) && COUNT($hits) > 0){
			foreach($hits AS $key){
				if(is_array($key)){
					foreach($key as $value){
						$annahme[] = $value;
					}
				}else{
					$annahme[] = $value;
				}
			}

			if (is_resource($hits)) {mssql_free_result($hits);}

 			if(is_array($annahme)){
 				return $annahme;
 			}else{
				return FALSE;
 			}
		}
		return FALSE; // keine $hits
	}

// ------------------------------
	
	/**
	 * Check if database-table ($sql_table) exist.
	 * 
	 * @param string $sql_table
	 * @access private
	 * @return Boolean
	 * @author Jens Raabe <jens@raabe-berlin.de>
	 */
	public function check_table($sql_table){

		$erg_tmp = $this->show_tables_sql(addslashes($sql_table));
		if(is_array($erg_tmp)){
			if(is_array($erg_tmp) && array_key_exists('0', $erg_tmp) && COUNT($erg_tmp) > 0){
				return TRUE;
			}else{
				return FALSE;	
			}
		}else{
			/* seems to be wrong
			return TRUE;*/
			return FALSE;
		}

	}

// ------------------------------
	
	/**
	 * Check if column exist in table.
	 * 
	 * @param string $sql_table
	 * @param string $column
	 * @access private
	 * @return Boolean
	 * @author Jens Raabe <jens@raabe-berlin.de>
	 */
	public function check_column($sql_table, $column){

		if($this->bind === FALSE)
			$this->db_bind();
				
		$qry = "IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS 
				WHERE TABLE_CATALOG LIKE '".CONFIG::ms_database."' 
				AND TABLE_NAME LIKE '".$sql_table."' 
				AND COLUMN_NAME LIKE '".$column."') 
				SELECT 'TRUE' AS column_exist 
				ELSE SELECT 'FALSE' AS column_exist";
		try {
			if (($result = $this->db_query($qry)) !== False){
				return ($result[0]['column_exist'] == 'TRUE') ? True : False;
			}else 
				throw new MyException("[MySql failed]: check_column(".$qry.")".$this->db_get_error());
		} catch (MyException $e) {
			$e->log(True);
			return FALSE;	
		}
		
	}
	
// -----------------------

	/**
	 * Returns specific number of rows.
	 * 
	 * @param string column(s) 	single value, comma seperated or *
	 * @param string table(s) 	evtl. complete JOIN construct
	 * @param string WHERE clause
	 * @param string start		row to start
	 * @param string rows		number of rows
	 * @param string id			id used for clause that is needed for start value
	 * @param string ext		Extensions to query like ORDER BY and GROUP BY
	 * @return array			result of query
	 */
	public function get_array($columns = NULL, $tables = NULL, $where = "1", $start = NULL, $rows = NULL, $id = NULL, $ext = NULL, $distinct = FALSE, $debug = NULL){
		if (isset($columns) && isset($tables)) {
			if($this->bind === FALSE)
				$this->db_bind();
			
			$qry = "SELECT ";
			
			if ($distinct)
				$qry .= "DISTINCT ";
			
			if (isset($start))
				$qry .= "TOP ".$rows." ";
									   
			$qry .= $columns." FROM ".$tables.($where == "1" ? "" : " WHERE ".$where);
			
			$qry .= (isset($start)
					? ($where == "1" ? " WHERE " : " AND ").$id." NOT IN (SELECT TOP ".$start." ".$id." FROM ".$tables.($where == "1" ? "" : " WHERE ".$where)." ".(isset($ext) ? $ext : "").")" 
					: "");
			$qry .= (isset($ext)	? " ".$ext : "");

			if (isset($debug)) 
				var_dump($qry);
			
			try {
				
				return $this->db_query($qry);
				/* Exception wird schon in db_query behandelt
				/* wenn das Ergebnis leer ist
				if ($result = $this->db_query($qry))
					return $result;
				else 
					throw new MyException("[MsSql failed]: get_array(".$qry."): ".$this->db_get_error());
				*/
				
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;	
			}
		} else
			return FALSE; //wrong parameters			
	}

// -----------------------

	/**
	 * Returns specific number of rows.
	 * 
	 * @param string column 	single value
	 * @param string table(s) 	evtl. complete JOIN construct
	 * @param string WHERE clause
	 * @param string start		row to start
	 * @param string rows		number of rows
	 * @param string id			id used for clause that is needed for start value
	 * @param string ext		Extensions to query like ORDER BY and GROUP BY
	 * @return string value		result of query
	 */
	public function get_single_value($column = NULL, $tables = NULL, $where = "1", $ext = NULL, $distinct = FALSE, $debug = NULL){
		if(isset($column) && isset($tables)) {
			if($this->bind === FALSE)
				$this->db_bind();

			$qry = "SELECT ";
			if ($distinct)
				$qry.= "DISTINCT ";
			$qry.= $column." FROM ".$tables.($where == "1" ? "" : " WHERE ".$where);
			$qry.= (isset($ext) ? " ".$ext : "");
			
			if (isset($debug)) 
				var_dump($qry);
			
			try {
				$result = $this->db_query($qry);

				if ($result !== False)
					return $result;
				else 
					throw new MyException("[MsSql failed]: get_single_value(".$qry."): ".$this->db_get_error());
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;	
			}
		} else
			return FALSE; //wrong parameters
	}

// -----------------------

	/**
	 * Creates Table
	 * 
	 * @param string name 		name of table
	 * @param array colums	 	[name of column, type, null, auto_inc, key, unique, default, comment]
	 * @param string type		type of table in MySQL not used for MSSQL
	 * @return boolean success
	 */
	public function create_table($table = '', $columns = NULL, $type='MyISAM'){
		if (isset($table) && is_array($columns)) {
			if($this->bind === FALSE)
				$this->db_bind();
			$qry = "CREATE TABLE ".$table." (";
			foreach ($columns as $indx => $column) {
				if ($indx > 0)
					$qry .= ", ";
				$qry .= "[".$column['name']."] ";
				$qry .= ($column['type'] == "tinyint(1)") ? "tinyint " : $column['type']." ";
				$qry .= ($column['null'] == TRUE) ? "NULL " : "NOT NULL ";
				$qry .= ($column['auto_inc'] == TRUE) ? "IDENTITY(1,1) " : " ";		
				$qry .= ($column['key'] == TRUE) ? "PRIMARY KEY " : " ";
				$qry .= ($column['unique'] == TRUE) ? "UNIQUE " : " ";
				$qry .= (isset($column['default'])) 
					? (is_numeric($column['default'])) 
					? "DEFAULT ".$column['default'].""
					: "DEFAULT '".$column['default']."'"
					: "";
			}
			$qry = trim($qry).")";
			try {
				if ($this->db_query($qry, -1, 1))
					return TRUE; // TODO $column['comment'] to be done by "EXEC sp_addextendedproperty" 
				else
					throw new MyException("[MSSQL failed] create_table( ".$qry."): ".$this->db_get_error());
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;		
			};
		} else {
			return FALSE; //(isset($name) && is_array($columns))
		};
	}

// -----------------------

	/**
	 * Alter a Table
	 * 
	 * @param string name 		name of table
	 * @param array colums	 	[name of column, type, null, auto_inc, unique, default, comment, alter]
	 * @return boolean success
	 */
	public function alter_table($name = '', $columns = NULL){
		if (isset($name) && is_array($columns)) {
			if($this->bind === FALSE)
				$this->db_bind();			
			$qry1 = "ALTER TABLE ".$name." ";
			try {
				foreach ($columns as $indx => $column) {
					//that's not runtime criticial
					if( isset($column['alter']) &&
						(($column['alter'] == 'ADD' && $this->check_column($name, $column['name']) === False) ||
						 (($column['alter'] == 'ALTER' || $column['alter'] == 'DROP')&& $this->check_column($name, $column['name']) !== False)
						 )
						){
						$qry = $qry1.$column['alter']." ";
						$qry .= $column['name']." ";
						$qry .= $column['type']." ";
						$qry .= ($column['null'] == TRUE) ? "NULL " : "NOT NULL ";
						$qry .= ($column['auto_inc'] == TRUE) ? "auto_increment " : " ";		
						$qry .= ($column['unique'] == TRUE) ? "UNIQUE " : " ";
						$qry .= (isset($column['default'])) 
							? (is_numeric($column['default'])) 
							? "DEFAULT ".$column['default']." "
							: "DEFAULT '".$column['default']."' "
							: " ";
						//$qry .= (isset($column['comment'])) ? "COMMENT '".$column['comment']."' " : " ";
						// TODO $column['comment'] to be done by "EXEC sp_addextendedproperty" 
							
						$qry = trim($qry);
						if (!$this->db_query($qry, -1, 1))
							throw new MyException("[MSSQL failed]: alter_table('.$qry.') ".$this->db_get_error());
					}
				}
			} catch (MyException $e) {
				$e->log(True);
				$this->bind = FALSE;
				return FALSE;		
			};
			return TRUE;					
		} else {
			return FALSE; //(isset($name) && is_array($columns))
		};
	}
	
// -----------------------

	/**
	 * Insert rows into table
	 * 
	 * @param string name 		name of table
	 * @param string colums	 	comma seperated
	 * @param string values	 	comma seperated
	 * @return boolean success
	 */
	public function insert($table = NULL, $columns = NULL, $values = NULL, $insertId = False, $debug=NULL){
		if (isset($table) && isset($columns) && isset($values)) {
			if($this->bind === FALSE)
				$this->db_bind();
			$columns = "[".str_replace(", ", "], [", $columns)."]";
			$qry = "INSERT INTO ".$table." (".$columns.") VALUES (".$values.")";
			
			if (isset($debug)) 
				var_dump($qry);
			
			try {
				if ($result = $this->db_insert($qry, $insertId))
					return $result;  // insert_id
				else
					throw new MyException("[MSSQL failed] insert_table(".$qry."): ".$this->db_get_error()); 
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;		
			};
		} else {
			return FALSE; //(isset($name) && is_array($columns))
		};
	}

// -----------------------

	/**
	 * Insert rows into table by using REPLACE
	 * 
	 * @param string name 		name of table
	 * @param string colums	 	comma seperated
	 * @param string values	 	comma seperated
	 * @return boolean success
	 */
	public function replace($table = NULL, $columns = NULL, $values = NULL, $insertId = False){
		if (isset($table) && isset($columns) && isset($values)) {
			if($this->bind === FALSE)
				$this->db_bind();
			$columns = "[".str_replace(", ", "], [", $columns)."]";
			$qry = "REPLACE INTO ".$table." (".$columns.") VALUES (".$values.")";
			try {
				if ($result = $this->db_insert($qry, $insertId))
					return $result;  // insert_id
				else
					throw new MyException("[MSSQL failed] insert_table(".$qry."): ".$this->db_get_error()); 
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;		
			};
		} else {
			return FALSE; //(isset($name) && is_array($columns))
		};
	}
	
// -----------------------

	/**
	 * Update a single value.
	 * 
	 * @param string table(s) 	evtl. complete JOIN construct
	 * @param string values 	single values
	 * @param string WHERE clause
	 * @param string ext		Extensions to query like ORDER BY and GROUP BY
	 * @return string value		result of query
	 */
	public function update_single_value($tables = NULL, $values = NULL, $where = "1", $ext = NULL, $debug = NULL){
		if(isset($values) && isset($tables)) {
			if($this->bind === FALSE)
				$this->db_bind();
				
			$qry = "UPDATE ".$tables." SET ".$values." WHERE ".$where;
			$qry.= (isset($ext) ? $ext : "");

			if (isset($debug)) 
				var_dump($qry);
			
			try {
				if ($result = $this->db_query($qry,'',1))
					return $result;
				else 
					throw new MyException("[MySql failed]: update_single_value(".$qry.")".$this->db_get_error());
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;	
			}
		} else
			return FALSE; //wrong parameters
	}
		
// -----------------------

	/**
	 * Delete a single value.
	 * 
	 * @param string column 	single value
	 * @param string table(s) 	evtl. complete JOIN construct
	 * @param string WHERE clause
	 * @param string ext		Extensions to query like ORDER BY and GROUP BY
	 * @return string value		result of query
	 */
	public function delete_single_value($columns = ".*", $tables = NULL, $where = "1", $ext = NULL){
		
		if(isset($tables)) {
			if($this->bind === FALSE)
				$this->db_bind();

			// for compatibility reason to mySQL
			$columns = str_replace(".*", "", $columns);
			
			$qry = "DELETE ".$columns." FROM ".$tables." WHERE ".$where;
			$qry.= (isset($ext) ? $ext : "");

			try {
				if ($result = $this->db_query($qry,'',1))
					return $result;
				else 
					throw new MyException("[MsSql failed]: delete_single_value(".$qry.")".$this->db_get_error());
			} catch (MyException $e) {
				$e->log(True);
				return FALSE;	
			}
		} else
			return FALSE; //wrong parameters
	}
	
// -----------------------

	/**
	 * Returns specific number of rows of the nested menu tree.
	 * 
	 * @param string userLevel 	user level as string
	 * @param string column(s) 	additional values, comma seperated
	 * @param string joins(s) 	JOIN constructs need for column(s) above
	 * @param string where			additional WHERE clause
	 * @return array
	 */
	public function get_menu_tree_array($rootID=NULL, $userLevel=NULL, $menuDiv=NULL, $brunchId=NULL, $getRoot=False){		
		if(isset($userLevel)) {
			if($this->bind === FALSE)
				$this->db_bind();

			$qry = "SELECT first.id, first.parentId, first.rootId, first.label, first.lft, first.rgt, "; 
			$qry.= "first.htmlDiv, first.ulid, first.cssid, first.lang, first.cid, ";
			$qry.= "first.xtpID, first.said, p.procName, p.classId, t.type, sa.sact, ";
			$qry.= "COUNT(*) - 1 AS LEVEL, ROUND((first.rgt - first.lft - 1) / 2, 0) AS CHILDS, ";
			$qry.= "ul.ul ";
			$qry.= "FROM ".CONFIG::sql_tbl_menu." AS first ";
			$qry.= "INNER JOIN ".CONFIG::sql_tbl_ul." AS ul ON ul.ulid=first.ulid ";
			$qry.= "INNER JOIN ".CONFIG::sql_tbl_sact." AS sa ON sa.said=first.said "; 
			$qry.= "INNER JOIN ".CONFIG::sql_tbl_xTP." AS x ON x.xtpID=first.xtpID "; 
			$qry.= "INNER JOIN ".CONFIG::sql_tbl_proc." AS p ON p.procId=x.procId "; 
			$qry.= "INNER JOIN ".CONFIG::sql_tbl_pTypes." AS t ON t.pTypId=x.pTypId, ";
			
			$qry.= " ".CONFIG::sql_tbl_menu." AS second";
			$qry.= " WHERE first.lft BETWEEN second.lft AND second.rgt ";
			$qry.= (($getRoot===FALSE)?("AND first.label NOT LIKE 'ROOT' "):(""));

			if(isset($rootID) && $rootID > 0 && !empty($menuDiv)){
				$qry.= "AND first.rootId = ".$rootID." AND second.rootId = ".$rootID."
								AND first.htmlDiv = '".$menuDiv."' AND second.htmlDiv = '";
				$qry.= $menuDiv."' " ;
			} else if(isset($rootID) && $rootID > 0){
				$qry.= "AND first.rootId = ".$rootID." AND second.rootId = ".$rootID." ";
			} else if (!empty($menuDiv)){
				$qry.= "AND first.htmlDiv = '".$menuDiv."' AND second.htmlDiv = '".$menuDiv."' " ;
			} else {
				$qry.= "AND first.id = ".$brunchId." ";
			}

			$qry.= "AND LEFT(ul.ul,1) <= ".(substr($userLevel,0,1))." ";
			$qry.= "AND first.lang LIKE '".(isset($_SESSION["language"])?($_SESSION["language"]):('en'))."' ";
			$qry.= "AND second.lang LIKE '".(isset($_SESSION["language"])?($_SESSION["language"]):('en'))."' ";
		
			$qry.= "GROUP BY first.lft, first.id, first.parentId, first.rootId, first.label, ";
			$qry.= "first.rgt, first.htmlDiv, first.ulid, first.cssid, first.lang, first.cid, ";
			$qry.= "first.xtpID, first.said, p.procName, p.classId, t.type, sa.sact, ul.ul ";
			$qry.= "ORDER BY first.lft";
			
			$result = $this->db_query($qry);
			return $result;		

		} else {
			return FALSE; //wrong parameters
		}
	}
	
// -----------------------

	/**
	 * Returns the ids of a label (array) in a menuDiv
	 * 
	 * @param string label 		name of the menu entry
	 * @param string userLevel 	user level as string	
	 * @param string menuDiv	in which div (html) the tree appears
	 * @return array
	 */
	public function get_menu_item_ids($label, $userLevel='000', $menuDiv='divVMenu'){		
		if($this->bind === FALSE)
			$this->db_bind();
		//TODO:	
		$qry = "SELECT first.*, ".CONFIG::sql_tbl_ul.".ul AS ul ";
		$qry.= "FROM ".CONFIG::sql_tbl_menu." AS first ";
		$qry.= "INNER JOIN ".CONFIG::sql_tbl_ul." ON ".CONFIG::sql_tbl_ul.".ulid=first.ulid ";
		$qry.= " ".CONFIG::sql_tbl_menu." AS second ";
		$qry.= "WHERE first.lft BETWEEN second.lft AND second.rgt AND first.label NOT LIKE 'ROOT' "; 
		$qry.= "AND first.htmlDiv = '".$menuDiv."' AND second.htmlDiv = '".$menuDiv."' " ;
		$qry.= "AND first.label = '".$label."' AND second.label = '".$label."' " ;
		$qry.= "AND LEFT(".CONFIG::sql_tbl_ul.".ul,1) <= ".(substr($userLevel,0,1))." ";
		$qry.= "AND first.lang LIKE '".(isset($_SESSION["language"])?($_SESSION["language"]):('en'))."' ";
		$qry.= "AND second.lang LIKE '".(isset($_SESSION["language"])?($_SESSION["language"]):('en'))."' ";
		$qry.= " GROUP BY first.lft ORDER BY first.lft";
		
		return $this->db_query($qry);
	}			
	

// -----------------------

	/**
	 * Reserve a table entry for access protection
	 * (optimistic method)
	 * Special response method, if the reserve task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a session variable $_SESSION["ctc"] holds some further informations 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctc_reserve_table($table, $eId){		
		if($this->bind === FALSE)
			$this->db_bind();

		return $this->ctc_lock_table($table, $eId, 'reserved');		
	}			
	
// -----------------------

	/**
	 * Lock an access protected table
	 * Special response method, if the lock task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a session variable $_SESSION["ctc"] holds some further informations 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @param string accType	access type
	 * @return boolean
	 */
	public function ctc_lock_table($table, $eId, $accType='locked'){		
		if($this->bind === FALSE)
			$this->db_bind();

		try {
//			//1) open a transaction by using InnoDB of MySQL
//			// similar sequence is used for each function
			$this->db_query('BEGIN TRANSACTION');
			
			//2) set the values 'locked' od 'reserved'
			if(count($ctcresp = $this->get_array("ctc.accType, usr.UserName",
						CONFIG::sql_tbl_ctc." AS ctc
						INNER JOIN ".CONFIG::sql_tbl_user." AS usr ON ctc.UserSession=usr.UserSession",
						"ctc.tableAndId LIKE '".($table."_".(string)$eId)."' AND ctc.accType LIKE 'locked'")) == 0) {		
			   	
				//refuse concurrent entries
				if($accType=='locked')
					$this->update_single_value(CONFIG::sql_tbl_ctc,
			   					"accType = 'refused'",
			   					"tableAndId LIKE '".($table."_".(string)$eId)."'");
			   	
			   	//determine kind of change
				if(count($this->get_array("ctc.accType",
								CONFIG::sql_tbl_ctc." AS ctc",
								"ctc.tableAndId LIKE '".($table."_".(string)$eId)."' AND ctc.UserSession LIKE '".session_id()."'")) == 1) { 
					//update the own entry
					$this->update_single_value(CONFIG::sql_tbl_ctc,
			   					"accType = '".($accType=='locked'?'locked':'reserved')."'",
			   					"tableAndId LIKE '".($table."_".(string)$eId)."' AND UserSession LIKE '".session_id()."'");
				} else {
					//create new entry
					$this->insert(CONFIG::sql_tbl_ctc, "tableAndId, UserSession, accType",
								"'".($table."_".(string)$eId)."', '".session_id()."', '".addslashes((string)$accType)."'");
				}

				$result = TRUE;
				
			} else {
				$result = FALSE;
			}
			//3) commit transaction 
			if(!$this->db_query('COMMIT TRANSACTION','',1))
					throw new MyException("CTC failure. Try again!");
			
			$_SESSION["ctc"] = is_array($ctcresp)&&	isset($ctcresp[0]['UserName'])?$ctcresp[0]['UserName']:NULL;	
			return $result;		
		
		} catch (MyException $e) {
			$this->db_query('ROLLBACK TRANSACTION');
			$_SESSION["ctc"] = $e->getMessage();
			return FALSE;	
		}
	}			
	
// -----------------------

	/**
	 * Check and Lock an a previously reserved table ID
	 * (optimistic method)
	 * Special response method, if the lock task succeed a TRUE will
	 * be returned. Otherwise FALSE will be given back and
	 * a session variable $_SESSION["ctc"] holds some further informations 
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctc_check_table($table, $eId){		
		if($this->bind === FALSE)
			$this->db_bind();
			
		//due to optimistic method we do not use a transaction for state check
		if(count($this->get_array("ctc.accType",
							CONFIG::sql_tbl_ctc." AS ctc",
							"ctc.tableAndId LIKE '".($table."_".(string)$eId)."' 
							AND ctc.UserSession LIKE '".session_id()."'
							AND ctc.accType LIKE 'reserved'")) == 1) {
							
			//lock transaction				
			return $this->ctc_lock_table($table, $eId);
			
		} else {
			
			//ensure a cleaned table (e.g. due to 'refused')
			$this->ctc_unlock_table($table, $eId);
			$_SESSION["ctc"] = 'Transaction aborted';
			return FALSE;
			
		}
	}
				
// -----------------------

	/**
	 * UnLock an access protected table
	 * 
	 * @param string table	 	name of table
	 * @param integer eId		ID of table element
	 * @return boolean
	 */
	public function ctc_unlock_table($table, $eId){		
		if($this->bind === FALSE)
			$this->db_bind();
			
		try {
			//1) open a transaction by using InnoDB of MySQL
			// similar sequence is used for each function
			$this->db_query('BEGIN TRANSACTION');
			
			//2) delete the entry
			$this->delete_single_value( CONFIG::sql_tbl_ctc.".*", 
										CONFIG::sql_tbl_ctc, 
										CONFIG::sql_tbl_ctc.".tableAndId LIKE '".($table."_".(string)$eId)."'
										AND ".CONFIG::sql_tbl_ctc.".UserSession LIKE '".session_id()."'");

			//3) commit transaction 
			$this->db_query('COMMIT TRANSACTION');
			$_SESSION["ctc"] = NULL;
			return TRUE;
					
		} catch (MyException $e) {
			$this->db_query('ROLLBACK TRANSACTION');
			$_SESSION["ctc"] = NULL;
			return FALSE;	
		}
	}			
	
// -----------------------

	/**
	 * Clean CTC Table
	 * 
	 * @param string UserSession	Session
	 * @return boolean
	 */
	public function ctc_clean_table($UserSession=NULL){		
		if($this->bind === FALSE)
			$this->db_bind();

		try {
			//1) open a transaction by using InnoDB of MySQL
			// similar sequence is used for each function
			$this->db_query('BEGIN TRANSACTION');
			
			//2) clean ctc table
			If( isset($UserSession)) {
			
				$this->delete_single_value( CONFIG::sql_tbl_ctc.".*", 
											CONFIG::sql_tbl_ctc, 
											CONFIG::sql_tbl_ctc.".UserSession LIKE '".$UserSession."'");
			} else {
				// vice versa, we check if there exist a locked table, 
				// but no user session exist for this entry
				if(($result = $this->get_array("ctc.UserSession, usr.uid",
												CONFIG::sql_tbl_ctc." AS ctc ".
												"LEFT JOIN ".CONFIG::sql_tbl_user." AS usr ON usr.UserSession LIKE ctc.UserSession",
												"ctc.UserSession IS NOT NULL"
												)) !== False) {
					if(isset($result) && !empty($result))
						foreach ($result as $indx => $blockedEntry) {
							if (!isset($blockedEntry['uid']))
								$this->delete_single_value( CONFIG::sql_tbl_ctc.".*", 
												CONFIG::sql_tbl_ctc, 
												CONFIG::sql_tbl_ctc.".UserSession LIKE '".$blockedEntry['UserSession']."'");
						}

				}
			}												
											
			//3) commit transaction 
			$this->db_query('COMMIT TRANSACTION');
			return TRUE;
					
		} catch (MyException $e) {
			$this->db_query('ROLLBACK TRANSACTION');
			return FALSE;	
		}
	}
	
// -----------------------

	/**
	 * Executes a stored procedure TODO: copied from MsSql, has to be adapted to MySql
	 *
	 * @param string spName		name of stored procedure 
	 * @param mixed params		array or string of parameter(s)
	 * @return string			or false in case of error
	 */
	public function run_stored_proc($spName, $params=NULL){
//		$qry = "EXEC [DBO_WMRD_IT_ST_DB].[".$spName."]";
		$qry = "EXEC [DBO_WMRD_IT_ST_DB].[".$spName." ";
		if(is_array($params)) {
			foreach($params as $param) {
				if (!is_array($param))
					$qry .= " ".$param.",";
				else
					$qry .= " @".$param[0]."='".$param[1]."',";
			}
			substr($qry, 0, -1);
		} else {
			if (!is_null($params))
				$qry .= $params; 
		}
		$qry .= "];";
		if (!empty($qry)) {
			if($this->bind === FALSE)
				$this->db_bind();
			try {
				return $this->db_query($qry);

			} catch (MyException $e) {
				$e->log(True);
				return FALSE;
			}
		} else
		return FALSE; //wrong parameters
	}
}
?>