<?php
/**
 * This Class module provides the MySQL drivers of the
 * jrCMS core (usable in plugins as well). 
 *
 * @package jrCMS: SQL Driver
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: cMySqlDriver.php 592 2011-09-06 17:27:06Z jensraabe $ 
 *
 * @copyright Copyright (c) 2008,2009, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iSqlDriver.php');

class MySql_Driver implements iSql_Driver {
	
	CONST VERSION			= '$Revision: 592 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS: MySQL Driver';
	CONST LAST_CH			= '$Date: 2011-09-06 19:27:06 +0200 (Di, 06 Sep 2011) $';
	
	/**
	 * DB-Connection
	 */
	public $bind			= FALSE;
	/**
	 * Gets the access data
	 */
	public $dbconf			= array();
	/**
	 * Gets the sql error
	 */
	public $err				= '';
	
	/**
	 * Get the database data from class CONFIG and save them in the
	 * $dbconf.
	 */
	public function __construct(){
		$this->dbconf = CONFIG::dbData();
		$this->pconf  = CONFIG::pData();
	}

// -----------------------

	/**
	 * quote text and add slashes to sql input
	 *
	 * @access public
	 * @param string $qstring
	 * @param int $count
	 * @param int $update
	 * @return string
	 */

	public function sql_qs($value, $type) {
		switch(strtoupper($type)){
		  case 'BLOB'     :
		  case 'VARCHAR'  :
	  	  case 'CHAR'     :
		  	if($value === ''){return 'NULL';}
	  		else { return "'" . addslashes($value) . "'";}
	  		break;
	  	  case 'NUMBER'   :
	  	  default         :
		  	if ( $value === 0 ) {return addslashes($value);}
		  	elseif ( $value === '' || $value === null) {
	  			return 'NULL';
	  		}
	  		else {
		  		return addslashes($value);
		  	}
		}
	}

// -----------------------

	/**
	 * Bind SQL DB
	 * 
	 * @access public
	 * @return Boolean
	 */
	public function db_bind(){
		try{
			if(!isset($this->dbconf['dbuser']))
				$this->dbconf = CONFIG::dbData();

			$this->bind = mysql_connect($this->dbconf['dbhost'], $this->dbconf['dbuser'], $this->dbconf['dbpwd']);
			if($this->bind == TRUE){
				$select_ = @mysql_select_db($this->dbconf['database']);
				if($select_ === TRUE){
					$this->bind = TRUE;
					return TRUE;
				}else{
					$this->err = mysql_error();
					throw new MyException('MYSQL DB select failed'.$this->err);
				}	
			}else{
				throw new MyException('MySQL connect failed');
			}
		}catch (MyException $e) {
				$e->log(True);
				$this->bind = FALSE;
				return FALSE;		
		}
	}

// -----------------------
	
	/**
	 * Process all querys and updates.
	 * If update != '', the function only returns true.
	 * 
	 * count 1 = COUNT(*) Abfrage
	 * $update 1 = Update ohne result 
	 *
	 * @param string $qstring, int $count, int $update
	 * @access public
	 */
	public function db_query($qstring, $count = '', $update = ''){
		try{
			if(!empty($qstring)){
				
				if($this->bind === FALSE){
					$this->db_bind();
				}
				$query = mysql_query($qstring);
				
				if($query !== FALSE) {
					if(empty($update)){
						return $this->db_result($query, $count);
					}elseif($update == 1){
						if (is_resource($query)) {mysql_free_result($query);}
						return TRUE;	
					}
				} else {
					$this->err = mysql_error();
					throw new MyException('MYSQL query ('.$qstring.') failed'.$this->err);
				}
			}else{
					return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}
	

// -----------------------
	
	/**
	 * Return the number of rows for a query string
	 * 
	 * @param string $qstring
	 * @access public
	 */
	public function db_num_rows($qstring){
		try{
			if(!empty($qstring)){
				if($this->bind === FALSE){
					$this->db_bind();
				}

				$query = mysql_query($qstring);

				if($query !== FALSE){
					$rows = mysql_num_rows($query);	
					if (is_resource($query)) {mysql_free_result($query);}
					return $rows;
				}else {
					$this->err = mysql_error();
					throw new MyException('MYSQL num rows ('.$qstring.') failed'.$this->err);
				}
			}else{
					return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}

// -----------------------
	
	/**
	 * Return the number of affected rows for an update string
	 * 
	 * @param string $qstring
	 * @access public
	 */
	public function db_affected_rows($qstring){
		try{
			if(!empty($qstring)){
				if($this->bind === FALSE){
					$this->db_bind();
				}

				if(mysql_query($qstring) === FALSE) throw new MyException($qstring);
				return mysql_affected_rows();

			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}


// -----------------------
	
	/**
	 * Proces all results and returned an array with the wished data.
	 * 
	 * count = 1 display only the scores
	 * 	 
	 * @param string $qstring, int $count
	 * @return mixed array
	 * @access public
	 */
	public function db_result($qquery, $count = FALSE){

		$data = array();
		if($this->bind === FALSE){
			$this->db_bind();
		}
		if(!empty($qquery) && empty($count)){
			while($daten=@mysql_fetch_assoc($qquery)){
				$data[] = $daten;	
			}

			if (is_resource($qquery)) {mysql_free_result($qquery);}

			//we try to work with empty $data as well
			return $data;
			/*
			if(is_array($data) && !empty($data)){
				return $data;	
			}else{
				return FALSE;	
			}
			*/
		}elseif(!empty($count) && !empty($qquery)){
			$data[] = mysql_result($qquery, 0);	

			if (is_resource($qquery)) {mysql_free_result($qquery);}

			//we try to work with empty $data as well
			return $data;
			/*
			if(is_array($data) && !empty($data)){
				return $data;	
			}else{
				return FALSE;	
			}
			*/
		}

		if (is_resource($qquery)) {mysql_free_result($qquery);}

		return FALSE;
	}
	
// -----------------------
	
	/**
	 * Process all inserts and retuned Boolean
	 * If lastID is given, the function returns the last insert id 
	 * 	 
	 * @access public
	 * @param string $insert_string, int $last_id
	 * @return mixed Boolean or int 
	 */
	public function db_insert($insert_string, $lastID = NULL){
		try{
			if(!empty($insert_string)){
				if($this->bind === FALSE){
					$this->db_bind();
				}
				//if(mysql_db_query($this->dbconf['database'], $insert_string) === FALSE) throw new MyException($insert_string);
				if(!mysql_query($insert_string)) {
					$this->err = mysql_error();
					throw new MyException("MYSQL insert (".$insert_string.") failed".$this->err);
				}
				
				if($lastID != NULL){
					return mysql_insert_id();
				}else{
					return TRUE;
				}	
			}else{
				return FALSE;	
			}
		}catch (MyException $e) {
				$e->log(True);
				return FALSE;		
		}
	}

// -----------------------

	public function db_backup(){
		
		//mysql --user=user --password=password --database=db --host=ip < mysql_infile.sql 

		$backupFile = $_SERVER['DOCUMENT_ROOT'].CONFIG::docPath."/backup/".$this->dbconf['database'].date("Y-m-d-H-i-s").".sql";
		$pw = ($this->dbconf['dbpwd'] != '')?("-p ".$this->dbconf['dbpwd']):(NULL);
		$command = CONFIG::mysqlpath."mysqldump -h ".$this->dbconf['dbhost']." ".$this->dbconf['database']." -u ".$this->dbconf['dbuser']." ".$pw." > ".$backupFile;
		
		$last_line = system($command, $fp);
		if ($fp==0) {
			if (!strpos($_SERVER['DOCUMENT_ROOT'], "C:")) {
				//zip
				system("gzip -9 -S .gz ".$backupFile);
				//clean up directory
				system("rm -f ".$backupFile);
			} 
			return TRUE;
		} else {
			//print_r("Return Code: ".$fp."<br />Last Line: ".$last_line."<br />Command: ".$command."<br />");
			return FALSE;
		}
		//import dump
		//system("/usr/bin/mysql -uxxx -pxxx -hxxx vxxx > /www/htdocs/admin/dump/dump.sql", $fp);
		//if ($fp==0) echo "imported"; else echo "failure";
		
	}

// -----------------------

	public function db_run_stored_procedure($sp = NULL, $param_list = null){
		return False;
	}
	
// ------------------------------

	public function db_get_error() {
		return $this->err;
	}
}
?>