<?php
/**
 * This Class module provides the data base access for 
 * the general user maintenance
 *
 * @package jrCMS: Class USER
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: cUser.php 607 2011-09-20 05:58:51Z jensraabe $ 
 * 
 * @copyright Copyright (c) 2011, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */
 
require_once($_SERVER['DOCUMENT_ROOT'].CONFIG::srcPath.'/src/'.'iUser.php');

class USER extends DBInterface implements iUser{
	
	CONST VERSION			= '$Revision: 607 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCMS';
	CONST LAST_CH			= '$Date: 2011-09-20 07:58:51 +0200 (Di, 20 Sep 2011) $';
	
	// no own __construct/__destruct is used

// ------------------------------

	/**
	 * Get the core user data even for a session id or for the selected user itself.
	 * This routine is an (internal) function which  could be replaced or re-used 
	 * by a user profile functionality. The core itself uses the general routine 
	 * get basic data only.
	 * 
	 * get_core_user() 									user of the active session
	 * get_core_user(-1, NULL, $start, $limit, $filter)	user list
	 * get_core_user(<uid>)								user of uid
	 * 
	 * @access	public
	 * @param	int $uid, boolean $ul_only, int $start, int $limit
	 * @return 	mixed Boolean, string (array), value (ul)
	 */
	 
	public function get_core_user($uid = -1, $ul_only = NULL, $start = NULL, $limit = 1, $filter=NULL){

		if(($uid == -1) && !isset($start)){
			//retrieve the user of a session
			$result = $this->get_single_value("usr.* , usrl.ul",
												CONFIG::sql_tbl_user." AS usr INNER JOIN ".CONFIG::sql_tbl_ul." AS usrl ON usr.ulid=usrl.ulid",
												"(((usr.UserSession)='".session_id()."'))");
			
		} else if(($uid == -1) && isset($start)) {
			
			//retrieve a user list
			$result = $this->get_array("usr.uid AS uid, usr.UserName AS UserName, usrl.ul, usrl.description ",
												CONFIG::sql_tbl_user." AS usr JOIN ".CONFIG::sql_tbl_ul." AS usrl ON usr.ulid=usrl.ulid ",
												(isset($filter) ? str_replace("\'", "'", $filter) : "1"),
												 $start, $limit, "usr.uid ", " ORDER BY usr.UserName");
			
		} else if(($uid != -1)) {
			//retrieve singel user by ID
			$result = $this->get_single_value("usr.* , usrl.ul",
												CONFIG::sql_tbl_user." AS usr INNER JOIN ".CONFIG::sql_tbl_ul." AS usrl ON usr.ulid=usrl.ulid",
												" usr.uid = ".$uid);
		}
		
		if(isset($result) && is_array($result)){
			
			return isset($ul_only) ? $result[0]['ul'] : $result;

		}else{

			return isset($ul_only) ? '000' : FALSE;

		}

	} // get_basic_user


//// ------------------------------
//
//	/**
//	 * Generic get user data functionality even for a session id or for the 
//	 * selected user itself. If a user profile functionality exist instead of 
//	 * core function (get_basic_user) the profile function is called. By convention
//	 * the name of the profile function must be upGetUser with the same
//	 * interface as get_user below. 
//	 * An important rule apply. There have to be a 1:1 relation between core user and
//	 * user profile.
//	 * 
//	 * get_user() 									user of the active session
//	 * get_user(-1, NULL, $start, $limit, $filter)	user list
//	 * get_user(<uid>)								user of uid
//	 * 
//	 * @access	public
//	 * @param	int $uid, boolean $ul_only, int $start, int $limit
//	 * @return 	mixed Boolean, string (array), value (ul)
//	 */
//	 
//	public function get_user($uid = -1, $ul_only = NULL, $start = NULL, $limit = 1, $filter=NULL){
//		
//		return $this->get_basic_user($uid, $ul_only, $start, $limit, $filter);
//		
//	} // get_user
		
	
// ------------------------------

	/**
	 * Get the number of user data
	 * 
	 * @access public
	 * @param none
	 * @return int
	 */
	 
	public function get_user_no($filter=NULL){

		return	count($this->get_array('uid' 
										,CONFIG::sql_tbl_user
										,(isset($filter) ? str_replace("\'", "'", $filter) : "1")
										));
			
	} // get_user_no

// ------------------------------

	/**
	 * insert user data
	 * 
	 * @access public
	 * @param string $name, string $pass, string $ul
	 * @return Boolean
	 */
	 
	public function insert_user($name, $pass, $ul = '100'){
			
		//retrieve user level Id
		$result = $this->get_single_value('ulid', CONFIG::sql_tbl_ul, CONFIG::sql_tbl_ul.".ul=".$ul);
		//assign default value in case of wrong processing
		$ulid = is_array($result) ? $result[0]['ulid'] : 1;
		return $this->insert(CONFIG::sql_tbl_user, "UserName, UserPass, UserSession, ulid", 
									"'".addslashes((string)$name)."', '".md5($pass)."', NULL, ".addslashes((int)$ulid));
		
	} // insert_user


// ------------------------------

	/**
	 * update user data
	 * 
	 * @access public
	 * @param int $uid, string $name, string $pass, string $ul
	 * @return Boolean
	 */
	 
	public function update_user($uid, $name = NULL, $pass = NULL, $ul = NULL){
	
		$un_set = NULL;
		$ul_set = NULL;
		$p_set = NULL;
				
		if (isset($pass)) 
			$p_set = "UserPass = '".md5($pass)."'";
			
		if (isset($ul)) {
			//retrieve user level Id
			$result = $this->get_single_value('ulid', CONFIG::sql_tbl_ul, CONFIG::sql_tbl_ul.".ul=".$ul);
			//assign default value in case of wrong processing
			$ul_set = (isset($pass) ? (" ,") : (""))."ulid = ".(is_array($result) ? $result[0]['ulid'] : 1);
		}
		
		if (isset($name))
			$un_set = ((isset($ul_set)||isset($p_set)) ? (" ,") : (""))."UserName = '".$name."'";
		
		return $this->update_single_value(CONFIG::sql_tbl_user, $p_set." ".$ul_set." ".$un_set, "uid = '".$uid."'");

	} // update_user

// ------------------------------

	/**
	 * Delete the user data
	 * 
	 * @access public
	 * @param int $uid
	 * @return Boolean
	 */
	 
	public function del_user($uid){

		return $this->delete_single_value(CONFIG::sql_tbl_user.".*", CONFIG::sql_tbl_user, CONFIG::sql_tbl_user.".uid = '".$uid."'");

	} // del_user

// ------------------------------

	/**
	 * Determine the ID of a User
	 * 
	 * @access public
	 * @param string $name
	 * @return mixed, boolean, int
	 */
	 
	public function get_user_id($name){

		$result = $this->get_single_value('uid', CONFIG::sql_tbl_user,
										 "UserName='".$name."'");
		
		if(isset($result)){
			return $result[0]['uid']; 
		}else{
			return FALSE;	
		}
	}

	
// ------------------------------

	/**
	 * Retrieve the actual User Timestamp
	 * 
	 * @access public
	 * @return string
	 */
	 
	public function get_user_timestamp(){

		$result = $this->get_single_value("usr.time_stamp",
											CONFIG::sql_tbl_user." AS usr",
											"(((usr.UserSession)='".session_id()."'))");

		if(is_array($result)){
			return $result[0]['time_stamp'];	
		}
		return FALSE;
	} // get_user_timestamp
	
// ------------------------------

	/**
	 * Set the User Timestamp
	 * 
	 * @access public
	 * @return Boolean
	 */
	 
	public function set_user_timestamp(){

		return $this->update_single_value(CONFIG::sql_tbl_user,
											 "time_stamp='".date("Y-m-d H:i:s")."'",
											"(((UserSession)='".session_id()."'))");
	} // set_user_timestamp
	
// ------------------------------

	/**
	 * Clean User Sessions in user table as well
	 * as in the concurrency control table
	 * 
	 * @access public
	 * @return Boolean
	 */
	 
	public function clean_user_sessions($username=NULL){
		
		//1.) clean up CTC table
		if(($result = $this->get_array("UserSession", 
										CONFIG::sql_tbl_user,
										((CONFIG::dbms == 'mysql')?	
										 ("(TIMESTAMPDIFF(MINUTE, time_stamp, '".date("Y-m-d H:i:s")."') > ".CONFIG::sessionTimeOut):
										 ("(DATEDIFF(minute, time_stamp, '".date("Y-m-d H:i:s")."') > ".CONFIG::sessionTimeOut)).
										 (isset($username)?(") OR (UserName LIKE '".$username."')"):")")
										 )) !== FALSE) {
			foreach($result as &$value) {
				if(isset($value['UserSession']))
					$this->ctc_clean_table($value['UserSession']);
			}
		}
		//2.) clean up the session and timestamp values
		$result = $this->update_single_value(CONFIG::sql_tbl_user,
											"time_stamp=NULL, UserSession=NULL",
											(CONFIG::dbms == 'mysql')?	
											("TIMESTAMPDIFF(MINUTE, time_stamp, '".date("Y-m-d H:i:s")."') > ".CONFIG::sessionTimeOut):
											("DATEDIFF(minute, time_stamp, '".date("Y-m-d H:i:s")."') > ".CONFIG::sessionTimeOut));
				
		//3.) update the own timestamp
		// former bad results will be ignored
		$result = $this->get_core_user();
		//if a user exist for this session, update the session timer
		if(is_array($result) && !empty($result)) 
			return $this->set_user_timestamp();
		else
			return FALSE;
		
	} // clean_user_sessions

// ------------------------------
	/**
	 * Perform LOGIN
	 * 
	 * @access public
	 * @param string $name, string $pass
	 * @return Boolean
	 */
	 
	public function login($name, $pass){
		
		//first clean up the user sessions
		$result = $this->clean_user_sessions($name);
		//now login
		return $this->update_single_value( CONFIG::sql_tbl_user, "UserSession='".session_id()."', time_stamp='".date("Y-m-d H:i:s")."'", "UserName='".$name."' AND UserPass='".md5($pass)."'" );
		
	} // login


// ------------------------------

	/**
	 * Perform LOGOUT
	 * 
	 * @access public
	 * @return Boolean
	 */
	 
	public function logout(){

		//first clean up the user sessions
		if(($result = $this->get_single_value("UserName",
											  CONFIG::sql_tbl_user,
											  "UserSession='".session_id()."'"
										 	  )) !== FALSE) {

			//own session first										 	  	
			$result = $this->clean_user_sessions($result[0]['UserName']);
			//check and clean still hanging ctc sessions
			$result = $this->ctc_clean_table();
			
		}
		//now kill the session										 	 
		return $this->update_single_value( CONFIG::sql_tbl_user, "UserSession=NULL, time_stamp=NULL", "UserSession='".session_id()."'");

	} // logout


// ------------------------------

	/**
	 * Check if the user is logged in
	 * 
	 * @access public
	 * @return mix Boolean, userlevel
	 */
	 
	public function logged_in(){
		$userlevel = '000';
		if($this->check_table(CONFIG::sql_tbl_user) !== FALSE){

			$result =	$this->get_single_value("ull.ul", 
									CONFIG::sql_tbl_ul." AS ull 
									INNER JOIN ".CONFIG::sql_tbl_user." AS usr ON usr.UserSession LIKE '".session_id()."' AND
									usr.time_stamp IS NOT NULL",
									"usr.ulid = ull.ulid");
			if(isset($result) && is_array($result) && !empty($result)) 
				return $result[0]['ul'];
		}
		//DB not jet initialized ??
		return $userlevel;
	} // logged_in

// ------------------------------

	/**
	 * Get all possible userlevel
	 * 
	 * @access public
	 */
	 
	public function get_userlevels($is_user=True){

//		$result = $this->get_single_value('*', CONFIG::sql_tbl_ul);
		$result = $this->get_single_value('ull.*', 
									CONFIG::sql_tbl_ul.' AS ull',
									($is_user?'ull.is_user=1':'1'),
									' ORDER BY ull.ul ASC');
		
		if(is_array($result)){
			return $result;	
		}
		return FALSE;
	} // get_userlevel

// ------------------------------

	/**
	 * Get userlevel description/role
	 * 
	 * @access public
	 */
	 
	public function get_userlevel_role($ul){

		$result = $this->get_single_value('ull.description', 
									CONFIG::sql_tbl_ul.' AS ull',
									'ull.ul LIKE '.$ul,
									' ORDER BY ull.ul ASC');

		if(isset($result) && is_array($result) && !empty($result)) 
			return $result[0]['description'];
		else
			return NULL;
	} // get_userlevel_role
}
?>