<?php
/**
 * This modules contains several utilities
 *
 * @package jrCMS: Generic Functions
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: jrcmsutilities.php 607 2011-09-20 05:58:51Z jensraabe $ 
 *
 * @copyright Copyright (c) 2011, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

// ------------------------------
/**
 * @access public
 * @param string 
 * @param string 
 * @return string
 * 
 * Convert parameter for SQL queries
 *  
 */
function sql_quoted ($value,$type){
	switch(strtoupper($type)){
	  case 'BLOB'     :
	  case 'VARCHAR'  :
	  case 'CHAR'     :
	  	if($value === ''){   return 'NULL';                                        }
	  	else {              return "'" . $value . "'";   }
	  	break;
	  case 'NUMBER'   :
	  default         :
	  	if ( $value === 0 ) {  return $value;   }
	  	elseif ( $value === '' || $value === null) {
	  		return 'NULL';
	  	}
	  	else {
	  		return $value;
	  	}
	}
}

// ------------------------------
/**
 * The function replaces HTML representations of special chars
 * ###deprecated###
 * 
 * @access public
 * @param string 
 * @return string
 */
function replaceChar($text)
{
	$search  = array ('', '', '', '', '', '', '');
	$replace = array ('&auml;', '&ouml;', '&uuml;', '&szlig;', '&Auml;', '&Ouml;', '&Uuml;');
	$str  = str_replace($search, $replace, $text);
//	$str = html_entity_decode($text);
	return $str;
}

// ------------------------------
/**
 * The function replaces special characters by HTML strings
 * ###deprecated###
 * 
 * @access public
 * @param string 
 * @return string
 */
function replaceCharSeq($text)
{
	$replace  = array ('', '', '', '', '', '', '');
	$search = array ('&auml;', '&ouml;', '&uuml;', '&szlig;', '&Auml;', '&Ouml;', '&Uuml;');
	$str  = str_replace($search, $replace, $text);
//	$str = htmlentities($test);
	return $str;
}

// ------------------------------
/**
 * By using GET requests with AJAX the the special char '&' have to be replaced
 * The function converts the string back.
 * ###deprecated###
 * 
 * @access public
 * @param string 
 * @return string
 */
function replaceInputCharSeq($text)
{
	$search  = array ('__auml;', '__ouml;', '__uuml;', '__szlig;', '__Auml;', '__Ouml;', '__Uuml;');
	$replace = array ('&auml;', '&ouml;', '&uuml;', '&szlig;', '&Auml;', '&Ouml;', '&Uuml;');
	$str  = str_replace($search, $replace, $text);
	return $str;
}

// ------------------------------
/**
 * replaces specific unicode numbers by charachter
 * 
 * @access public
 * @param string 
 * @return string
 */
function entities2char($text)
{
	/*
	&#34; //"
	&#39; //'
	&#128; //
	&#196; //
	&#214; //
	&#220; //
	&#223; //
	&#246; //
	&#252; //
	*/

	$search  = array ('&#34;', '&#39;', '&#128;', '&#196;', '&#214;', '&#220;', '&#223;', '&#246;', '&#252;');
	//$replace = array ('', '', '', '', '', '', '');
	$replace = array ('"', '\'', 'ae', 'Ae', 'Oe', 'Ue', 'ss', 'oe', 'ue');
	$str  = str_replace($search, $replace, $text);
	return $str;
}

// ------------------------------
/**
 * Used in Menu functions to repeat HTML sequences
 * 
 * @access public
 * @param string (e.g. HTML to display)
 * @param integer 
 * @return string
 */
function strRepeat($str,$no){
	$hstr = $str;
	for($ctr=1; $ctr < $no; $ctr++) {
		$hstr.=$str;
	}	
	return $hstr;
}

// ------------------------------
/**
 * @access public
 * @param string  (function to be calles)
 * @param integer (no of entries)
 * @param integer (max no of lines)
 * @param string  (parameters)
 * 
 * @return string
 * 
 * 
 */
function recordSetToHtml($recordSet=NULL, $function=NULL, $jsParam=NULL) {
	if (isset($recordSet)){
		//pageList($function, )
		
//		$arParam = array();
//		$arParam = json_decode($jsParam);
//		if (!empty($arParam)) 
//			print_r($arParam);
//		$newarParam[] = array ("filter" => "true");
//		if (!empty($arParam)) 
//			print_r($newarParam);
//		$urlParam = urlencode(json_encode($arParam));
		
		$html = '<form id="frmTable"><table>';
		foreach ($recordSet as $indx => $record) {
			if ($indx == 0) {
				$html.= '<tr>';
				foreach (array_keys($record) as $column) 
					$html.= '<th  class="tdHeader">'.$column.'</th>';
				$html.= '</tr><tr>';
				// TODO: define size of input field dynamically, against TD-width in px or %  
//								<input '.(isset($function) ? 'onKeyUp="'.$function.'('.str_replace("}", ",filter:true}", $param).', this);"' : '').' 
//								<input '.(isset($function) ? 'onKeyUp="'.$function.'('.$urlParam.', this, true);"' : '').' 
				foreach (array_keys($record) as $column) 
					$html.= '<td class="tdFilter">
								<input '.(isset($function) ? 'onKeyUp="'.$function.'('.str_replace("}", ",filter:true}", $jsParam).', this);"' : '').' 
									id="'.$column.'" 
									name="'.$column.'" 
									type="text" 
									size="'.strlen($column).'"
								>
							</td>';
				$html.= '</tr>';
			} else
				$html.= '<tr>';
			foreach ($record as $value)
				$html.= '<td class="tdContent'.($indx % 2).'">'.$value.'</td>';
			$html.= '</tr>';
		}
		$html.= '</table></form>';
		//echo $html;
		return $html;
	} else {
		return NULL; // wrong parameter
	}
}
	
// ------------------------------

/**
 * @access public
 * @param string  (function to be calles)
 * @param integer (no of entries)
 * @param integer (max no of lines)
 * @param string  (parameters)
 * 
 * @return string
 * 
 * 
 */
function pageList1() {
	
}

// ------------------------------

/**
 * @access public
 * @param string  (function to be calles)
 * @param integer (no of entries)
 * @param integer (max no of lines)
 * @param string  (parameters)
 * 
 * @return string
 * 
 * 
 */
function pageList($func, $entries, $maxlines, $parm=NULL) {
	$sel_parm = NULL;
	
	if (isset($parm)){
		$sel_parm = $parm.',';
	}

	$strHtml = '<div id="pageselect">';
		$start = 0;
		$page = 1;
		$limit = ($entries < $maxlines) ? $entries : $maxlines;
		$strHtml.= 'Page: ';
		$strHtml.= '<span class="nav" name="page" onclick="'.$func.'({'.$sel_parm.'start:'.$start.',limit:'.$limit.'}); return false;">< </span>';
		$strHtml.= '<span class="nav" name="page" onclick="'.$func.'({'.$sel_parm.'start:'.$start.',limit:'.$limit.'}); return false;">'.$page.'</span>';
                $entries = $entries - $limit;
		while( $entries > 0 ) {
			$page++;
			$start = $start + $limit;
			$strHtml.= '<span class="nav" name="page" onclick="'.$func.'({'.$sel_parm.'start:'.$start.',limit:'.$limit.'}); return false;">, '.$page.'</span>';
                        $entries = $entries - $limit;
                        //$strHtml.= ($entries > 0) ? ',':'';
                }		
		$strHtml.= '<span class="nav" name="page" onclick="'.$func.'({'.$sel_parm.'start'.$start.',limit'.$limit.'}); return false;"> ></span>';
	$strHtml.= '</div>';

	return $strHtml;
}

// -----------------------
	/**
 	* transfer date format (yyyy-dd-mm => dd.mm.)
 	* 
 	* @access public
 	* @param string 
 	* @return string
 	*/
	function dateDDMM($dat){
		return substr($dat, 8, 2).'.'.substr($dat, 5, 2).'.';
	}

// -----------------------
//	/**
// 	* transfer date format (yyyy-dd-mm => dd.mm.yyyy)
// 	* 
// 	* @access public
// 	* @param string 
// 	* @return string
// 	*/
//	function dateDDMMYYYY($dat){
//		return substr($dat, 8, 2).'.'.substr($dat, 5, 2).'.'. substr($dat, 0, 4);
//	}
	
// -----------------------
	/**
	 * @access public
	 * 
	 * @return string
	 * 
	 */
	function list_files_by_ext($extension, $path){
    $list = array(); //initialise a variable
    $dir_handle = @opendir($path) or die("Unable to open $path"); //attempt to open path
    while($file = readdir($dir_handle)){ //loop through all the files in the path
        if($file == "." || $file == ".."){continue;} //ignore these
        $filename = explode(".",$file); //seperate filename from extenstion
        $cnt = count($filename); $cnt--; $ext = $filename[$cnt]; //as above
        if(strtolower($ext) == strtolower($extension)){ //if the extension of the file matches the extension we are looking for...
            array_push($list, $file); //...then stick it onto the end of the list array
        }
    }
    if(isset($list[0])){ //...if matches were found...
    	return $list; //...return the array
    } else {//otherwise...
    	return false;
    }
	}
	
// -----------------------
	/**
	 * @access public
	 * 
	 * @return string
	 * 
	 */
	function list_jpg_files($dirname, $dironly=False, $pattern="(\.jpg$)|(\.jpeg$)", $exclusion_pattern=NULL) { 
		$imgArray = Array();
		//$pattern="(\.jpg$)|(\.jpeg$)"; //valid image extensions
		if($handle = opendir($dirname)) { 
			while(false !== ($file = readdir($handle))){
				//validation
				//$allowed = "/[^a-z0-9\\040]/i"; preg_replace($allowed,"",$str);
				if (!isset($exclusion_pattern) || 
					(isset($exclusion_pattern) && preg_match($exclusion_pattern,$file)<1)) {
					if (is_dir("$dirname/$file") && $file!='.' && $file!='..') {
						$imgLoopArray = list_jpg_files("$dirname/$file", $dironly, $pattern, $exclusion_pattern);
						if(is_array($imgLoopArray) && !empty($imgLoopArray)) $imgArray = array_merge($imgArray, $imgLoopArray);
					} else {
						if(eregi($pattern, $file)){ //if this file is a valid image 
							$imgArray[] = $dironly 
											? str_replace('//','\\',$dirname)
											: str_replace('//','\\',$dirname)."/".$file;
							$imgArray = array_unique($imgArray);
						}
					}
				}	
			}
			closedir($handle); 
		}
		return ($dironly ? array_unique($imgArray) : $imgArray);
	} 
	

// -----------------------
	/**
	 * @access public
	 * 
	 * @return string
	 * 
	 */
	$listThumbnailFilesDir = null;
	function list_thumbnail_files($dirname, $thumbDir = '.thumbs', $exclusion_pattern = NULL) {
		//assign parameter
		global $listThumbnailFilesDir;
		$listThumbnailFilesDir = $thumbDir;
		
		function arrayThumbFilter($var){
			global $listThumbnailFilesDir;
    		return (strpos($var,$listThumbnailFilesDir)!==False);
		}
		$img_array = list_jpg_files($dirname, False, "(\.jpg$)|(\.jpeg$)|(\.png$)|(\.gif$)", $exclusion_pattern);
		$img_array = array_filter($img_array, "arrayThumbFilter");
		$img_array = array_merge($img_array);
		return $img_array;
		
	} 
	
	
// -----------------------
	function getTime(){
		$timestamp = time();
		$d = date("d.m.Y",$timestamp);
	  	$cl = date("H:i",$timestamp);
	 	return ("> ".$d."-".$cl." ");
	}

// -----------------------
	function getDateSQL(){
		$timestamp = time();
		return date("Y-m-d",$timestamp);
	}
	
// ------------------------------
	/**
	 * @access public
	 * 
	 * @return string
	 * 
	 */
 	function removeTagsWContent($s , $expand = 'script|style|noframes|select|option'){
		/**///remove tags with content between them
        if(strlen($expand) > 0){
            $e = explode('|',$expand);
            for($i=0;$i<count($e);$i++){
                while(stripos($s,'<' . $e[$i]) !== False && stripos($s,'<' . $e[$i]) >= 0){
                    $len[1] = strlen('<' . $e[$i]);
                    $pos[1] = stripos($s,'<' . $e[$i]);
                    $pos[2] = stripos($s,$e[$i] . '>', $pos[1] + $len[1]);
                    $len[2] = $pos[2] - $pos[1] + $len[1];
                    $x = substr($s,$pos[1],$len[2]);
                    $s = str_replace($x,'',$s);
                }
            }
        }
        return $s;
    }
    
	/**
	 * taken from php.net
	 * This could be overkill but this strips all HTML tags and 
	 * gives you the option to preserve the ones you define. 
	 * It also takes into account tags like <script> removing all the javascript, too! 
	 * You can also strip out all the content between any tag that has an opening 
	 * and closing tag, like <table>, <object>, etc.
	 * Have fun. Let me know what you think. http://zt.a.atr.im
	 * 
	 */
    function removeHTML($s , $keep = '' , $expand = 'script|style|noframes|select|option'){
        /**///prep the string
        $s = ' ' . $s;
       
        /**///initialize keep tag logic
        if(strlen($keep) > 0){
            $k = explode('|',$keep);
            for($i=0;$i<count($k);$i++){
                $s = str_replace('<' . $k[$i],'[{(' . $k[$i],$s);
                $s = str_replace('</' . $k[$i],'[{(/' . $k[$i],$s);
            }
        }
       
        //begin removal
        /**///remove comment blocks
        while((stripos($s,'<!--') > 0) && (stripos($s,'-->', stripos($s,'<!--'))!==False)){
            $pos[1] = stripos($s,'<!--');
            $pos[2] = stripos($s,'-->', $pos[1]);
            $len[1] = $pos[2] - $pos[1] + 3;
            $x = substr($s,$pos[1],$len[1]);
            $s = str_replace($x,'',$s);
        }
       
		$s = removeTagsWContent($s , $expand);
       
        /**///remove remaining tags
        while((stripos($s,'<') > 0) && (stripos($s,'>', stripos($s,'<'))!==False)){
            $pos[1] = stripos($s,'<');
            $pos[2] = stripos($s,'>', $pos[1]);
            $len[1] = $pos[2] - $pos[1] + 1;
            $x = substr($s,$pos[1],$len[1]);
            $s = str_replace($x,'',$s);
        }
       
        /**///finalize keep tag
        for($i=0;$i<count($k);$i++){
            $s = str_replace('[{(' . $k[$i],'<' . $k[$i],$s);
            $s = str_replace('[{(/' . $k[$i],'</' . $k[$i],$s);
        }
       
        return trim($s);
    }
	