<?php
/**
 * This module processes all tasks regarding image show
 *
 * @package jrCmsImgShow
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: jrcmsimgshow.php 531 2011-05-27 07:23:11Z jensraabe $
 *  
 * @copyright Copyright (c) 2008,2009,2010 Jens Raabe
 * license: MIT-style license, see jrcms.js
 */
 
// set some headers values
header("Last-Modified: " . gmdate( "D, d M Y H:i:s" ) . " GMT" ); 
header("Content-Type: text/plain; charset=utf-8");
header("Connection: close");
// set header
session_cache_limiter( 'nocache' ); //Expires, Cache-Control, Pragma
// start session
session_start(); 
// usage of classes
require_once('../../../conf/cConfig.php');
//global language values
CONFIG::loadLanguage();
//language support of plugin
require_once(('lang/lang.'.$_SESSION["language"]));
$lang_php = array_merge($lang_mnu, $lang_php);
CONFIG::loadClasses();
CONFIG::loadScripts();
// retrieve parameter
if(isset($_GET['a'])) {$act = $_GET['a'];} else {if(isset($_POST['a'])) {$act = $_POST['a'];} else {$act=NULL;}}
define("sql_tbl_imgShowConf", 'web_imageshowconf');
define("sql_tbl_imgShowData", 'web_imageshowdata');

// ------------------------------
// Database routines
// ------------------------------

	/**
 	 * get image show data for a content id or menu id
 	 * 
 	 * @param int  	- table id
 	 * @param string - type of id menu/imageShow (default 'menu')
 	 * @access private
 	 * @return array
 	 */

	function getImgShowConfDB($api, $id, $idType='imageShow') {
		
		// SELECT is.* FROM web_imageShowConf AS is
		// INNER JOIN CONFIG::sql_tbl_menu AS m ON m.id=$id    
		// WHERE m.cid = is.cid;
		return ($idType == 'menu')
				? $api->getSingleValueFromDb('i.*',
											sql_tbl_imgShowConf.' AS i
											INNER JOIN '.CONFIG::sql_tbl_menu.' AS m ON m.id='.$id,
											'm.cid = i.cid')
				: $api->getSingleValueFromDb('*',sql_tbl_imgShowConf,'cid= '.$id);
		
	} // getImgShowConfDB
	
	// ------------------------------
	/**
 	 * create image show data
 	 * 
 	 * @param
 	 * @access private
 	 * @return Id / boolean
 	 */

	function createImgShowConfDB($api, $id, $showType='standard',
  								$height=Null,$width=Null,$hu='content/uploads',$linked=0) {
  									
  		//determine the content Id
  		if(isset($id)) 
  			if(($result = $api->getSingleValueFromDb('m.cid', CONFIG::sql_tbl_menu.' AS m', 'm.id='.$id)) !== False) {
  				//at first verify if there exist an old entry which has to be reused
  				if(($result2 = getImgShowConfDB($api, $id, 'menu')) !== False && !empty($result2)) {
  					return $result2[0]['isid'];
  				} else {
  					return $api->insertSingleValueInDb(sql_tbl_imgShowConf,
											"cid, showType, height, width, hu, linked, delay, duration, left, top",
  											(int)$result[0]['cid'].
  											",'".addslashes((string)$showType)."','".addslashes((string)$height).
  											"','".addslashes((string)$width)."','".addslashes((string)$hu).
											"',".(int)$linked.",2000,2000,0,0", True);
  				}
  			}
  		return False; 									
  									
  	} // createImgShowConfDB
  	
	// ------------------------------
	/**
 	 * update image show data
 	 * 
 	 * @param 
 	 * @access private
 	 * @return boolean
 	 */

	function updateImgShowConf($api,$isid,$showType='standard',
  								$captions=0,$controler=0,$thumbnails=0,
  								$random=0,$titles=1,$linked=0,
  								$delay=Null,$duration=Null,$color=Null,
  								$left=Null,$top=Null,$height=Null,
  								$width=Null,$hu='content/uploads',$showSAction=Null) {

  		//remark: left must be tagged because of sql keyword
  		if(isset($isid))
  			return $api->updateSingleValueOfDb(sql_tbl_imgShowConf,
  												"showType = '".$showType."'
  												,captions = ".$captions."
  												,controler = ".$controler."
  												,thumbnails = ".$thumbnails."
  												,random = ".$random."
  												,titles = ".$titles."
  												,linked = ".$linked
  												.(isset($delay) ? (", delay = ".$delay) : (""))
  												.(isset($duration) ? (", duration = ".$duration) : (""))
  												.(isset($color) ? (", color = '".$color."'") : (""))
  												.(isset($left) ? (", `left` = '".$left."'") : (""))
  												.(isset($top) ? (", top = '".$top."'") : (""))
  												.(isset($height) ? (", height = '".$height."'") : (""))
  												.(isset($width) ? (", width = '".$width."'") : (""))
  												.(isset($hu) ? (", hu = '".$hu."'") : (""))
  												.(isset($showSAction) ? (", showSAction = '".$showSAction."'") : (""))
  												,"isid = ".$isid);
 		
	} // updateImgShowConf
	
	// ------------------------------
	/**
 	 * delete image show data
 	 * 
 	 * @param
 	 * @access private
 	 * @return boolean
 	 */

	function deleteImgShowConf($api, $isid) {
  									
		return $api->deleteSingleValueOfDb("i.*", sql_tbl_imgShowConf." AS i", "i.isid = ".$isid);
		
  	}

	// ------------------------------
	/**
 	 * get image list of isid
 	 * 
 	 * @param int $isid, object $api
 	 * @access private
 	 * @return array
 	 */

	function getImgShowData($api, $isid) {
		
		// SELECT * FROM web_imageShowData WHERE isid = $isid;
		return $api->getArrayFromDb('*',sql_tbl_imgShowData,'isid= '.$isid);
		
	} // getImgShowData

	// ------------------------------
	/**
 	 * replace images
 	 * 
 	 * @param
 	 * @access private
 	 * @return boolean
 	 */

	function replaceImages($api, $isid, $imgArray) {
		
		//first simple solution delete all images
		if(($result = deleteImages($api, $isid)) === True){
			foreach($imgArray as $key => $value){
				//remark: one entry of imgArray['img', 'caption', 'href']
				$result&= $api->replaceSingleValueInDb(sql_tbl_imgShowData,
											"isid, img"
											.(isset($value['caption'])?(", caption"):(NULL))
											.(isset($value['href'])?(", href"):(NULL)),
											(int)$isid."
											, '".addslashes((string)$value['img'])."'"
											.(isset($value['caption'])?(", '".addslashes((string)$value['caption'])."'"):(NULL))
											.(isset($value['href'])?(", '".addslashes((string)$value['href'])."'"):(NULL)));
			}
		}
		return $result;
  	}
	
	// ------------------------------
	/**
 	 * delete images
 	 * 
 	 * @param
 	 * @access private
 	 * @return boolean
 	 */

	function deleteImages($api, $isid=NULL) {
		
		return (isset($isid)
				? $api->deleteSingleValueOfDb("d.*", sql_tbl_imgShowData." AS d", "d.isid = ".$isid)
				: False);		
	}
  	
	// ------------------------------
	/**
 	 * get used show types
 	 * 
 	 * @param
 	 * @access private
 	 * @return boolean
 	 */

	function getUsedShowTypesDB($api) {
		
		return $api->getArrayFromDb('DISTINCT i.showType',sql_tbl_imgShowConf." AS i");		
	}
	
// ------------------------------
// General routines
// ------------------------------

		
	/**
 	 * get image show data and image list for a defined content id
 	 * 
 	 * @param int $cid
 	 * @access private
 	 * @return array
 	 */

	function getImgShowDataSet($api, $id, $xml=False) {
		
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		//******** Determine content Id ********************
		//$result = $api->getSingleValueFromDb('cid',CONFIG::sql_tbl_menu,'id = '.$id);

		//******** Image Show Configuration Data ***********
		//2.1 ensure presence of table
		if($ul == CONFIG::webmasterLevel && ($api->checkTable(sql_tbl_imgShowConf) !== True)){
			require_once('jrcmsimgshowinit.php');
			if(!initImgShowDb())
				throw new MyException('jrcmsimgshow.getImgShowDataSet DB wrong initialized!');
		}
		//2.2 retrieve the config data of image show
		//$result = getImgShowConfDB($api, $result[0]['cid']);
		$result = getImgShowConfDB($api, $id, 'menu');

		//2.3 build response structure
		if(is_array($result) && !empty($result)) {
			//access rights
			$ar = $api->checkAccessRight($id, (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000'), 'write', CONFIG::webmasterLevel);
			//skip all NULL fields and perform some convertion of values
			foreach($result[0] as $key => $value){
				if (isset($value)) {
					switch($key) {
						case "isid":		//provided as header 
						case "replace":		//not yet supported by DB
							//json_decode($value);
							break;
						case "color":
							$isDialogData[] = $api->dialogElement($key, json_decode($value), 'imgShowDialog', 'hidden');
							break;
						case "controler":	//patch wrong configuration of DB
							$isDialogData[] = $api->dialogElement('controller', ($value==1), 'imgShowDialog', 'input:jrCmsImgShow');
							break;
						case "captions":
						case "thumbnails":
						case "random":
						case "titles":
							$isDialogData[] = $api->dialogElement($key, ($value==1), 'imgShowDialog', 'input:jrCmsImgShow');
							break;
						case "linked":
							$isDialogData[] = $api->dialogElement($key, ($value==1), 'imgShowDialog', 'hidden');
							break;
						case "left":
						case "top":
						case "height":
						case "width":
						case "duration":
						case "delay":
							$isDialogData[] = $api->dialogElement($key, ((intval($value) != 0) && (strpos($value, '%')===False)?intval($value):$value), 'imgShowDialog', 'input', 'validate-integer');
							break;
						case "showType":
							$isDialogData[] = $api->dialogElement($key, $value, 'imgShowDialog', 'select:jrCmsImgShow', 'required');
							break;
						case "hu":
							$isDialogData[] = $api->dialogElement($key, $value, 'imgShowDialog', 'select:jrCmsImgShow', 'required');
							break;
						default:
							//showSAction, cid
							if((intval($value) != 0) && (strpos($value, '%')===False)) 
								$isDialogData[] = $api->dialogElement($key, intval($value), 'imgShowDialog', 'hidden');
							else
								$isDialogData[] = $api->dialogElement($key, $value, 'imgShowDialog', 'hidden');
					}
				}
			}
			// provide the id as well
			$isDialogData[] = $api->dialogElement('id', $id, 'imgShowDialog', 'hidden');
			// we use the same thumbnail directory and prefix as for the xinha editor
			$isDialogData[] = $api->dialogElement('replace', CONFIG::thumbSubDir.'/'.CONFIG::thumbPrefix, 'imgShowDialog', 'hidden');
			$isid = $result[0]['isid'];
			
			// retrieve the images
			$result = getImgShowData($api, $isid);
			if(is_array($result)) {
				foreach($result as &$value){
					$imgset = NULL;
					if(isset($value['caption']))$imgset['caption'] = $value['caption'];
					if(isset($value['href']))$imgset['href'] = $value['href'];
					if(isset($imgset)) $data[$value['img']] = $imgset;
					else $data[] = $value['img'];
				}
			} else {
				$data = Array();
			}
		} else {
			$isDialogData = Array();
			$isid = NULL;
			$data = Array();
		}	
		// dataset
		$dataset = array('jr_imgShowDialog' => array(
							'header' => $api->dialogHeader('isid', $isid),
				 			'data' => $isDialogData),
						 'jr_imgShowData' => array(
							'images' => $data));
		$jsonStr = json_encode($dataset);
//		return $jsonStr;
//print_r($dataset);		
		return ($xml ? $api->array2Xml($dataset) : json_encode($dataset));
	}
	

// ------------------------------
// Image show
// ------------------------------

	/**
 	 * provides menu structure as Html list 
 	 * 
 	 * @param global
 	 * @access global
 	 * @return html string
 	 */
	function getImgShow($api, $task){
		
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		if(isset($_GET['root'])) {$root = $_GET['root'];} else {if(isset($_POST['root'])) {$root = $_POST['root'];} else {$root=NULL;}}
		if(isset($_GET['id'])) {$id = $_GET['id'];} else {if(isset($_POST['id'])) {$id = $_POST['id'];} else {$id=-1;}}
		if(isset($_GET['cssid'])) {$cssid = $_GET['cssid'];} else {if(isset($_POST['cssid'])) {$cssid = $_POST['cssid'];} else {$cssid=-1;}}
		if(isset($_GET['div'])) {$div = $_GET['div'];} else {if(isset($_POST['div'])) {$div = $_POST['div'];} else {$div=CONFIG::divHMenu;}}

		//provided as xml
		return getImgShowDataSet($api, (isset($_POST['id']) ? $_POST['id'] : Null ), True);
				
	} //getImgShow
	
	// ------------------------------

	/**
 	 * create image show header data 
 	 * 
 	 * @param global
 	 * @access global
 	 * @return html string
 	 */
	function createImgShow($api){
		
		global $lang_php;
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		try{

			//ensure presence of table
			if($ul == CONFIG::webmasterLevel && ($api->checkTable(sql_tbl_imgShowConf) !== True)){
				require_once('jrcmsimgshowinit.php');
				if(!initImgShowDb())
					throw new MyException('jrcmsimgshow.getImgShowDataSet DB wrong initialized!');
			}
			
			$result = createImgShowConfDB($api,
					(isset($_POST['id']) ? $_POST['id'] : Null ),
					(isset($_POST['showType']) ? $_POST['showType'] : 'standard' ),
					(isset($_POST['height']) ? $_POST['height'] : Null ),
					(isset($_POST['width']) ? $_POST['width'] : Null ),
					(isset($_POST['hu']) ? $_POST['hu'] : 'content/uploads' ),
					(isset($_POST['linked']) ? $_POST['linked'] : 0 ));
		
			//up to now the dialog only provides the processing of jacascript and 
			//therefore we have build an inline script
			if($result !== False) {
				insertImages($api, $result); 
				$data = getImgShowDataSet($api, (isset($_POST['id']) ? $_POST['id'] : Null ));
				$strScript = '<script type="text/javascript">';
//				$strScript.= 'var iS = jrCmsImgShow.imgShowOptions(\''.$data.'\');';
				//data are provided as object
				$strScript.= 'var iS = jrCmsImgShow.imgShowOptions('.$data.');';
				$strScript.= 'jrCmsImgShow.setOptions({\'jr_imgShow\': iS});';
				$strScript.= 'jrCmsImgShow.updateImgShowInlineScript({a:\'createImgShow\',id:'.(isset($_POST['id']) ? $_POST['id'] : 0 ).'});';
				$strScript.= 'jrCmsEditor.save();';
				$strScript.= '</script>';
				return $strScript;				
			} else
				return '<script type="text/javascript">alert("'.$lang_php['i_data_insert_failure'].'");</script>';
			
		}catch(MyException $e){
			$e->log();
			return "[jrcmsimgshow.createImgShow] failure!"; //TODO:		
		}
	} // createImgShow

	// ------------------------------

	/**
 	 * update image show header data 
 	 * 
 	 * @param global
 	 * @access global
 	 * @return html string
 	 */
	function updateImgShow($api){
		
		global $lang_php;
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		try{
			$result=true;
			$result = updateImgShowConf($api,
					(isset($_POST['isid']) ? $_POST['isid'] : Null ),
					(isset($_POST['showType']) ? $_POST['showType'] : 'standard' ),
					(isset($_POST['captions']) ? $_POST['captions'] : 0 ),
					(isset($_POST['controller']) ? $_POST['controller'] : 0 ),
					(isset($_POST['thumbnails']) ? $_POST['thumbnails'] : 0 ),
					(isset($_POST['random']) ? $_POST['random'] : 0 ),
					(isset($_POST['titles']) ? $_POST['titles'] : 0 ),
					(isset($_POST['linked']) ? $_POST['linked'] : 0 ),
					(isset($_POST['delay'])? (int)$_POST['delay'] : 2000 ),
					(isset($_POST['duration']) ? (int)$_POST['duration'] : 2000 ),
					(isset($_POST['color']) ? json_encode($_POST['color']) : Null ),	//not supported
					(isset($_POST['left']) ? $_POST['left'] : 0 ),
					(isset($_POST['top']) ? $_POST['top'] : 0 ),
					(isset($_POST['height']) ? $_POST['height'] : Null ),
					(isset($_POST['width']) ? $_POST['width'] : Null ),
					(isset($_POST['hu']) ? $_POST['hu'] : 'content/uploads' ),
					(isset($_POST['showSAction']) ? $_POST['showSAction'] : Null ));
			
			if($result) {
				//at first we have to ensure the images are up to date
				//insert images will replace the images assigned
				insertImages($api, (isset($_POST['isid']) ? $_POST['isid'] : Null ));
				//next build an answer				
				//up to now the dialog only provides the processing of jacascript and 
				//therefore we have build an inline script
				$data = getImgShowDataSet($api, (isset($_POST['id']) ? $_POST['id'] : Null ));
				$strScript = '<script type="text/javascript">';
//				$strScript.= 'var iS = jrCmsImgShow.imgShowOptions(\''.$data.'\');';
				//data are provided as object
				$strScript.= 'var iS = jrCmsImgShow.imgShowOptions('.$data.');';
				$strScript.= 'jrCmsImgShow.setOptions({\'jr_imgShow\': iS});';
//				$strScript.= 'alert("'.$lang_php['i_data_updated'].'");';
				$strScript.= 'jrCmsEditor.save();';
				$strScript.= '</script>';
				return $strScript;				
			} else
				return '<script type="text/javascript">alert("'.$lang_php['i_data_update_failure'].'");</script>';
			
		}catch(MyException $e){
			$e->log();
			return "[jrcmsimgshow.updateImgShow] failure!"; //TODO:		
		}
	} // updateImgShow
	
	// ------------------------------

	/**
 	 * delete image show 
 	 * 
 	 * @param global
 	 * @access global
 	 * @return html string
 	 */
	function deleteImgShow($api){
		
		global $lang_php;
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		try{
			//1) delete all images in DB
			if(deleteImages($api, (isset($_POST['isid']) ? $_POST['isid'] : Null )))
				//2) delete the conf data
				if(deleteImgShowConf($api, (isset($_POST['isid']) ? $_POST['isid'] : Null ))){
					//up to now the dialog only provides the processing of jacascript and 
					//therefore we have build an inline script
					$strScript = '<script type="text/javascript">';
					$strScript.= 'jrCmsImgShow.setOptions({\'jr_imgShowDialog\': undefined});';
					$strScript.= 'jrCmsImgShow.updateImgShowInlineScript({a:\'deleteImgShow\',id:'.(isset($_POST['id']) ? $_POST['id'] : 0 ).'});';
					$strScript.= 'jrCmsEditor.save();';
					$strScript.= '</script>';
					return $strScript;				
				}
			throw new MyException('wrong isid or db failure!');
		
		}catch(MyException $e){
			$e->log();
			return "[jrcmsimgshow.deleteImgShow] failure!"; //TODO:		
		}
	} // deleteImgShow
	
	// ------------------------------
	
	/**
 	 * insert images 
 	 * 
 	 * @param
 	 * @access
 	 * @return 
 	 */
	function insertImages($api, $isid = Null){
		
		if(!isset($isid))
			if(isset($_GET['isid'])) {$isid = $_GET['isid'];} else {if(isset($_POST['isid'])) {$isid = $_POST['isid'];} else {$isid=Null;}}
		if(isset($_GET['hu'])) {$imgPath = $_GET['hu'];} else {if(isset($_POST['hu'])) {$imgPath = $_POST['hu'];} else {$imgPath='content/uploads';}}
		if(!isset($isid)) return false;
		
		// as first simple solution:
		// try to update the DB on base of the directory content
		// no further maintenance is provided
		$imgArray = Array();
		$imgPath = $_SERVER['DOCUMENT_ROOT'].CONFIG::docPath.'/'.$imgPath;
		$jpg_files = list_files_by_ext("jpg", $imgPath);
		
		foreach ($jpg_files as $filename) {
			//do not use special files like resized files of Xinha
			if(!strrpos($filename, "/."))
				$imgArray[] = Array('img' => $filename, 'caption' => str_replace(".jpg","",$filename), 'href' => Null);
		}
		
		replaceImages($api,$isid,$imgArray);
		
	} // insertImages
	
	// ------------------------------
	
	/**
 	 * get first images 
 	 * 
 	 * @param
 	 * @access
 	 * @return 
 	 */
	function getFirstImage($api){
		
		if(isset($_GET['hu'])) {$imgPath = $_GET['hu'];} else {if(isset($_POST['hu'])) {$imgPath = $_POST['hu'];} else {$imgPath='content/uploads';}}

		$imgArray = Array();
		$imgPath = $_SERVER['DOCUMENT_ROOT'].CONFIG::docPath.'/'.$imgPath;
		$jpg_files = list_files_by_ext("jpg", $imgPath);
		
		foreach ($jpg_files as $filename) {
			//do not use special files like resized files of Xinha
			if(!strrpos($filename, "/."))
				$imgArray[] = $filename;
		}
		$json = '{"jr_imgShowFirstImage":';
		$json.= '"'.(isset($_GET['hu'])?$_GET['hu']:$_POST['hu']).'/'.$imgArray[0].'"';
		$json.= '}';
		return $json;
		
	} // insertImages
	
	// ------------------------------
	
	/**
 	 * get image directories 
 	 * 
 	 * @param
 	 * @access
 	 * @return 
 	 */
	function getImageDirs(){
		
		//function to alter the directory array
		function alterDir(&$item, $key){
			$item = str_replace($_SERVER['DOCUMENT_ROOT'].CONFIG::docPath.'/','',$item);
		}
		$imgDirArray = list_jpg_files($_SERVER['DOCUMENT_ROOT'].CONFIG::docPath.CONFIG::uploadDIR, True);
		array_walk($imgDirArray, 'alterDir');
		$json = '{"jr_imgDirs":';
		$json.= json_encode($imgDirArray);
		$json.= '}';
		return $json;	
	}
	
	// ------------------------------
	
	/**
 	 * get used Image Show types 
 	 * 
 	 * @param
 	 * @access
 	 * @return 
 	 */
	function getShowTypes($api){

		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		//ensure presence of table
		if($ul == CONFIG::webmasterLevel && ($api->checkTable(sql_tbl_imgShowConf) !== True)){
			require_once('jrcmsimgshowinit.php');
			if(!initImgShowDb())
				throw new MyException('jrcmsimgshow.getImgShowDataSet DB wrong initialized!');
		}
		
		if(($result = getUsedShowTypesDB($api)) !== False) {
			return $api->array2Xml($result);
		}

	}

// ############## MAIN ##############
// calling functions of jrcmsimgshow.php
$api = new JRCMSAPI();
if (isset($act)) {
	switch ($act) {
	case "getImgShow":
	case "editImgShow":
	case "getPreviewImgShow":
		// retrieves all data and provide a xml container
		$e = getImgShow($api, $act);
		break;
	case "createImgShow":
		$e = createImgShow($api);
		break;
	case "updateImgShow":
		// updates configuration and performs and editImgShow task
		$e = updateImgShow($api);
		break;
	case "deleteImgShow":
		$e = deleteImgShow($api);
		break;
	case "insertImages":
		$e = insertImages($api);
		break;
	case "getImageDirs":
		$e = getImageDirs();
		break;
	case "getShowTypes":
		$e = getShowTypes($api);
		break;	
	case "getFirstImage":
		$e = getFirstImage($api);
		break;	
	default: 
		print_r("jrcmsimgshow.php with wrong action called!!");
	}

} else {

	print_r("jrcmsimgshow.php with wrong parameter called!!");
	
}
$api = NULL;
echo $e;
flush(); // send really out all stuff
?>