<?php
/**
 * This module initializes the database used by the jrCmsImgShow
 *
 * @package jrCMS: jrCmsImgShow
 * @author Jens Raabe
 * @version $Id: jrcmsimgshowinit.php 534 2011-05-31 06:30:31Z jensraabe $
 *  
 * @copyright Copyright (c) 2008,2009 Jens Raabe
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR 
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

function initImageShowTbl($api){

	$columns = array();
	$columns[0] = array('name' => 'isid', 'type' => 'int', 'null' => FALSE,	'auto_inc' => TRUE, 'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Image Show ID');
	$columns[1] = array('name' => 'cid', 'type' => 'int', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => TRUE, 'default' => NULL, 'comment' => 'Reference to Content (ID)');
	$columns[2] = array('name' => 'showType', 'type' => 'varchar(24)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => 'standard', 'comment' => 'Type of image show');
	$columns[3] = array('name' => 'captions', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Whether to show captions');
	$columns[4] = array('name' => 'controler', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Whether to show controller');
	$columns[5] = array('name' => 'thumbnails', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Whether to show thumbnails');
	$columns[6] = array('name' => 'random', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Random show, combined with thumbnails');
	$columns[7] = array('name' => 'titles', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '1', 'comment' => 'Whether to use captions for image and thumbnail title attributes');
	$columns[8] = array('name' => 'linked', 'type' => 'tinyint(1)', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Whether the class should automatically link each slide to the fullsize image (Lightbox)');
	$columns[9] = array('name' => 'delay', 'type' => 'int', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Whether the class should automatically link each slide to the fullsize image (Lightbox)');
	$columns[10] = array('name' => 'duration', 'type' => 'int', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'The duration of the effect in milliseconds');
	$columns[11] = array('name' => 'color', 'type' => 'varchar(150)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'colors used for flash mode');
	$columns[12] = array('name' => 'left', 'type' => 'varchar(10)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Image show frame position left');
	$columns[13] = array('name' => 'top', 'type' => 'varchar(10)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Image show frame position top');
	$columns[14] = array('name' => 'height', 'type' => 'varchar(10)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Optional height value for the show as a whole integer');
	$columns[15] = array('name' => 'width', 'type' => 'varchar(10)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Optional width value for the show as a whole integer');
	$columns[16] = array('name' => 'hu', 'type' => 'varchar(240)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Path to the image directory, relative or absolute');
	$columns[17] = array('name' => 'showSAction', 'type' => 'varchar(250)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'special action (jscript) clicking the slide show');

	return $api->createTable(sql_tbl_imgShowConf, $columns);
}

function initImageShowData($api){

	$columns = array();
	$columns[0] = array('name' => 'isdid', 'type' => 'int', 'null' => FALSE, 'auto_inc' => TRUE, 'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Image Data ID');
	$columns[1] = array('name' => 'isid', 'type' => 'int', 'null' => FALSE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Reference to image show (ID)');
	$columns[2] = array('name' => 'img', 'type' => 'varchar(250)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => 'image.jpg', 'comment' => 'image file name');
	$columns[3] = array('name' => 'caption', 'type' => 'varchar(250)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'caption of image');
	$columns[4] = array('name' => 'href', 'type' => 'varchar(250)', 'null' => TRUE, 'auto_inc' => FALSE, 'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'href of image');

	return $api->createTable(sql_tbl_imgShowData, $columns);
}

function initImgShowDb() {
	$api = new JRCMSAPI();
	
	$result &= initImageShowTbl($api);
	//print_r(' initImageShowTbl: '.$result.'<br>');
	$result &= initImageShowData($api);
	//print_r(' initImageShowData: '.$result.'<br>');
	
	$api = NULL;
	return (!isset($result) || $result === False)?False:True;
}

?>