<?php
/**
 * This module processes all tasks regarding User
 *
 * @package jrCMS: User
 * @author Jens Raabe <jens@raabe-berlin.de
 * @version $Id: jrcmsuser.php 594 2011-09-08 08:01:48Z jensraabe $ 
 * 
 * @copyright Copyright (c) 2008,2009, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */
 
// set some headers values
header("Last-Modified: " . gmdate( "D, d M Y H:i:s" ) . " GMT" ); 
header("Content-Type: text/plain; charset=utf-8");
header("Connection: close");
// set header
session_cache_limiter( 'nocache' ); //Expires, Cache-Control, Pragma
// start session
session_start(); 
// usage of classes
require_once('../../../conf/cConfig.php');
CONFIG::loadLanguage();
//language support of plugin
require_once(('lang/lang.'.$_SESSION["language"]));
$lang_php = array_merge($lang_user, $lang_php);
CONFIG::loadClasses();
CONFIG::loadScripts();
$userTaskList = array(
					0 => array(
						// Task: NEW
						"script" => Null,
						"id" => "task_new"),
					1 => array(
						// Task: EDIT
						"script" => Null,
						"id" => "task_edit"),
					2 => array(
						// Task: VIEW
						"script" => Null,
						"id" => "task_view"),
					3 => array(
						// Task: SAVE
						"script" => Null,
						"id" => "task_save"),
					4 => array(
						// Task: DELETE
						"script" => Null,
						"id" => "task_trash"),
					5 => array(
						// Separator
						"script" => '##',
						"id" => NULL),
					6 => array(
						// Task: RESET PASSWORD
						"script" => NULL,
						"id" => "task_chgpwd"),
					7 => array(
						// Task: COPY
						"script" => NULL,
						"id" => "task_copy"),
					8 => array(
						// Task: CLEAR
						"script" => NULL,
						"id" => "task_clear"),
					9 => array(
						// Separator
						"script" => '##',
						"id" => NULL),
					10 => array(
						// Task: Help
						"script" => NULL,
						"id" => "task_help")
					);

// retrieve parameter
if(isset($_GET['a'])) {$act = $_GET['a'];} else {if(isset($_POST['a'])) {$act = $_POST['a'];} else {$act=NULL;}}

// ------------------------------
// Utilities
// ------------------------------

	/**
	 * write a table line with two columns
	 * 
	 * @access private
	 * @param string	option
	 * @param int		parameter
	 */
	function tblRow2Col($opt, $param, $tblclass=NULL) {
		
		$str = '<tr><td'.(isset($tblclass)?' class='.$tblclass.'_left,'.$opt.'>':'>').$opt.'</td>';
		$str.= '<td'.(isset($tblclass)?' class='.$tblclass.'_right,'.$opt.'>':'>').$param.'</td></tr>';
		return $str;
		
	}

// ------------------------------
// Database routines
// ------------------------------


// ------------------------------
// Task routines
// ------------------------------

	/**
 	* retrieves all Users from DB and pack them to Grid 
 	* 
 	* @param global
 	* @access private
 	* @return HTML string
 	* @author Clemens Schultz
 	*/
	function getUsers($api){

		$ret = array();
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		if(isset($_GET['filter'])) {$filter = $_GET['filter'];} else {if(isset($_POST['filter'])) {$filter = $_POST['filter'];} else {$filter=NULL;}}
		$pagination = false;
		if ( isset($_REQUEST["page"]) ) {
			$pagination = true;
			$page = intval($_REQUEST["page"]);
			$perpage = intval($_REQUEST["perpage"]);
		}
		// this variables Omnigrid will send only if serverSort option is true
		//$sorton = $_REQUEST["sorton"];
		//$sortby = $_REQUEST["sortby"];
		
		$n = ( $page -1 ) * $perpage;
		$total = $api->getUserNumber($filter);
		$ret = $api->getUserList($n,$perpage,$filter);
		
		$ret = array("page"=>$page, "total"=>$total, "data"=>$ret);

		return $api->getContentAndButtons(json_encode($ret),
				/**
				 * xxx
				 */ 
				$api->convertTaskList('maint',
				//activate the task icon #8 of standard tasks with the a reload script
				array(0 => "jrCmsUser.createUser(); return false;"),
				(isset($_SESSION["taskType"])?($_SESSION["taskType"]):('icon'))), NULL);
	}
	
// ------------------------------

	/**
 	* edit user data, called by create as well 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function editUser($api, $act){
		
		global $lang_php;
		global $userTaskList;
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		if(isset($_GET['uid'])) {$uid = $_GET['uid'];} else {if(isset($_POST['uid'])) {$uid = $_POST['uid'];} else {$uid=NULL;}}
		if(isset($_GET['upref'])) {$upref = $_GET['upref'];} else {if(isset($_POST['upref'])) {$upref = $_POST['upref'];} else {$upref=False;}}
		if(isset($_GET['st'])) {$start = $_GET['st'];} else {if(isset($_POST['st'])) {$start = $_POST['st'];} else {$start=0;}}
		if(isset($_GET['li'])) {$limit = $_GET['li'];} else {if(isset($_POST['li'])) {$limit = $_POST['li'];} else {$limit=13;}}
		if(isset($_GET['dlgopt'])) {$dlgopt = $_GET['dlgopt'];} else {if(isset($_POST['dlgopt'])) {$dlgopt = $_POST['dlgopt'];} else {$dlgopt=NULL;}}
		
		try{
			if(!isset($uid)) {
				// ---
				// CREATE Task assumed no CTC necessary
				// ---
				$w_b0 = Null;	
				$w_b3 = "jrCmsUser.insertUser({a:'insertUser',start:".$start.",limit:".$limit."}); return false;";
				$w_b6 = NULL; 
				$w_b8 = NULL;
			} else if(isset($uid) && $api->ctcReserveTable(CONFIG::sql_tbl_user, $uid)===True) {
				//---
				// CTC: START OPTIMISTIC TRANSACTION - RESERVE
				//---
				$w_b3 = "jrCmsUser.saveUser({uid:".$uid.",a:'saveUser',start:".$start.",limit:".$limit."}); return false;";
				$w_b6 = "jrCmsUser.resetPW({uid:".$uid.",a:'resetPW',start:".$start.",limit:".$limit."}); return false;";
				$w_b8 = NULL;
			} else {
				// CTC reservation fails, no update/create possible
				$w_b3 = "\$lambda(); return false;";
				$w_b6 = "\$lambda(); return false;";
				$w_b8 = NULL;
				$userHeaderData = Array();	//provide an empty array
				$userDialogData = Array();	//provide an empty array
			}
			if($act != 'editUserNoGrid')
				$w_b8 = "jrCmsUser.getUserLst({start:".$start.",limit:".$limit."}); return false;";
			else {	
				$w_b3 = "jrCmsUser.saveUser({uid:".$uid.",a:'saveUser',nogrid:'1'}); return false;";
				$w_b6 = "jrCmsUser.resetPW({uid:".$uid.",a:'resetPW',nogrid:'1'}); return false;";
				$w_b8 = Null;
			}
			//2.) retrieve User Data and prepare right management
			$udata = (isset($uid)?$api->getUser($uid):array(0=>array('uid' => -1, 'UserName' => '', 'UserPass' => '', 'ul' => '000')));
			$owndata = $api->getUser(-1);
			//webmaster only
			$readonly = !$api->checkAccessRight(0,$ul,'create',CONFIG::webmasterLevel);
			//3.) data depending on rights
			if (isset($udata) && isset($owndata) && 
				($udata[0]['uid'] == $owndata[0]['uid']) || 
				 $api->checkAccessRight(0,$ul,'create',CONFIG::webmasterLevel)) {
				
				// Build up user header data for dialog
				$userHeaderData = $api->dialogHeader('uid', $uid, 'todo');
				//5.) Build array of dialog data 	
				foreach($udata[0] as $key => $value){
					if (isset($value)) {
						switch($key) {
						case "uid":
							break;	//provided as header
						case "UserPass":
							$userDialogData[] = $api->dialogElement($key, $lang_php['u_pw'], 'userDialog', 'input:jrCmsUser', 'required');
							$userDialogData[] = $api->dialogElement($key.'old', ($udata[0]['uid']!=-1?$lang_php['u_pw']:''), 'userDialog', 'hidden');
							break;
						case "UserName":
							$userDialogData[] = $api->dialogElement($key, $value, 'userDialog', 'input', ($readonly?'disabled':'required'));
							break;
						case "ul":
							$userDialogData[] = $api->dialogElement($key, $value, 'userDialog', 'select:jrCmsUser', ($readonly?'disabled':'required'));
								break;
							default:	
								break;	//internal only	
						}
					}
				}
				// User Profile data will be attached 
				// must be configured as dialog, no tests
				if($upref && isset($udata[1])) {
					$i=1;
					while(isset($udata[$i]))
						$userDialogData[] = $udata[$i++];
				
				} 
			} else {
				// no access right
				$w_b0 = "\$lambda(); return false;";
				$w_b2 = "\$lambda(); return false;";
				$userHeaderData = Array();	//provide an empty array
				$userDialogData = Array();	//provide an empty array
			}
			$dataset = array('header' => $userHeaderData,
							 'data' => $userDialogData,
							 'dlgoptions' => $dlgopt);
			
			return $api->getContentAndButtons( '',
								$api->convertOwnTaskList($userTaskList
									,array(0 => NULL
										  ,3 => $w_b3
										  ,6 => $w_b6
										  ,8 => $w_b8
										  ,10 => "jrCmsUser.getHelpDoc();return false;"
									),
									(isset($_SESSION["taskType"])?($_SESSION["taskType"]):('icon')))
								,NULL
								,$dataset);
		
		} catch (MyException $e) {
			
			//$e->log(True);
			return $api->getContentAndButtons('Please try again!', NULL);
		}
								
	}

	// ------------------------------

	/**
 	* create user data 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function createUser($api){
	
		global $act;
		global $lang_php;

		//no CTC is needed
		return editUser($api, 'editUser');
	}

	
// ------------------------------

	/**
 	* save user data 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function saveUser($api){
	
		global $lang_php;
		$ul = (isset($_SESSION["ul"]))?($_SESSION["ul"]):('000');
		
		if(isset($_GET['uid'])) {$uid = $_GET['uid'];} else {if(isset($_POST['uid'])) {$uid = $_POST['uid'];} else {$uid=NULL;}}
		if(isset($_GET['UserName'])) {$un = ($_GET['UserName']);} else {if(isset($_POST['UserName'])) {$un = ($_POST['UserName']);} else {$un=NULL;}}
		if(isset($_GET['UserPass'])) {$up = ($_GET['UserPass']);} else {if(isset($_POST['UserPass'])) {$up = ($_POST['UserPass']);} else {$up=NULL;}}
		if(isset($_GET['ul'])) {$userlevel = $_GET['ul'];} else {if(isset($_POST['ul'])) {$userlevel = $_POST['ul'];} else {$userlevel=NULL;}}
		if(isset($_GET['upref'])) {$upref = $_GET['upref'];} else {if(isset($_POST['upref'])) {$upref = $_POST['upref'];} else {$upref=False;}}
		
		// Transaction started by Edit Task
		// CTC: CHECK OPTIMISTIC TRANSACTION - LOCK

		if($api->ctcCheckAndLockTable(CONFIG::sql_tbl_user, $uid)) {


			// save User Profile 
			if($upref) {
				; //update profile with preferences
					
			}
					
			if(isset($un)||isset($up)||isset($userlevel))
				// save  user data
				$result = $api->updateUser($uid, $un, $up, $userlevel);
			else
				// simulate at least an successfull task
				$result = TRUE;
		}
					
		if (isset($result) && $result === TRUE )
			$return = $lang_php['us_saved'];			
		else
			$return = $lang_php['us_not_saved'].(isset($_SESSION["ctc"])?' \n(CTC: '.$_SESSION["ctc"].')':'');

			
		// CTC: CLOSE OPTIMISTIC TRANSACTION - UNLOCK
		$api->ctcUnlockTable(CONFIG::sql_tbl_user, $uid);

		return '<script type="text/javascript">alert("'.$return.'");</script>';
	}

// ------------------------------

	/**
 	* reset password 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function resetPW($api){
	
		global $lang_php;
		
		if (isset($_GET['uid'])) $uid = $_GET['uid']; else if (isset($_POST['uid'])) $uid = $_POST['uid']; else $uid = NULL;
		
		// Transaction started by Edit Task
		// CTC: CHECK OPTIMISTIC TRANSACTION - LOCK
		if($api->ctcCheckAndLockTable(CONFIG::sql_tbl_user, $uid)) {

			// update password
			$result = $api->updateUser($uid, NULL, '123', NULL);

		}
		
		if (isset($result) && $result === TRUE )
			$return = $lang_php['us_pw_saved'];			
		else
			$return = $lang_php['us_pw_not_saved'].(isset($_SESSION["ctc"])?(' \n(CTC: '.$_SESSION["ctc"].')'):'');

		// CTC: CLOSE OPTIMISTIC TRANSACTION - UNLOCK
		$api->ctcUnlockTable(CONFIG::sql_tbl_user, $uid);
		
		return '<script type="text/javascript">alert("'.$return.'");</script>';
	}

// ------------------------------

	/**
 	* insert user data 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function insertUser($api){
	
		global $lang_php;
		
		if(isset($_GET['uid'])) {$uid = $_GET['uid'];} else {if(isset($_POST['uid'])) {$uid = $_POST['uid'];} else {$uid=NULL;}}
		if(isset($_GET['UserName'])) {$un = ($_GET['UserName']);} else {if(isset($_POST['UserName'])) {$un = ($_POST['UserName']);} else {$un=NULL;}}
		if(isset($_GET['UserPass'])) {$up = ($_GET['UserPass']);} else {if(isset($_POST['UserPass'])) {$up = ($_POST['UserPass']);} else {$up=NULL;}}
		if(isset($_GET['ul'])) {$userlevel = $_GET['ul'];} else {if(isset($_POST['ul'])) {$userlevel = $_POST['ul'];} else {$userlevel='000';}}
		
		//no CTC is needed
		$result = $api->insertUser($un, $up, $userlevel);
		
		if ($result === TRUE )
			$return = $lang_php['us_created'];			
		else
			$return = $lang_php['us_not_created'];

		return '<script type="text/javascript">alert("'.$return.'");</script>';
	}

// ------------------------------

	/**
 	* delete user data 
 	* 
 	* @param global
 	* @access global
 	* @return Html string
 	* @author Jens Raabe <jens@raabe-berlin.de>
 	*/
	function deleteUser($api){
	
		global $lang_php;
		
		if(isset($_GET['uid'])) {$uid = $_GET['uid'];} else {if(isset($_POST['uid'])) {$uid = $_POST['uid'];} else {$uid=NULL;}}

		// Single Transaction, not started before
		// CTC: PESSIMISTIC TRANSACTION - LOCK
		if($api->ctcLockTable(CONFIG::sql_tbl_user, $uid)) {

			// update password
			$result = $api->deleteUser($uid);

			// CTC: CLOSE PESSIMISTIC TRANSACTION - UNLOCK
			$api->ctcUnlockTable(CONFIG::sql_tbl_user, $uid);
		
		}
		
		if (isset($result) && $result === TRUE )
			$return = $lang_php['us_deleted'];			
		else
			$return = $lang_php['us_not_deleted'];

		return '<script type="text/javascript">alert("'.$return.'");</script>';
	}

	/**
 	* retrieve all available user level 
 	* 
 	* @param global
 	* @access global
 	* @return Json string
 	*/
	function getUserLevelList($api){
		
		$dataset = array();
		if (($ultyp = $api->getUserlevelList()) !== FALSE) {
			foreach($ultyp as &$value) {
				$dataset[] = array('idx'=>$value['ul'],'value'=>$value['description']);
			}
		}
		return $api->array2Xml($dataset);				
	}
// ############## MAIN ##############
$api = new JRCMSAPI();

if (isset($act)) {
	switch ($act) {
	case "getUserLst":
		$rsp = $api->getMessageContainer(getUsers($api), $api->getMenu('xml'), $api->getStyles());
		break;
	case "editUser":
	case "editUserNoGrid":
		$rsp = $api->getMessageContainer(editUser($api, $act));
		break;
	case "createUser":
		$rsp = $api->getMessageContainer(createUser($api));
		break;
	case "saveUser":
		$rsp = saveUser($api);
		break;
	case "resetPW":
		$rsp = resetPW($api);
		break;
	case "insertUser":
		$rsp = insertUser($api);
		break;
	case "deleteUser":
		$rsp = deleteUser($api);
		break;
	case "getUserLevelList":
		$rsp = getUserLevelList($api);
		break;
	default: 
		$rsp = "jrcmsuser.php with wrong action called!!";
	}

} else {

	$rsp = "jrcmsuser.php with wrong action called!!";
		
}
echo $rsp;
$api = NULL;
flush(); // send really out all stuff
?>