<?php
/**
 * This Class extend the user maintenance by an application specific user profile.
 * 
 * Following the conventions the class extend the definition of the USER class 
 * as used by the core for predefined methods. 
 * 
 * Prerequisite for usage are configuration values to be provided
 * in the generic data table of the core:
 * 
 * 	getUserProf				Class name of the application specific 
 *							user profile interface,
 *							e.g. 'USERPROF'
 * 	getUserProfSrc			filename containing the user profile class
 *							including the relative path from the root
 *							directory e.g. '/src/plugins/jrcmsuserprof/cUserProf.php'
 * 
 * Be carefully in using this settings. There are no verifications
 * inside of the core. If methods as defined below are not available the
 * core functions will be used instead.
 * 
 * Reserved Methods are:
 * 
 *  upCheckAccessRight			To be used for specific access rights providing 
 *								the third column of the access right string.
 *  
 *  upGetUser					Replaces the get_core_user where additional data
 *								could be provided for a user or user list
 *  
 *  upLoginWSessionParameter 	Called instead of the USER:login, which internally
 *  							processes the USER:login but give the possibilty
 *  							to set user depended session variables
 *  							Session variables to be used are documented in the
 *  							JRCMSAPI class
 *  
 * @package jrCmsUserProf: Class USERPROF
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @version $Id: cUserProf.php 598 2011-09-12 05:55:32Z jensraabe $ 
 * 
 * @copyright Copyright (c) 2011, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */
 
require_once('iUserProf.php');
class USERPROF extends USER implements iUserProf{
	
	CONST VERSION			= '$Revision: 598 $';
	CONST AUTHOR			= 'Jens Raabe';
	CONST PACKAGE			= 'jrCmsUserProf';
	CONST LAST_CH			= '$Date: 2011-09-12 07:55:32 +0200 (Mo, 12 Sep 2011) $';
	
	// no own __construct/__destruct is used
	
// ------------------------------
	/**#@+
	 * method which could be used to provide the access level (right)
	 * for the third part of the access right string. The method is
	 * called inside of the JRCMAPI:checkAccessRight.
	 * 
	 * According to the given ACL selector the access level digit
	 * (0..9) must be returned as character.
	 * 
	 * @access	public
	 * @param 	string	access control level selector
	 * @return 	character
	 */
	public function upCheckAccessRight($aclSel){
		//return max right level as dummy value
		return '9';
	}
	
// ------------------------------
	/**#@+
	 * intended to replace the core function by an own list of data to be
	 * retrieved for each user, e.g. with additional columns/properties
	 * which are defined in a separate table.
	 * The coding below is an example but applications should follow the
	 * inline documentation/rules carefully. There are no verifications in the
	 * core to speed up the performance.
	 * 
	 * @access	public
	 * @param	integer	user Id
	 * @param	string	user level only flag
	 * @param	integer	start value of user list
	 * @param	integer	number of users
	 * @param	string	filter conditions
	 * @return 	mixed Boolean, string (array), value (ul)
	 */
	public function upGetUser($uid = -1, $ul_only = NULL, $start = NULL, $limit = 1, $filter=NULL){
		
		//== do not change this part ==
		if(isset($ul_only))
			return $this->get_core_user($uid, $ul_only, $start, $limit, $filter);
		else { 
			$userdata = $this->get_core_user($uid, $ul_only, $start, $limit, $filter);
			
		//=====================================================
		//== own functionality to enhance the retrieved data
		//=====================================================
			if(!isset($start)) {
				// a single user is requested where the data could be
				// enhanced by an own functionality
				// --------------------------------
				// conventions
				// --------------------------------
				// - first array of the userdata is used for the core
				//   data itself
				// - following arrays could be used for data provided
				//   as dialog structure 
				// - identifier used in dialogs must be defined in the 
				//   project language file /lang/lang.<nn>
				// --------------------------------
				// maintenance of user profile data
				// --------------------------------
				// By using the functionality of the dialog as provided
				// by the core (jrcms.js) different server files could 
				// be addressed by the same dialog. This function has to
				// be used for the maintenance. 
				// Please have a look at jrcms.js class:JRCmsForm 

				$api = new JRCMSAPI();	
				$userdata[] = $api->dialogElement('User_Name', 'Sandman ;-)', 'userProfDialog:jrCmsUserProf', 'input:jrCmsUserProf'); 
				;
			} else {
				// a user list is requested where the data could be
				// enhanced by own functionality
				// this could be done either by an own DB request
				// or by a postprocessing of the retrieved userdata
				// ---
				// assuming that the user maintenance isn't runtime
				// critical the postprocessing is prefered
				function alterUser(&$item, $key){
					$item['Name'] = 'Sandman';
				}
				array_walk($userdata, 'alterUser');
			}
		//=====================================================
		//== end of own functionality
		//=====================================================
			return $userdata;
		}	
	}

// ------------------------------
	/**#@+
	 * method which could be used to set predefined session variables
	 * depending on the user level or user profile.
	 * This function is called instead of the USER:login method if
	 * available.
	 * 
	 * @access	public
	 * @param 	none
	 * @return 	character
	 */
	public function upLoginWSessionParameter($name, $pass){
		
		//perform the login
		if(($result = parent::login($name, $pass)) === True){
			// based on the user data
			// or profile data 
			// the following session variables could be set
			// --Xinha------------------------
			//$_SESSION['allow_new_dir'] = true;
			//$_SESSION['allow_delete'] = true;
			//$_SESSION['images_subdir'] = '/exampleimages';
			// -------------------------------
		}
		return $result;
	}
}	
?>