/*!
 * jrCmsUserProf
 *
 * This module enhance the basic functionality for user maintenance
 *
 * @package jrCMS
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @version $Id: jrcmsuserprof.js 595 2011-09-09 06:35:39Z jensraabe $
 *  
 * @copyright Copyright (c) 2008,2009, Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

var JRCmsUserProf = new Class({
	
	_pluginInfo: {
		name:	 		"JRCmsUserProf",
		version:	 	"0.3.0",					//Release.Version.Minor
		revision:		"$Revision: 595 $",			//Revision set by cvs
		developer:		"Jens Raabe",
		developer_url:	"http://www.raabe-berlin.de",
		license: 		"MIT",
		classId:		2600
	}  
});

(function(){
	JRCmsUserProf.implement({

		Implements: [Options, JRCmsApi]
		         	
   		,options: {
			jr_userProfLevel: 	'110'	//should have the same level as defined
										//for the class in cConfig.js
		}
	
		/*Public*/
		,getClassId: function() {
			return this._pluginInfo.classId;
		}
	
		/*Public*/
		,getVersion: function() {
			return (this._pluginInfo.version+'_'+
					this._pluginInfo.revision.slice(this._pluginInfo.revision.search(/:.+/)+1)).replace(/\$/g,'').replace(/ /g,'');
		}
		             
		/**
   		 * Constructor: initialize
   		 * Creates an instance of the JRCmsUserProf class.
   		 *
   		 * @param array options
   		 * @access public
   		 */
   		,initialize: function(options, instanceName){
		     			
			// retrieve plugin styles
			this.loadStyleFiles({'cssfile':'JRCmsUserProf.css'
								,'path': options.jr_pluginsUrl+'jrcmsuserprof/css/'});	
			
			// for next steps call the refresh method
			this.refresh(options);
		
//			// register methods external usable (ajax history pool)
//			this.registerFunctions({instance: ($defined(instanceName)?instanceName:'jrCmsUserProf'),
//				classId:this.getClassId(),
//				functions:[{functionName:'getUserProf', methodId: 1, functionType: 'ajax' }
//							,{functionName:'editUserProf', methodId: 2, functionType: '' }
//				           ]});
   		}

		/**
		 * Refresh constructor
		 * Updates the instance in case of reloading the plugin
		 *
		 * @param array options
		 * @access public
		 */
		,refresh: function(options) {
			
			// set options
			this.setOptions(options);
			this.setOptions({jr_phpSRC: 'jrcmsuserprof/jrcmsuserprof.php'});
			
			// language support, only needed if userlevel fits
			if(this.writeRights(this.getUserLevel(), this.options.jr_userProfLevel))
				this.retrieveLangFile(this.options.jr_pluginsUrl+'jrcmsuserprof/lang/', 'lang.' + this.options.jr_lang,
						function(){this.setOptions({jr_uLangLoaded: true});}.bind(this));
			
//			// refactor the class in case of maintenance
//			if(this.writeRights(this.getUserLevel(), this.options.jr_editorLevel)) {
//				this.loadScriptFiles({'jsfile':'JRCmsUserProf-M' +
//					   				           (($defined(this.options.cmp) && !this.options.cmp)?(".js"):(".cmp.js"))
//					   				 ,'path': this.options.jr_pluginsUrl + 'jrcmsuserprof/'
//					   				 ,'loadfunc': function(){
//											// if needed process a post load function
//											; 
//									 }.bind(this)
//				});			
//			} else
				// refactor the user class only
				this.refactorUserMaint();
			
		}

		/**
		 * Try to redefined the generic functions
		 * If that works we could use it as solution for a more generic
		 * approach by refactoring the jrCMS. 
		 */
		,refactorUserMaint: function(){
			try{
				// set switch for user profile
				jrCmsUser.setOptions({jr_userProfile: true});
				
				// reconfigure the user grid list
				//omnigrid options
				jrCmsUser.setOptions({columnModel: [
				    {
			       		header: "id"
			       		,dataIndex: 'uid'
			       		,dataType:'number'
			       		,width: 30
		       		}
					,{
						header: "User Account"
						,dataIndex: 'UserName'
						,dataType:'string'
						,width: 100
					}
					,{
						header: "User Name"
						,dataIndex: 'Name'
						,dataType:'string'
						,width: 220
						}
					,{
						header: "User Level"
						,dataIndex: 'description'
						,dataType:'string'
						,width: 270
					}]
				});
					
			} catch (err) {
				//try it again
				this.jrLog("T:3#>> jrCmsUserProf.refactorUserMaint, try again after 200ms",3);
				this.refactorUserMaint.delay(200, this);
			}
		}
		
		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
		//Dialog
		/**
		 * 
		 */
		,processDialogValue: function(el, valueName, value){
			switch (valueName) {
			case 'User_Name':
				// === example for special processing ===
				el.value = el.value + ' (mod. by jrCmsUserProf:processDialogValue)';
				break;
			default:
				break;
			}
			return el;
		}
	});
})();

var jrCmsUserProf = new JRCmsUserProf(window.jrCmsInit.options);