<?php
/**
 * This module initializes the database used by the jrCmsUser Profiles
 *
 * @package jrCMS: jrCmsUSerProf
 * @author Jens Raabe
 *  
 * @copyright Copyright (c) 2010 Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

/**
 * Initial definition of value types (dbTyp)
 * !! do not change !!
 * 
* @return array valueTypes
 */
function getValueTypes(){
	return array(0 => array("valueTyp" => "int")
				  ,1 => array("valueTyp" => "text")
				  ,2 => array("valueTyp" => "varchar(3)")
				  ,3 => array("valueTyp" => "varchar(120)")
				  ,4 => array("valueTyp" => "varchar(250)")
				  ,5 => array("valueTyp" => "datetime"));
}

/**
 * Init a function for history
 * 
 * @param object $api
 * @return boolean
 */
function initProcedure($api){
	//TODO:as first try we use fix values
	//return $api->insertProcedure('jrCmsUserProf.getUserProf',2601,'ajax','');
	return TRUE;
}

/**
 * A user could have several entries, each for the defined profile
 * settings.  
 * 
 * 	  uprofid		Profile ID
 *    uid			User ID
 *    valueNameId	Value Name ID
 * 
 * @param object $api
 * @return boolean
 */
function initUserProfileTbl($api){

	$table = sql_tbl_userprof;
	if($api->checkTable($table) === FALSE) {
		$columns = array();
		$columns[0] = array('name' => 'uprofid', 'type' => 'int', 'null' => FALSE, 'auto_inc' => TRUE,
							'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Profile ID');
		$columns[1] = array('name' => 'uid', 'type' => 'int', 'null' => TRUE,	'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'user ID');
		$columns[2] = array('name' => 'valueNameId', 'type' => 'int', 'null' => TRUE,	'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value Name ID');
		
		if (($result = $api->createTable($table, $columns)) !== FALSE) {
			// write initial data
			$result &= $api->insertSingleValueInDb($table, "uid, valueNameId","4, 1", FALSE);
			return $result;
			
		} else 
			return FALSE; // createTable() failed
					
	} else 
		return TRUE; // table already exist
}

/**
 * A valueTyp consist of:
 * 
 *    valueTypId	Value Typ ID
 *    dbTyp			Database typ
 *    accessProc	access function if needed (for enhanced functionality)
 * 
 * @param object $api
 * @return boolean
 */
function initUserProfValueTypeTbl($api){
	
	$valueTyps = getValueTypes();
	$table = sql_tbl_userprofvaluetyp;
	if($api->checkTable($table) === FALSE) {
	
		$columns = array();
		$columns[0] = array('name' => 'valueTypId', 'type' => 'int', 'null' => FALSE, 'auto_inc' => TRUE,
							'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value Typ ID');
		$columns[1] = array('name' => 'dbTyp', 'type' => 'varchar(120)', 'null' => FALSE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => 'none', 'comment' => 'Database typ');
		$columns[2] = array('name' => 'accessProc', 'type' => 'varchar(240)', 'null' => FALSE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => 'none', 'comment' => 'access function');
		
		if (($result = $api->createTable($table, $columns)) !== FALSE) {
			// write initial data
			foreach($valueTyps as &$value) {
				$result &= $api->insertSingleValueInDb($table, "dbTyp, accessProc","'".$value['valueTyp']."', 'none'", FALSE);
			} 
			return $result;
		} else 
			return FALSE; // createTable() failed
					
	} else 
		return TRUE; // table already exist
}

/**
 *
 * @param object $api
 * @param string $valueName
 * @param string $valueTyp
 * @return boolean
 */
function initUserProfValueTbl($api, $valueName, $valueTyp){

	$table = str_replace('(','_',sql_tbl_userprof.'_'.$valueName.'_'.$valueTyp);
	$table = str_replace(')','',$table);
	if($api->checkTable($table) === FALSE) {
		$columns = array();
		$columns[0] = array('name' => 'valueid', 'type' => 'int', 'null' => FALSE, 'auto_inc' => TRUE,
							'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value ID');
		$columns[1] = array('name' => 'value', 'type' => $valueTyp, 'null' => FALSE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => 'none', 'comment' => 'Value');
		
		if (($result = $api->createTable($table, $columns)) !== FALSE) {
			
			return $api->insertSingleValueInDb($table, "value"
												,($valueType != 'int')?("'".$valueName."'"):$valueName
												, TRUE);
		}
	}	
	return 0;
}

/**
 * A value name consist of
 * 
 *    valueNameId   Value Name ID
 *    valueName		Value Name
 *    valueAccRight Access Right of Value
 *    valueTypId	Value Typ ID
 * 	  valueid		Value ID
 * 
 * For each valueName/valueTyp a own table will be build
 * 
 * @param object $api
 * @return boolean
 */
function initUserProfValueNameTbl($api){

	$table = sql_tbl_userprofvaluename;
	if($api->checkTable($table) === FALSE) {
	
		$columns = array();
		$columns[0] = array('name' => 'valueNameId', 'type' => 'int', 'null' => FALSE, 'auto_inc' => TRUE,
							'key' => TRUE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value Name ID');
		$columns[1] = array('name' => 'valueName', 'type' => 'varchar(30)', 'null' => FALSE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value Name');
		$columns[2] = array('name' => 'valueAccRight', 'type' => 'varchar(3)', 'null' => FALSE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => '0', 'comment' => 'Access Right of Value');
		$columns[3] = array('name' => 'valueTypId', 'type' => 'int', 'null' => TRUE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value Type ID');
		$columns[4] = array('name' => 'valueid', 'type' => 'int', 'null' => TRUE, 'auto_inc' => FALSE,
							'key' => FALSE, 'unique' => FALSE, 'default' => NULL, 'comment' => 'Value ID');
		
		if (($result = $api->createTable($table, $columns)) !== FALSE) {
			
			//1) create the value table of a valueName/valueType pair
			if(($vid = initUserProfValueTbl($api, 'Name', 'varchar(120)')) > 0 ) 
				$result &= $api->insertSingleValueInDb($table, "valueName, valueAccRight, valueTypId, valueid"
														,"'Name', '999', 4,".$vid, FALSE);
			
		} else 
			return FALSE; // createTable() failed
					
	} else 
		return TRUE; // table already exist
}


/**
 * 
 * @return boolean
 */
function initUserProfileDb() {
	$api = new JRCMSAPI();
	
	$result = initProcedure($api);
	//print_r(' initProcedure: '.$result.'<br>');
	$result &= initUserProfileTbl($api);
	//print_r(' initUserProfileTbl: '.$result.'<br>');
	$result &= initUserProfValueTypeTbl($api);
	//print_r(' initUserProfValueTypeTbl: '.$result.'<br>');
	$result &= initUserProfValueNameTbl($api);
	//print_r(' initUserProfValueNameTbl: '.$result.'<br>');
	
	$api = NULL;
	return (!isset($result) || $result === False)?False:True;
}

?>